<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Api;

use RuntimeException;
use Shopware\Core\Framework\Uuid\Uuid;
use Saferpay\Shopware\Api\Exception\ApiRequestException;
use Saferpay\Shopware\SaferpayShopware;
use Saferpay\Shopware\Service\LoggerService;
use Shopware\Core\Checkout\Payment\Exception\InvalidTokenException;
use Shopware\Core\Checkout\Payment\Exception\InvalidTransactionException;

class MissingRedirectUrlException extends \RuntimeException
{
}
class MissingTokenException extends \RuntimeException
{
}
class MissingTransactionException extends \RuntimeException
{
}
class MissingTransactionStatusException extends \RuntimeException
{
}
class MissingTransactionOrderIdException extends \RuntimeException
{
}
class MissingLiabilityAndLiabilityShiftException extends \RuntimeException
{
}
class MissingCaptureIdException extends \RuntimeException
{
}
class MissingCaptureStatusException extends \RuntimeException
{
}
class MissingAssertStatusException extends \RuntimeException
{
}
class FailedToParseSaferpayApiResponseException extends \RuntimeException
{
}
class MissingRequestIdException extends \RuntimeException
{
}

class SaferpayPaymentApiClient
{
    // Payment Page Endpoints.
    const ENDPOINT_PAYMENT_PAGE_INITIALIZE = '/Payment/v1/PaymentPage/Initialize';
    const ENDPOINT_PAYMENT_PAGE_ASSERT = '/Payment/v1/PaymentPage/Assert';
    const ENDPOINT_TRANSACTION_CAPTURE = '/Payment/v1/Transaction/Capture';
    const ENDPOINT_TRANSACTION_CANCEL = '/Payment/v1/Transaction/Cancel';
    const ENDPOINT_TRANSACTION_ASSERT_CAPTURE = '/Payment/v1/Transaction/AssertCapture';
    const ENDPOINT_REFUND = '/Payment/v1/Transaction/Refund';
    const ENDPOINT_REFUND_ASSERT = '/Payment/v1/Transaction/AssertRefund';

    //Transaction Via Card Endpoints.
    const ENDPOINT_TRANSACTION_INITIALIZE = '/Payment/v1/Transaction/Initialize';
    const ENDPOINT_TRANSACTION_AUTHORIZE = '/Payment/v1/Transaction/Authorize';
    const ENDPOINT_ACCESS_TOKEN = '/rest/customers/[customerId]/terminals/[terminalId]/fields-access-tokens';

    //Save card endpoints.
    const ENDPOINT_TRANSACTION_AUTHORIZE_DIRECT = '/Payment/v1/Transaction/AuthorizeDirect';
    const ENDPOINT_ALIAS_DELETE = '/Payment/v1/Alias/Delete';
    const ENDPOINT_ALIAS_UPDATE = '/Payment/v1/Alias/Update';
    const ENDPOINT_ALIAS_INSERT = '/Payment/v1/Alias/Insert';
    const ENDPOINT_ALIAS_ASSERT_INSERT = '/Payment/v1/Alias/AssertInsert';

    private string $customerId;
    private string $terminalId;
    private string $username;
    private string $password;
    private string $shopInfo;
    private string $apiBaseUrl;
    private string $apiVersion;

    /**
     * Initialize variables of the class
     */
    public function __construct(
        string $customerId,
        string $terminalId,
        string $username,
        string $password,
        string $shopInfo,
        string $apiBaseUrl,
        string $apiVersion
    ) {
        $this->customerId = $customerId;
        $this->terminalId = $terminalId;
        $this->username = $username;
        $this->password = $password;
        $this->shopInfo = $shopInfo;
        $this->apiBaseUrl = $apiBaseUrl;
        $this->apiVersion = $apiVersion;
    }

    /**
     * 1. Populates the contents for deliveryAddress array
     * 2. Populates array of payload
     * 3. Sends request to initialize payment with payload
     * 4. If no redirectUrl or Token in response, RuntimeException is thrown
     *
     * @throws RunTimeException
     * @param array $payloadData
     * @return object
     */
    public function initializePaymentPage(array $payloadData): object
    {
        $payload = [
            'TerminalId' => $this->terminalId,
            'PaymentMethods' => $payloadData['paymentMethods'],
            'Wallets' => $payloadData['wallets'],
            'Payment' => [
                'Amount' => [
                    'Value' => $this->amountToMinorUnit($payloadData['totalAmount']),
                    'CurrencyCode' => $payloadData['currencyCode']
                ],
                'OrderId' => $payloadData['orderNumber'],
                'Description' => $payloadData['description']
            ],
            'BillingAddressForm' => [
                'AddressSource' => 'NONE'
            ],
            'DeliveryAddressForm' => [
                'AddressSource' => 'NONE'
            ],
            'ReturnUrl' => [
                'Url' => $payloadData['returnUrl']
            ],
            'Notification' => [
                'SuccessNotifyUrl' => $payloadData['successNotifyUrl'],
                'FailNotifyUrl' => $payloadData['failNotifyUrl']
            ],
            'Styling' => [
                'Theme' => $payloadData['theme']
            ],
            'CardForm' => [
                'HolderName' => $payloadData['HolderName']
            ]
        ];
        if (!empty($payloadData['merchantEmails'])) {
            $payload['Notification']['MerchantEmails'] = $payloadData['merchantEmails'];
        }
        if ($payloadData['customerConfirmationEmailSettings'] !== '0') {
            $payload['Notification']['PayerEmail'] = $payloadData['customerEmail'];
        }
        if ($payloadData['customerAddressSettings']  === 'both') {
            $billingAddress = $payloadData['billingAddress'];
            $billingAddress['Id'] = $payloadData['customerId'];
            $payload['Payer']['BillingAddress'] = $billingAddress;
            $deliveryAddress = $payloadData['deliveryAddress'];
            $deliveryAddress['Email'] = $payloadData['customerEmail'];
            $payload['Payer']['DeliveryAddress'] = $deliveryAddress;
        } elseif ($payloadData['customerAddressSettings'] === 'delivery') {
            $deliveryAddress = $payloadData['deliveryAddress'];
            $deliveryAddress['Email'] = $payloadData['customerEmail'];
            $payload['Payer']['DeliveryAddress'] = $deliveryAddress;
            $payload['Payer']['BillingAddress']['Id'] = $payloadData['customerId'];
        } elseif ($payloadData['customerAddressSettings'] === 'billing') {
            $billingAddress = $payloadData['billingAddress'];
            $billingAddress['Id'] = $payloadData['customerId'];
            $payload['Payer']['BillingAddress'] = $billingAddress;
        } else {
            $payload['Payer']['BillingAddress']['Id'] = $payloadData['customerId'];
        }

        if ($payloadData['currentPaymentMethodId'] === SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY_ID) {
            $payload['Payer']['DeliveryAddress']['Email'] = $payloadData['customerEmail'];
        }

        if ($payloadData['preAuthSetting'] === '1') {
            $payload['Payment']['Options']['PreAuth'] = true;
        }

        if (
            $payloadData['customerSelectedIdealBank'] &&
            $payloadData['currentPaymentMethodId'] === SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL_ID
        ) {
            $payload['PaymentMethodsOptions']['Ideal']['IssuerId'] = $payloadData['customerSelectedIdealBank'];
        }

        if ($payloadData['currentPaymentMethodId'] === SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID) {
            $payload = array_merge($payload, $payloadData['orderData']);
            $payload['Payer']['BillingAddress'] = $payloadData['billingAddress'];
            $payload['Payer']['BillingAddress']['Id'] = $payloadData['customerId'];
        }

        if ($payloadData['configSet']) {
            $payload['ConfigSet'] = $payloadData['configSet'];
        }

        if (isset($payloadData['isSaveCard'])) {
            $payload['RegisterAlias'] = [
                'IdGenerator' => 'RANDOM_UNIQUE'
            ];
        }

        if (isset($payloadData['forceAuthentication']) && $payloadData['forceAuthentication'] === 'yes') {
            $payload['Authentication']['ThreeDsChallenge'] = SaferpayShopware::THREEDS_FORCE;
        }

        $response = $this->sendRequest(static::ENDPOINT_PAYMENT_PAGE_INITIALIZE, $payload);

        if (
            !isset($response->RedirectUrl)
            || !is_string($response->RedirectUrl)
            || trim($response->RedirectUrl) === ''
        ) {
            throw new MissingRedirectUrlException(
                'Initialize payment page response has no valid property "RedirectUrl"',
                1656597985
            );
        }

        if (!isset($response->Token) || !is_string($response->Token) || trim($response->Token) === '') {
            throw new InvalidTokenException(
                'Initialize payment page response has no valid property "Token"',
            );
        }

        return $response;
    }

    /**
     * Send initialize transaction api call
     */
    public function transactionInitialize(array $payloadData): object
    {
        $payload = array();
        $payload = [
            'TerminalId' => $this->terminalId,
            'PaymentMethods' => $payloadData['paymentMethods'],
            'Payment' => [
                'Amount' => [
                    'Value' => $this->amountToMinorUnit($payloadData['totalAmount']),
                    'CurrencyCode' => $payloadData['currencyCode']
                ],
                'OrderId' => $payloadData['orderNumber'],
                'Description' => $payloadData['description']
            ],
            'BillingAddressForm' => [
                'AddressSource' => 'NONE'
            ],
            'DeliveryAddressForm' => [
                'AddressSource' => 'NONE'
            ],
            'ReturnUrl' => [
                'Url' => $payloadData['returnUrl']
            ],
            'RedirectNotifyUrls' => [
                'Success' => $payloadData['successNotifyUrl'],
                'Fail' => $payloadData['failNotifyUrl']
            ],
            'Styling' => [
                'Theme' => $payloadData['theme']
            ],
            'CardForm' => [
                'HolderName' => $payloadData['HolderName']
            ]
        ];
        if ($payloadData['customerAddressSettings']  === 'both') {
            $billingAddress = $payloadData['billingAddress'];
            $billingAddress['Id'] = $payloadData['customerId'];
            $payload['Payer']['BillingAddress'] = $billingAddress;
            $deliveryAddress = $payloadData['deliveryAddress'];
            $deliveryAddress['Email'] = $payloadData['customerEmail'];
            $payload['Payer']['DeliveryAddress'] = $deliveryAddress;
        } elseif ($payloadData['customerAddressSettings'] === 'delivery') {
            $deliveryAddress = $payloadData['deliveryAddress'];
            $deliveryAddress['Email'] = $payloadData['customerEmail'];
            $payload['Payer']['DeliveryAddress'] = $deliveryAddress;
            $payload['Payer']['BillingAddress']['Id'] = $payloadData['customerId'];
        } elseif ($payloadData['customerAddressSettings'] === 'billing') {
            $billingAddress = $payloadData['billingAddress'];
            $billingAddress['Id'] = $payloadData['customerId'];
            $payload['Payer']['BillingAddress'] = $billingAddress;
        } else {
            $payload['Payer']['BillingAddress']['Id'] = $payloadData['customerId'];
        }

        if (!empty($payloadData['cssUrl'])) {
            $payload['Styling']['CssUrl'] = $payloadData['cssUrl'];
            $payload['Styling']['ContentSecurityEnabled'] = true;
        }

        if ($payloadData['configSet']) {
            $payload['ConfigSet'] = $payloadData['configSet'];
        }
        
        if (isset($payloadData['usePreviousSavedCard']) && $payloadData['usePreviousSavedCard']) {
            unset($payload['CardForm']);
            unset($payload['PaymentMethods']);
            $payload['PaymentMeans']['Alias']['Id'] = $payloadData['usePreviousSavedCard'];
            if ($payloadData['verificationCodeMandatory']) {
                $payload['CardForm']['VerificationCode'] = $payloadData['verificationCodeMandatory'];
            }
        }
        if (isset($payloadData['saferpayFieldsToken']) && $payloadData['saferpayFieldsToken'] != '') {
            unset($payload['PaymentMethods']);
            $payload['PaymentMeans']['SaferpayFields']['Token'] = $payloadData['saferpayFieldsToken'];
        }

        if (isset($payloadData['forceAuthentication']) && $payloadData['forceAuthentication'] === 'yes') {
            $payload['Authentication']['ThreeDsChallenge'] = SaferpayShopware::THREEDS_FORCE;
        }

        $response = $this->sendRequest(static::ENDPOINT_TRANSACTION_INITIALIZE, $payload);
        if (
            (isset($response->RedirectRequired) && $response->RedirectRequired)
            &&
            (!isset($response->Redirect->RedirectUrl)
                || !is_string($response->Redirect->RedirectUrl)
                || trim($response->Redirect->RedirectUrl) === '')
        ) {
            throw new MissingRedirectUrlException(
                'Initialize transaction response has no valid property "RedirectUrl"',
                1656597985
            );
        }

        if (!isset($response->Token) || !is_string($response->Token) || trim($response->Token) === '') {
            throw new InvalidTokenException(
                'Initialize transaction response has no valid property "Token"',
            );
        }

        return $response;
    }

    /**
     * Send transaction authorize call
     */
    public function transactionAuthorize(string $token, bool $isSaveCard): object
    {
        $payload = array();
        $payload = [
            'Token' => $token,
        ];
        if ($isSaveCard) {
            $payload['RegisterAlias'] = [
                'IdGenerator' => 'RANDOM_UNIQUE',
                'Lifetime' => '1600'
            ];
        }
        $response = $this->sendRequest(static::ENDPOINT_TRANSACTION_AUTHORIZE, $payload);
        if (
            !isset($response->Transaction->Id)
            || trim($response->Transaction->Id) === ''
        ) {
            throw new MissingRedirectUrlException(
                'Initialize authorize response has no valid property "Transaction Id"',
                1656597985
            );
        }

        return $response;
    }

    /**
     * 1. Send request for Payment page assert
     * 2. Exception is thrown if no Transaction parameter in response
     * 3. Exception is thrown when no Status, Id, OrderId in Transaction
     * 4. Exception is thrown when there is Liability in response but no LiabilityShift
     *
     * @throws RuntimeException
     * @return Response
     */
    public function assertPaymentPage(string $token): object
    {
        $response = $this->sendRequest(static::ENDPOINT_PAYMENT_PAGE_ASSERT, [
            'Token' => $token
        ]);

        if (!isset($response->Transaction) || !is_object($response->Transaction)) {
            throw new InvalidTransactionException(
                'Assert payment page response has no valid property "Transaction"'
            );
        }

        if (
            !isset($response->Transaction->Status)
            || !is_string($response->Transaction->Status)
            || trim($response->Transaction->Status) === ''
        ) {
            throw new InvalidTransactionException(
                'Assert payment page response has no valid property "Transaction.Status"'
            );
        }

        if (
            !isset($response->Transaction->Id)
            || !is_string($response->Transaction->Id)
            || trim($response->Transaction->Id) === ''
        ) {
            throw new InvalidTransactionException(
                'Assert payment page response has no valid property "Transaction.Id"'
            );
        }

        if (
            !isset($response->Transaction->OrderId)
            || !is_string($response->Transaction->OrderId)
            || trim($response->Transaction->OrderId) === ''
        ) {
            throw new MissingTransactionOrderIdException(
                'Assert payment page response has no valid property "Transaction.OrderId"',
                1657210058
            );
        }

        if (isset($response->Liability) && (!is_object($response->Liability)
            || !isset($response->Liability->LiabilityShift)
            || !is_bool($response->Liability->LiabilityShift))) {
            throw new MissingLiabilityAndLiabilityShiftException(
                'Assert payment page response has property "Liability"
                 but no valid property "Liability.LiabilityShift"',
                1662114981
            );
        }

        return $response;
    }

    /**
     * 1. Transaction Capture request is send
     * 2. If no CaptureId or Status in response, exception is thrown
     *
     * @throws RuntimeException
     */
    public function captureTransaction(string $transactionId, string $pendingNotifyUrl): object
    {
        $response = $this->sendRequest(static::ENDPOINT_TRANSACTION_CAPTURE, [
            'TransactionReference' => [
                'TransactionId' => $transactionId
            ],
            'PendingNotification' => [
                'NotifyUrl' => $pendingNotifyUrl
            ]
        ]);

        if (!isset($response->CaptureId) || !is_string($response->CaptureId) || trim($response->CaptureId) === '') {
            throw new MissingCaptureIdException(
                'Capture transaction response has no valid property "CaptureId"',
                1657210376
            );
        }

        if (!isset($response->Status) || !is_string($response->Status) || trim($response->Status) === '') {
            throw new MissingCaptureStatusException(
                'Capture transaction response has no valid property "Status"',
                1657210388
            );
        }

        return $response;
    }

    /**
     * Sends request to cancel a transaction
     */
    public function cancelTransaction(string $transactionId)
    {
        return $this->sendRequest(static::ENDPOINT_TRANSACTION_CANCEL, [
            'TransactionReference' => [
                'TransactionId' => $transactionId
            ]
        ]);
    }

    /**
     * 1. Sends request to assert capture
     * 2. If no status in response, exception is thrown
     *
     * @throws \RuntimeException
     */
    public function assertCapture(string $captureId): object
    {
        $response = $this->sendRequest(
            static::ENDPOINT_TRANSACTION_ASSERT_CAPTURE,
            [
                'CaptureReference' => [
                    'CaptureId' => $captureId
                ]
            ]
        );

        if (!isset($response->Status) || !is_string($response->Status) || trim($response->Status) === '') {
            throw new MissingAssertStatusException(
                'Assert transaction capture response has no valid property "Status"',
                1657646519
            );
        }

        return $response;
    }

    /**
     * Converts amount to minor unit
     */
    protected function amountToMinorUnit(float $amount): float
    {
        return (int) (round($amount * 100));
    }

    /**
     * Adds request header to payload and returns the same
     */
    protected function addRequestHeaderToPayload(array &$payload): void
    {
        $payload['RequestHeader'] = [
            'SpecVersion' => $this->apiVersion, //latest
            'CustomerId' => $this->customerId,
            'RequestId' => Uuid::randomHex(),
            'RetryIndicator' => 0,
            'ClientInfo' => [
                'ShopInfo' => $this->shopInfo
            ]
        ];
    }

    /**
     * 1. Adds a request header to payload
     * 2. Creates stream context
     * 3. Sends http request
     * 4. If error in http request, ApiRequestException is thrown
     * 5. If the response from request is not object, RuntimeException is thrown
     *
     * @throws ApiRequestException
     * @throws \RuntimeException
     * @return $responseObject
     */
    protected function sendRequest(string $endpoint, array $payload, bool $isAltHeaderRequired = false): object
    {
        $this->addRequestHeaderToPayload($payload);
        $randomHex = Uuid::randomHex();
        if (!$isAltHeaderRequired) {
            $streamContext = stream_context_create([
                'http' => [
                    'ignore_errors' => true,
                    'method' => 'POST',
                    'header' => 'Content-type: application/json' . "\r\n"
                        . 'Accept: application/json; charset=utf-8' . "\r\n"
                        . 'Authorization: Basic ' . base64_encode($this->username . ':' . $this->password) . "\r\n",
                    'content' => json_encode($payload)
                ],
                'ssl' => [
                    'verify_peer' => true
                ]
            ]);
        } else {
            $streamContext = stream_context_create([
                'http' => [
                    'ignore_errors' => true,
                    'method' => 'POST',
                    'header' => 'Content-type: application/json' . "\r\n"
                        . 'Accept: application/json; charset=utf-8' . "\r\n"
                        . 'Authorization: Basic ' . base64_encode($this->username . ':' . $this->password) . "\r\n"
                        . 'Saferpay-RequestId: ' . $randomHex . "\r\n"
                        . 'Saferpay-ApiVersion: ' . $this->apiVersion . "\r\n",
                    'content' => json_encode($payload)
                ],
                'ssl' => [
                    'verify_peer' => true
                ]
            ]);
        }


        $requestUri = $this->apiBaseUrl . $endpoint;
        $responseBody = (string) file_get_contents($requestUri, false, $streamContext);
        if (
            !isset($http_response_header[0]) ||
            !is_string($http_response_header[0]) ||
            !str_contains($http_response_header[0], '200 OK')
        ) {
            throw new ApiRequestException(
                $requestUri,
                $responseBody,
                1656597979
            );
        }

        $responseObject = json_decode($responseBody);
        LoggerService::logRequestOrResponse(json_encode([
            'http' => [
                'ignore_errors' => true,
                'method' => 'POST',
                'header' => 'Content-type: application/json' . "\r\n"
                    . 'Accept: application/json; charset=utf-8' . "\r\n"
                    . 'Authorization: Basic ' . base64_encode($this->username . ':' . $this->password) . "\r\n"
                    . 'Saferpay-RequestId: ' . $randomHex . "\r\n"
                    . 'Saferpay-ApiVersion: ' . $this->apiVersion . "\r\n",
                'content' => json_encode($payload)
            ],
            'ssl' => [
                'verify_peer' => true
            ]
        ]), 'request');
        LoggerService::logRequestOrResponse($responseBody, 'response');
        if (!is_object($responseObject)) {
            throw new FailedToParseSaferpayApiResponseException(
                'Failed to parse Saferpay API response body of endpoint ' . $endpoint . ':' . $responseBody,
                1656597980
            );
        }

        return $responseObject;
    }

    /**
     *
     * Sends request to cancel a transaction
     */
    public function refund($requestArray)
    {
        return $this->sendRequest(static::ENDPOINT_REFUND, [
            'Refund' => [
                'Amount' => [
                    'Value' => $this->amountToMinorUnit((float) $requestArray['refundAmount']),
                    'CurrencyCode' => $requestArray['currencyCode']
                ],
                'OrderId' => $requestArray['orderId'],
                'Description' => $requestArray['reasonRefund'],
                'RestrictRefundAmountToCapturedAmount' => true
            ],
            'CaptureReference' => [
                'CaptureId' => $requestArray['captureId'],
            ],
            'PendingNotification' => [
                'NotifyUrl' => $requestArray['notifyUrl'],
            ]
        ]);
    }

    /**
     * Refund capture request
     */
    public function refundTransactionCapture($requestArray)
    {
        return $this->sendRequest(static::ENDPOINT_TRANSACTION_CAPTURE, [
            'TransactionReference' => [
                'TransactionId' => $requestArray['transactionId']
            ],
        ]);
    }

    /**
     * Refund assert request
     */
    public function refundAssertCall($requestArray)
    {
        return $this->sendRequest(static::ENDPOINT_REFUND_ASSERT, [
            'TransactionReference' => [
                'TransactionId' => $requestArray['transactionId']
            ],
        ]);
    }

    /**
     * API call to generate Field Access Token
     *
     * @param array $requestArray
     * @return object
     */
    public function generateFieldAccessToken(array $requestArray): object
    {
        if ($requestArray['customerId'] && $requestArray['terminalId']) {
            $endpoint = str_replace(
                ['[customerId]', '[terminalId]'],
                [$requestArray['customerId'], $requestArray['terminalId']],
                static::ENDPOINT_ACCESS_TOKEN
            );
            return $this->sendRequest($endpoint, [
                'SourceUrls' => $requestArray['SourceUrls'],
                'Description' => $requestArray['Description']
            ], true);
        } else {
            return false;
        }
    }

    /**
     * Authorize Direct request for payment using saved cards
     */
    public function transactionAuthorizeDirect($payloadData)
    {
        $payload = [
            'TerminalId' => $this->terminalId,
            'Payment' => [
                'Amount' => [
                    'Value' => $this->amountToMinorUnit($payloadData['totalAmount']),
                    'CurrencyCode' => $payloadData['currencyCode']
                ],
                'OrderId' => $payloadData['orderNumber'],
                'Description' => $payloadData['description']
            ],
            'PaymentMeans' => [
                'Alias' => [
                    'Id' => $payloadData['usePreviousSavedCard']
                ]
            ]
        ];
        $response = $this->sendRequest(static::ENDPOINT_TRANSACTION_AUTHORIZE_DIRECT, $payload);
        if (
            !isset($response->Transaction->Id)
            || trim($response->Transaction->Id) === ''
        ) {
            throw new MissingTransactionException(
                'Initialize authorize response has no valid property "Transaction Id"',
                1656597985
            );
        }

        return $response;
    }

    /**
     * Delete Saved Card Alias
     */
    public function aliasDelete($aliasId)
    {
        $payload = [
            'AliasId' => $aliasId
        ];
        $response = $this->sendRequest(static::ENDPOINT_ALIAS_DELETE, $payload);
        if (
            !isset($response->ResponseHeader->RequestId)
            || trim($response->ResponseHeader->RequestId) === ''
        ) {
            throw new MissingRequestIdException(
                'Initialize authorize response has no valid property "Request Id"',
                1656597985
            );
        }

        return $response;
    }

    /**
     * Update Saved Card Alias
     */
    public function aliasUpdate($payloadData)
    {
        $payload = [
            'UpdateAlias' => [
                'Id' => $payloadData['aliasId'],
                'Lifetime' => 1600
            ],
            'UpdatePaymentMeans' => [
                'Card' => [
                    'ExpMonth' => (int) $payloadData['expMonth'],
                    'ExpYear' => (int) $payloadData['expYear']
                ]
            ]
        ];
        $response = $this->sendRequest(static::ENDPOINT_ALIAS_UPDATE, $payload);
        if (
            !isset($response->ResponseHeader->RequestId)
            || trim($response->ResponseHeader->RequestId) === ''
        ) {
            throw new MissingRequestIdException(
                'Initialize authorize response has no valid property "Request Id"',
                1656597985
            );
        }

        return $response;
    }

    /**
     * Insert Alias
     */
    public function aliasInsert($payloadData)
    {
        $payload = [
            'RegisterAlias' => [
                'IdGenerator' => 'RANDOM_UNIQUE',
                'Lifetime' => 1600
            ],
            'Type' => $payloadData['type'],
            'ReturnUrl' => [
                'Url' => $payloadData['returnUrl']
            ],

        ];
        if ($payloadData['type'] == 'CARD') {
            $payload['Check'] = [
                'Type' => 'ONLINE_STRONG',
                'TerminalId' => $payloadData['terminalId']
            ];
            $payload['PaymentMeans'] = [
                'SaferpayFields' => [
                    'Token' => $payloadData['saferpayFieldsAccessToken']
                ]
            ];
        }
        $response = $this->sendRequest(static::ENDPOINT_ALIAS_INSERT, $payload);
        if (
            !isset($response->ResponseHeader->RequestId)
            || trim($response->ResponseHeader->RequestId) === ''
        ) {
            throw new MissingRequestIdException(
                'Initialize authorize response has no valid property "Request Id"',
                1656597985
            );
        }
        return $response;
    }

    /**
     * Assert Alias Insert
     */
    public function assertAliasInsert($token)
    {
        $payload['Token'] = $token;
        $response = $this->sendRequest(static::ENDPOINT_ALIAS_ASSERT_INSERT, $payload);
        if (
            !isset($response->ResponseHeader->RequestId)
            || trim($response->ResponseHeader->RequestId) === ''
        ) {
            throw new MissingRequestIdException(
                'Initialize authorize response has no valid property "Request Id"',
                1656597985
            );
        }

        return $response;
    }
}
