<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Checkout\Payment\Cart\PaymentHandler;

use DateTime;
use Saferpay\Shopware\Api\Exception\ApiRequestException;
use Saferpay\Shopware\Service\LogEntryService;
use Saferpay\Shopware\Service\LoggerService;
use Saferpay\Shopware\Service\PaymentService;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Cart\PaymentHandler\AsynchronousPaymentHandlerInterface;
use Shopware\Core\Checkout\Payment\PaymentException;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

class SaferpayPaymentHandler implements AsynchronousPaymentHandlerInterface
{
    private PaymentService $paymentService;
    private LoggerService $logger;
    private LogEntryService $logEntryService;

    /**
     * Initialize the class parameters
     */
    public function __construct(
        PaymentService $paymentService,
        LoggerService $logger,
        LogEntryService $logEntryService
    ) {
        $this->paymentService = $paymentService;
        $this->logger = $logger;
        $this->logEntryService = $logEntryService;
    }

    /**
     * Calls payment service of the plugin to proceed with payment
     *
     * @throws Exception
     * @throws PaymentException
     */
    public function pay(
        AsyncPaymentTransactionStruct $transaction,
        RequestDataBag $dataBag,
        SalesChannelContext $salesChannelContext
    ): RedirectResponse {
        try {
            $redirectUrl = $this->paymentService->initializePayment(
                $transaction->getOrderTransaction(),
                $transaction->getOrder(),
                $transaction->getReturnUrl(),
                $salesChannelContext->getSalesChannel(),
                $salesChannelContext->getContext()
            );
        } catch (\Exception $exception) {
            $errorMessage = 'Error Code : 1031 : An error occured while initializing Saferpay payment page: '
                . PHP_EOL . $exception->getMessage();
            $this->logger->error($errorMessage);
            $errorContext = [
                'orderId' => $transaction->getOrder()->getId(),
                'orderNumber' => $transaction->getOrder()->getOrderNumber(),
                'orderTransactionId' => $transaction->getOrderTransaction()->getId(),
                'function' => 'pay',
                'saferpayErrorCode' => '1031',
                'message' => $errorMessage
            ];
            $this->logEntryService->addLogEntry($salesChannelContext->getContext(), $errorMessage, $errorContext);
            throw PaymentException::asyncProcessInterrupted(
                $transaction->getOrderTransaction()->getId(),
                'An error occurred during the communication with external payment gateway' . PHP_EOL . $errorMessage
            );
        }

        return new RedirectResponse($redirectUrl);
    }

    /**
     * 1. If customer cancels the payment, Exception is thrown
     * 2. Calls payment service of the plugin to finalize the payment
     *
     * @throws PaymentException
     */
    public function finalize(
        AsyncPaymentTransactionStruct $transaction,
        Request $request,
        SalesChannelContext $salesChannelContext
    ): void {
        $orderTransaction = $transaction->getOrderTransaction();
        try {
            $this->paymentService->finalizePayment(
                $transaction->getOrderTransaction(),
                $salesChannelContext->getSalesChannel(),
                $salesChannelContext->getContext(),
                'paymentHandler'
            );
        } catch (\Exception $exception) {
            $errorMessage = 'An error occured while finalizing Saferpay payment: ' . PHP_EOL . $exception->getMessage();
            $this->logger->error($errorMessage);
            $errorContext = [
                'orderId' => $transaction->getOrder()->getId(),
                'orderNumber' => $transaction->getOrder()->getOrderNumber(),
                'orderTransactionId' => $transaction->getOrderTransaction()->getId(),
                'function' => 'finalize',
                'saferpayErrorCode' => '1051',
                'message' => $errorMessage
            ];
            $this->logEntryService->addLogEntry($salesChannelContext->getContext(), $errorMessage, $errorContext);
            /**
             * Manage PaymentPage Process - Cancel
             */
            if (strpos($errorMessage, ApiRequestException::ERROR_NAME_TRANSACTION_ABORT) !== false) {
                $this->errorCheckAndDetermineTransactionAbort($errorMessage, $orderTransaction->getId());
            }

            if($orderTransaction->getPaymentMethod()->getName() == "TWINT" &&
            strpos($errorMessage, ApiRequestException::ERROR_NAME_TRANSACTION_DECLINED) !== false) {
                $this->errorCheckAndDetermineTransactionDeclined($errorMessage, $orderTransaction->getId());
            }
            throw PaymentException::asyncFinalizeInterrupted($orderTransaction->getId(), $errorMessage);
        }
    }

    /**
     * Function To Check Error And Determine Transaction Abort
     *
     * @param string $errorMessage
     * @param string $orderTransactionId
     *
     * @throws PaymentException
     * @return void
     */
    public function errorCheckAndDetermineTransactionAbort(string $errorMessage,string $orderTransactionId) {
        $messageSplitPos = strpos($errorMessage, '):');
        if ($messageSplitPos) {
            $errorMessageSaferpay = substr($errorMessage, $messageSplitPos + 2);
        }
        if (($errorMessageSaferpay ?? 0) && is_string($errorMessageSaferpay)) {
            $errorMessageJson = json_decode($errorMessageSaferpay);
            if (
                $errorMessageJson &&
                $errorMessageJson->ErrorName &&
                $errorMessageJson->ErrorName == ApiRequestException::ERROR_NAME_TRANSACTION_ABORT
            ) {
                throw PaymentException::customerCanceled(
                    $orderTransactionId,
                    'Customer canceled the payment on the Saferpay page'
                );
            }
        }


    }

    /**
     * Function To Check Error And Determine Transaction Declined
     *
     * @param string $errorMessage
     * @param string $orderTransactionId
     *
     * @throws PaymentException
     * @return void
     */
    public function errorCheckAndDetermineTransactionDeclined(string $errorMessage,string $orderTransactionId) {
        $messageSplitPos = strpos($errorMessage, '):');
        if ($messageSplitPos) {
            $errorMessageSaferpay = substr($errorMessage, $messageSplitPos + 2);
        }
        if (($errorMessageSaferpay ?? 0) && is_string($errorMessageSaferpay)) {
            $errorMessageJson = json_decode($errorMessageSaferpay);
            if (
                $errorMessageJson &&
                $errorMessageJson->ErrorName &&
                $errorMessageJson->ErrorName == ApiRequestException::ERROR_NAME_TRANSACTION_DECLINED
            ) {

                throw PaymentException::customerCanceled(
                    $orderTransactionId,
                    'Customer canceled the payment on the Saferpay page'
                );
            }
        }
    }
}
