<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Core\Framework\Api\Controller;

use Composer\Util\Http\Response as HttpResponse;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Attribute\Route;
use Saferpay\Shopware\Service\PaymentService;
use Shopware\Core\Framework\Log\Package;
use Saferpay\Shopware\SaferpayShopware;

#[Route(defaults: ['_routeScope' => ['api']])]
#[Package('api')]
class ApiController extends AbstractController
{
    private EntityRepository $orderTransactionRepository;
    private OrderTransactionStateHandler $orderTransactionStateHandler;
    private PaymentService $paymentService;

    /**
     * Initialize the parameters of the class
     */
    public function __construct(
        EntityRepository $orderTransactionRepository,
        OrderTransactionStateHandler $orderTransactionStateHandler,
        PaymentService $paymentService
    ) {
        $this->orderTransactionRepository = $orderTransactionRepository;
        $this->orderTransactionStateHandler = $orderTransactionStateHandler;
        $this->paymentService = $paymentService;
    }

    /**
     * 1. Validates the transactionId and notifyToken, if invalid throws Exception
     * 2. If a valid orderTransaction is got from fetchOrderTransaction, payment is finalized
     */
    #[Route(path: '/api/_action/saferpay-shopware/notify/{transactionId}/success',
    name: 'api.saferpay-shopware.notify.success', defaults: ["auth_required"=>false],
    methods: ['GET'], requirements: ["version"=>"\d+", "transactionId"=>"[0-9a-f]+"])]
    public function notifySuccess(string $transactionId, Request $request, Context $context): Response
    {

        sleep(30);
        /** @var OrderTransactionEntity */
        $orderTransaction = $this->fetchOrderTransaction(
            $transactionId,
            $request->get('notifyToken', ''),
            $context
        );

        $this->paymentService->finalizePayment(
            $orderTransaction,
            $orderTransaction->getOrder()->getSalesChannel(),
            $context,
            'notifySuccess'
        );

        return new JsonResponse(null, Response::HTTP_NO_CONTENT);
    }

    /**
     * 1. Validates the transactionId and notifyToken, if invalid throws Exception
     * 2. If a valid orderTransaction is got from fetchOrderTransaction, payment is finalized
     */
    #[Route(path: '/api/_action/saferpay-shopware/notification/{transactionId}/notify',
    name: 'api.saferpay-shopware.notify.notify', defaults: ["auth_required"=>false],
    methods: ['GET'], requirements: ["version"=>"\d+", "transactionId"=>"[0-9a-f]+"])]
    public function handleNotifyAction(string $transactionId, Request $request, Context $context): Response
    {
        sleep(20);
        /** @var OrderTransactionEntity */
        $orderTransaction = $this->fetchOrderTransaction(
            $transactionId,
            $request->get('notifyToken', ''),
            $context
        );
        $orderTransactionId = $orderTransaction->getId();

        $this->paymentService->handleRefundNotifyUrl($orderTransactionId, $context);

        return new JsonResponse(null, Response::HTTP_NO_CONTENT);
    }

    /**
     * 1. If no notifyToken, BadRequestException is thrown
     * 2. Checks if transactionId gets a valid orderTransaction, if not NotFoundHttpException is thrown
     * 3. Checks if the notifyToken belongs to the transaction, if not AccessDeniedHttpException is thrown
     *
     * @throws BadRequestHttpException
     * @throws NotFoundHttpException
     * @throws AccessDeniedHttpException
     * @return $orderTransaction
     */
    private function fetchOrderTransaction(
        string $transactionId,
        string $notifyToken,
        Context $context
    ): OrderTransactionEntity {
        $notifyToken = trim($notifyToken);
        if (!$notifyToken) {
            throw new BadRequestHttpException('Query parameter "notifyToken" is missing');
        }

        $orderTransaction = $this->orderTransactionRepository
            ->search((new Criteria([$transactionId]))
                ->addAssociation('stateMachineState')
                ->addAssociation('order')
                ->addAssociation('order.salesChannel'), $context)
            ->first();

        if (!$orderTransaction instanceof OrderTransactionEntity) {
            throw new NotFoundHttpException('Order transaction ' . $transactionId . ' not found');
        }

        $orderNotifyToken = trim((string) ($orderTransaction->getCustomFields()
        [SaferpayShopware::CUSTOM_FIELD_SAFERPAY_NOTIFY_TOKEN] ?? ''));
        if ($notifyToken !== $orderNotifyToken) {
            throw new AccessDeniedHttpException('Given notifyToken does not belong to given order transaction');
        }

        return $orderTransaction;
    }

    /**
     * 1. Checks if the transactionId and notifyToken is valid, if not exception is thrown
     * 2. orderTransactionStateHandler declares the payment as failed
     *
     */
    #[Route(path: '/api/_action/shopware-saferpay/notify/{transactionId}/fail',
    name: 'api.saferpay-shopware.notify.fail', defaults: ["auth_required"=>false],
    methods: ['GET'], requirements: ["version"=>"\d+", "transactionId"=>"[0-9a-f]+"])]
    public function notifyFail(string $transactionId, Request $request, Context $context): Response
    {
        sleep(20);
        $this->fetchOrderTransaction($transactionId, $request->get('notifyToken', ''), $context);

        $this->orderTransactionStateHandler->fail($transactionId, $context);

        return new JsonResponse(null, Response::HTTP_NO_CONTENT);
    }

    /**
     * 1. Checks if the transactionId and notifyToken is valid, if not exception is thrown
     * 2. In payment service, transaction is updated as pending
     */
    #[Route(path: '/api/_action/saferpay-shopware/notify-pending/{transactionId}',
    name: 'api.saferpay-shopware.notify-pending', defaults: ["auth_required"=>false],
    methods: ['GET'], requirements: ["version"=>"\d+", "transactionId"=>"[0-9a-f]+"])]
    public function notifyPending(string $transactionId, Request $request, Context $context): Response
    {
        sleep(20);
        /** @var OrderTransactionEntity */
        $orderTransaction = $this->fetchOrderTransaction($transactionId, $request->get('notifyToken', ''), $context);

        $this->paymentService
            ->updatePendingTransaction($orderTransaction, $orderTransaction->getOrder()->getSalesChannel(), $context);

        return new JsonResponse(null, Response::HTTP_NO_CONTENT);
    }


    /**
     * Refund Transaction
     */
    #[Route(path: '/api/_action/saferpay-shopware/refund-transaction', 
    name: 'api.saferpay_shopware.refund_transaction', defaults: ["auth_required"=>false], 
    methods: ['POST'], requirements: ["version"=>"\d+", "transactionId"=>"[0-9a-f]+"])]
    public function refundTransaction(
        Request $request,
        Context $context
    ): JsonResponse {

        try {
            $payloadContent = $request->getContent();
            $payloadContentArray = json_decode($payloadContent, true);
            $status = $this->paymentService->handleRefund($payloadContentArray, $context);

            return new JsonResponse([
                'success' => true,
                'status' => $status,
                'message' => 'Refund successful, restock item manually'
            ]);
        } catch (\Throwable $th) {

            return new JsonResponse([
                'success' => false,
                'message' => 'Saferpay API Request Failed'
            ]);
        }
    }
}
