<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Installer;

use Shopware\Core\Defaults;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\Plugin\Context\ActivateContext;
use Shopware\Core\Framework\Plugin\Context\DeactivateContext;
use Shopware\Core\Framework\Plugin\Context\InstallContext;
use Shopware\Core\Framework\Plugin\Context\UninstallContext;
use Shopware\Core\System\CustomField\CustomFieldTypes;
use Saferpay\Shopware\SaferpayShopware;

class CustomFieldsInstaller extends InstallerHelper
{
    
    private EntityRepository $customFieldRepository;
    private EntityRepository $customFieldSetRepository;
    private array $countries;
    private array $customFields;
    private array $customFieldSets;

    public function __construct(
        EntityRepository $customFieldSetRepository,
        EntityRepository $customFieldRepository,
        array $countries
    ) {
        $this->customFieldRepository = $customFieldRepository;
        $this->customFieldSetRepository = $customFieldSetRepository;
        $this->countries = $countries;

        $this->customFieldSets = $this->getFieldSets();

        $this->addSaferpayCustomFields();
        $this->addCountryCurrencyCustomFields();
        $this->addOrderTotalCustomFields();
        $this->addCustomerCustomFields();
        $this->addSpecificCustomFields();
        $this->addCreditCardCustomFields();
    }

    /**
     * Add specific custom fields for payment methods
     */
    private function addSpecificCustomFields()
    {
        $this->customFields[] = $this->getSaferpayShopwarePreAuthorizationCustomFields();
        $this->customFields[] = $this->getSaferPayShopwareCapturingCustomFields();
        $this->customFields[] = $this->getSaferpayShopwareAccountDetailsCustomFields();
        $this->customFields[] = $this->getTestAccountDetailsCustomFields();
        $this->customFields[] = $this->getShopwareAuthorizationMethodCustomFields();
        $this->customFields[] = $this->getSaveCardCustomField();
        $this->customFields[] = $this->displayCardHolderNameConfig();
    }

    private function addCreditCardCustomFields()
    {
        $this->customFields[] = $this->getSaferpayFieldAccessToken();
        $this->customFields[] = $this->getSaferpayFieldCssUrl();
        $this->customFields[] = $this->getSaferpayFieldJsLibraryUrl();
        $this->customFields[] = $this->getSaferpayFieldApiUrl();
        $this->customFields[] = $this->getCreditCardTheme();

    }

    /**
     * Add custom fields in saferpay tab
     */
    private function addSaferpayCustomFields()
    {
        $this->customFields[] = [
            'id' => '4bcb2ebdfff24d46ae4a622c2b22e59b',
            'name' => 'saferpay_shopware_payment_means',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Payment means',
                    'de-DE' => 'Zahlungsmittel',
                    Defaults::LANGUAGE_SYSTEM => 'Payment means',
                ],
                'helpText' => [
                    'en-GB' => 'Restricts the means of payment which are available on the Saferpay payment page for
                     this payment method. If empty, all Saferpay payment means are available.',
                    'de-DE' => 'Schränkt die Zahlungsmittel ein, die auf der Saferpay-Zahlungsseite für diese
                     Zahlungsart zur Verfügung stehen. Wenn leer, sind alle Saferpay Zahlungsmittel verfügbar.',
                    Defaults::LANGUAGE_SYSTEM => 'Restricts the means of payment which are available on the Saferpay
                     payment page for this payment method. If empty, all Saferpay payment means are available.',
                ],
                'componentName' => 'sw-multi-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => $this->getPaymentMeansOptions()
            ],
        ];
        $this->customFields[] = [
            'id' => 'b6ff9dff81474bac8bd4fe430c73fd50',
            'name' => 'saferpay_shopware_wallets',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'de-DE' => 'Wallets',
                    'en-GB' => 'Wallets',
                    Defaults::LANGUAGE_SYSTEM => 'Wallets',
                ],
                'helpText' => [
                    'en-GB' => 'Sets which wallets are available on the Saferpay payment page for this payment
                     method. If only one wallet and no payment mean is set, the payment selection step on the
                     Saferpay payment page is skipped. If empty, no wallets are available.',
                    'de-DE' => 'Legt fest, welche Wallets auf der Saferpay-Zahlungsseite für diese Zahlungsart
                     zur Verfügung stehen. Wenn nur eine Wallet und kein Zahlungsmittel eingestellt ist, wird der
                     Schritt der Zahlungsauswahl auf der Saferpay-Zahlungsseite übersprungen. Wenn leer, sind
                     keine Wallets verfügbar.',
                    Defaults::LANGUAGE_SYSTEM => 'Sets which wallets are available on the Saferpay payment page
                     for this payment method. If only one wallet and no payment mean is set, the payment
                     selection step on the Saferpay payment page is skipped. If empty, no wallets are
                     available.',
                ],
                'componentName' => 'sw-multi-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 50,
                'options' => $this->getWalletOptions()
            ],
        ];
    }

    /**
     * Add country currency custom fields
     */
    private function addCountryCurrencyCustomFields()
    {
        $this->customFields[] = [
            'id' => 'c22dc09efc2e427fb1fb6d639845d53b',
            'name' => 'saferpay_shopware_payment_countries',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::COUNTRY_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Country',
                    'de-DE' => 'Land',
                    Defaults::LANGUAGE_SYSTEM => 'Country',
                ],
                'helpText' => [
                    'en-GB' => 'Select payment allowed countries if any specification else leave it free',
                    'de-DE' => 'Wählen Sie die Länder aus, in denen die Zahlung zulässig ist, und lassen
                     Sie die Angabe anderer Angaben frei',
                    Defaults::LANGUAGE_SYSTEM => 'Select payment allowed countries if any specification
                     else leave it free',
                ],
                'componentName' => 'sw-multi-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => $this->getCountryOptions()
            ]
        ];
        $this->customFields[] = [
            'id' => '918d4fded15e41919462a1ca8d1e9cf9',
            'name' => 'saferpay_shopware_payment_currencies',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::COUNTRY_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Currency',
                    'de-DE' => 'Währung',
                    Defaults::LANGUAGE_SYSTEM => 'Currency',
                ],
                'helpText' => [
                    'en-GB' => 'Choose the currencies for which the payment method need to be shown',
                    'de-DE' => 'Wählen Sie die Währungen aus, für die die Zahlungsmethode angezeigt
                     werden soll',
                    Defaults::LANGUAGE_SYSTEM => 'Choose the currencies for which the payment method
                     need to be shown',
                ],
                'componentName' => 'sw-multi-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 50,
                'options' => $this->getCurrencyOptions()
            ]
        ];
    }

    /**
     * Add order total custom fields
     */
    private function addOrderTotalCustomFields()
    {
        $this->customFields[] = [
            'id' => '0189451a92027088b2c8db6c2bb690c7',
            'name' => 'saferpay_shopware_payment_min_order_total',
            'type' => CustomFieldTypes::FLOAT,
            'customFieldSetId' => self::ORDER_TOTAL_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'number',
                'label' => [
                    'en-GB' => 'Minimal Order Total',
                    'de-DE' => 'Mindestbestellsumme',
                    Defaults::LANGUAGE_SYSTEM => 'Minimal Order Total',
                ],
                'helpText' => [
                    'en-GB' => 'Set here the minimal order total for which this payment method is
                     available. If it is set to zero, it is always available.',
                    'de-DE' => 'Stellen Sie hier den Mindestbestellwert ein, für den diese Zahlungsart
                     verfügbar ist. Wenn es auf Null gesetzt ist, ist es immer verfügbar.',
                    Defaults::LANGUAGE_SYSTEM => 'Set here the minimal order total for which
                     this payment method is available. If it is set to zero, it is always available.'
                ],
                'numberType' => 'float',
                'componentName' => 'sw-field',
                'customFieldType' => 'number',
                'customFieldPosition' => 0
            ]
        ];
        $this->customFields[] = [
            'id' => '0189451a92027088b2c8db6c2c9cc805',
            'name' => 'saferpay_shopware_payment_max_order_total',
            'type' => CustomFieldTypes::FLOAT,
            'customFieldSetId' => self::ORDER_TOTAL_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'number',
                'label' => [
                    'en-GB' => 'Maximal Order Total',
                    'de-DE' => 'Maximale Bestellsumme',
                    Defaults::LANGUAGE_SYSTEM => 'Maximal Order Total',
                ],
                'helpText' => [
                    'en-GB' => 'Set here the maximal order total for which this payment method
                         is available. If it is set to zero, it is always available.',
                    'de-DE' => 'Stellen Sie hier die maximale Bestellsumme ein, für die diese
                         Zahlungsart verfügbar ist. Wenn es auf Null gesetzt ist, ist es immer verfügbar.',
                    Defaults::LANGUAGE_SYSTEM => 'Set here the maximal order total for which
                         this payment method is available. If it is set to zero, it is always available.'
                ],
                'numberType' => 'float',
                'componentName' => 'sw-field',
                'customFieldType' => 'number',
                'customFieldPosition' => 50
            ]
        ];
    }

    /**
     * Add custom fields for customer field set
     */
    private function addCustomerCustomFields()
    {
        $this->customFields[] = [
            'id' => 'a7f706260b8d482cb4f978f2cc24215d',
            'name' => 'saferpay_shopware_payment_customer_address',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::CUSTOMER_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Customer Address',
                    'de-DE' => 'Kundenadresse',
                    Defaults::LANGUAGE_SYSTEM => 'Customer Address',
                ],
                'helpText' => [
                    'en-GB' => 'Should the customer address be sent to Saferpay?',
                    'de-DE' => 'Soll die Kundenadresse an Saferpay übermittelt werden?',
                    Defaults::LANGUAGE_SYSTEM => 'Should the customer address be sent to Saferpay?',
                ],
                'componentName' => 'sw-single-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => [
                    [
                        'value' => 'none',
                        'label' => [
                            'de-DE' => 'Nein',
                            'en-GB' => 'No',
                            Defaults::LANGUAGE_SYSTEM => 'No',
                        ]
                    ],
                    [
                        'value' => 'delivery',
                        'label' => [
                            'de-DE' => 'Lieferadresse senden',
                            'en-GB' => 'Send delivery address',
                            Defaults::LANGUAGE_SYSTEM => 'Send delivery address',
                        ]
                    ],
                    [
                        'value' => 'billing',
                        'label' => [
                            'de-DE' => 'Rechnungsadresse senden',
                            'en-GB' => 'Send billing address',
                            Defaults::LANGUAGE_SYSTEM => 'Send billing address',
                        ]
                    ],
                    [
                        'value' => 'both',
                        'label' => [
                            'de-DE' => 'Senden Sie Rechnung und Versand',
                            'en-GB' => 'Send both billing and shipping',
                            Defaults::LANGUAGE_SYSTEM => 'Send both billing and shipping',
                        ]
                    ],
                ]
            ]
        ];
        $this->customFields[] = [
            'id' => 'a6ff6bf36f7744dfa3f43c72636024b3',
            'name' => 'saferpay_shopware_payment_customer_confirmation_email',
            'type' => CustomFieldTypes::JSON,
            'customFieldSetId' => self::CUSTOMER_FIELDSET_ID_PAYMENT_METHOD,
            'config' => [
                'type' => 'select',
                'label' => [
                    'en-GB' => 'Customer Confirmation Email',
                    'de-DE' => 'Kundenbestätigungs-E-Mail',
                    Defaults::LANGUAGE_SYSTEM => 'Customer Confirmation Email',
                ],
                'helpText' => [
                    'en-GB' => 'Should Saferpay send a confirmation email to the customer. (Available
                     only if the customer is redirected to Saferpay.)',
                    'de-DE' => 'Sollte Saferpay eine Bestätigungs-E-Mail an den Kunden senden?
                     (Nur verfügbar, wenn der Kunde zu Saferpay weitergeleitet wird.)',
                    Defaults::LANGUAGE_SYSTEM => 'Should Saferpay send a confirmation email to the
                     customer. (Available only if the customer is redirected to Saferpay.)',
                ],
                'componentName' => 'sw-single-select',
                'customFieldType' => 'select',
                'customFieldPosition' => 0,
                'options' => [
                    [
                        'value' => '1',
                        'label' => [
                            'de-DE' => 'Ja',
                            'en-GB' => 'Yes',
                            Defaults::LANGUAGE_SYSTEM => 'Yes',
                        ]
                    ],
                    [
                        'value' => '0',
                        'label' => [
                            'de-DE' => 'Nein',
                            'en-GB' => 'No',
                            Defaults::LANGUAGE_SYSTEM => 'No',
                        ]
                    ]
                ]
            ]
        ];
    }

    /**
     * Get countries as options
     */
    private function getCountryOptions()
    {
        $countries = array();
        if (!empty($this->countries)) {
            foreach ($this->countries as $countryId => $countryNames) {
                $countries[] = [
                    'value' => $countryId,
                    'label' => [
                        'de-DE' => $countryNames['de-DE'],
                        'en-GB' => $countryNames['en-GB'],
                        Defaults::LANGUAGE_SYSTEM => $countryNames['en-GB']
                    ]
                ];
            }
        }
        return $countries;
    }

    /**
     * Get FieldSets
     */
    private function getFieldSets()
    {
        return [
            [
                'id' => self::FIELDSET_ID_PAYMENT_METHOD,
                'name' => 'saferpay_shopware_payment_method',
                'global' => true,
                'config' => [
                    'label' => [
                        'en-GB' => 'Saferpay',
                        'de-DE' => 'Saferpay',
                        Defaults::LANGUAGE_SYSTEM => 'Saferpay',
                    ],
                ],
                'relations' => [
                    [
                        'id' => '40fa22d5a3094bd195ff0983b45e041f',
                        'entityName' => 'payment_method',
                    ]
                ],
            ],
            [
                'id' => self::COUNTRY_FIELDSET_ID_PAYMENT_METHOD,
                'name' => 'saferpay_shopware_payment_country_currency',
                'global' => true,
                'config' => [
                    'label' => [
                        'en-GB' => 'Country & Currency',
                        'de-DE' => 'Land und Währung',
                        Defaults::LANGUAGE_SYSTEM => 'Country & Currency',
                    ]
                ],
                'relations' => [
                    [
                        'id' => '524e15f42296415790ef09067902b07c',
                        'entityName' => 'payment_method',
                    ]
                ]
            ],
            [
                'id' => self::ORDER_TOTAL_FIELDSET_ID_PAYMENT_METHOD,
                'name' => 'saferpay_shopware_payment_order_total',
                'global' => true,
                'config' => [
                    'label' => [
                        'en-GB' => 'Order Total',
                        'de-DE' => 'Auftragssumme',
                        Defaults::LANGUAGE_SYSTEM => 'Order Total',
                    ]
                ],
                'relations' => [
                    [
                        'id' => '384eb01995394018a9edabfbeaf2175e',
                        'entityName' => 'payment_method',
                    ]
                ]
            ],
            [
                'id' => self::CUSTOMER_FIELDSET_ID_PAYMENT_METHOD,
                'name' => 'saferpay_shopware_payment_customer',
                'global' => true,
                'config' => [
                    'label' => [
                        'en-GB' => 'Customer',
                        'de-DE' => 'Auftraggeber',
                        Defaults::LANGUAGE_SYSTEM => 'Customer',
                    ]
                ],
                'relations' => [
                    [
                        'id' => '6cbd75bc815542f0bd1f16be34792e0d',
                        'entityName' => 'payment_method',
                    ]
                ]
            ],
            [
                'id' => self::SPECIFIC_FIELDSET_ID_PAYMENT_METHOD,
                'name' => 'saferpay_shopware_payment_specific',
                'global' => false,
                'config' => [
                    'label' => [
                        'en-GB' => 'Other',
                        'de-DE' => 'Weiter',
                        Defaults::LANGUAGE_SYSTEM => 'Other',
                    ]
                ],
                'relations' => [
                    [
                        'id' => '9d246d53bdee4173bd327af84958a258',
                        'entityName' => 'payment_method',
                    ]
                ]
            ]
        ];
    }

    /**
     * Get the options to display as wallets in Payment Method
     */
    private function getWalletOptions()
    {
        $applePayText = 'Apple Pay';
        $googlePayText = 'Google Pay';
        return [
            [
                'value' => SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY,
                'label' => [
                    'de-DE' => $applePayText,
                    'en-GB' => $applePayText,
                    Defaults::LANGUAGE_SYSTEM => $applePayText,
                ]
            ],
            [
                'value' => SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY,
                'label' => [
                    'de-DE' => $googlePayText,
                    'en-GB' => $googlePayText,
                    Defaults::LANGUAGE_SYSTEM => $googlePayText,
                ]
            ],
        ];
    }

    /**
     * Get the options to display as payment means in Payment Methods
     */
    private function getPaymentMeansOptions()
    {
        $paymentMethods = array(
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY => 'Alipay',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS => 'American Express',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT => 'Bancontact',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB => 'Diners Club/Discover',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS => 'eps',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY => 'Przelewy24',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL => 'iDEAL',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB => 'JCB',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA => 'Klarna',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT => 'Sofort',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO => 'Maestro',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD => 'Mastercard',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE => 'myONE',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL => 'Paypal',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT => 'paydirekt',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD => 'PostFinance Card',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE => 'PostFinance E-Finance',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT => 'TWINT',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY => 'UnionPay',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA => 'Visa',
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS => 'WL Crypto Payments'
        );
        $paymentMeans = [
            [
                'value' => SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT,
                'label' => [
                    'de-DE' => 'Lastschrift',
                    'en-GB' => 'Direct Debit',
                    Defaults::LANGUAGE_SYSTEM => 'Direct Debit',
                ]
            ]
        ];
        foreach ($paymentMethods as $key => $value) {
            $paymentMeans[] = array(
                'value' => $key,
                'label' => [
                    'de-DE' => $value,
                    'en-GB' => $value,
                    Defaults::LANGUAGE_SYSTEM => $value
                ]
            );
        }
        return $paymentMeans;
    }

    /**
     * Install all custom field sets and custom fields
     */
    public function install(InstallContext $installContext): void
    {
        $context = $installContext->getContext();
        foreach ($this->customFieldSets as $customFieldSet) {
            $this->customFieldSetRepository->upsert([$customFieldSet], $context);
        }

        foreach ($this->customFields as $customField) {
            $this->customFieldRepository->upsert([$customField], $context);
        }
    }

    /**
     * Activate all custom field sets and custom fields
     */
    public function activate(ActivateContext $activateContext): void
    {
        $context = $activateContext->getContext();
        foreach ($this->customFieldSets as $customFieldSet) {
            $customFieldSet['active'] = true;
            $this->customFieldSetRepository->upsert([$customFieldSet], $context);
        }

        foreach ($this->customFields as $customField) {
            $customField['active'] = true;
            $this->customFieldRepository->upsert([$customField], $context);
        }
    }

    /**
     * Deactivate all custom field sets and custom fields
     */
    public function deactivate(DeactivateContext $deactivateContext): void
    {
        $context = $deactivateContext->getContext();
        $this->deactivateFields($context);
    }

    /**
     * Execute deactivation by setting active false
     */
    private function deactivateFields($context): void
    {
        foreach ($this->customFieldSets as $customFieldSet) {
            $customFieldSet['active'] = false;
            $this->customFieldSetRepository->upsert([$customFieldSet], $context);
        }

        foreach ($this->customFields as $customField) {
            $customField['active'] = false;
            $this->customFieldRepository->upsert([$customField], $context);
        }
    }

    /**
     * 1. Checks if keepUserData is set for uninstall
     * 2. If yes, custom field sets and custom fields are deactivated
     * 3. If no, custom field sets and custom fields are deleted
     */
    public function uninstall(UninstallContext $uninstallContext): void
    {
        $context = $uninstallContext->getContext();
        if ($uninstallContext->keepUserData()) {
            $this->deactivateFields($context);
        } else {
            foreach ($this->customFieldSets as $customFieldSet) {
                $this->customFieldSetRepository->delete([
                    [
                        'id' => $customFieldSet['id']
                    ]
                ], $context);
            }

            foreach ($this->customFields as $customField) {
                $this->customFieldRepository->delete([
                    [
                        'id' => $customField['id']
                    ]
                ], $context);
            }
        }
    }

}
