<?php

declare(strict_types=1);

/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Installer;

use Shopware\Core\Defaults;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Plugin\Context\DeactivateContext;
use Shopware\Core\Framework\Plugin\Context\InstallContext;
use Shopware\Core\Framework\Plugin\Context\UninstallContext;
use Saferpay\Shopware\Checkout\Payment\Cart\PaymentHandler\SaferpayPaymentHandler;
use Saferpay\Shopware\SaferpayShopware;
use Shopware\Core\Checkout\Payment\PaymentMethodEntity;
use Shopware\Core\Content\Media\File\FileSaver;
use Shopware\Core\Content\Media\File\MediaFile;
use Shopware\Core\Framework\Uuid\Uuid;

class PaymentMethodInstaller extends InstallerHelper
{
    private EntityRepository $paymentMethodRepository;
    private EntityRepository $mediaRepository;
    private EntityRepository $mediaFolderRepository;
    private EntityRepository $languageRepository;
    private array $paymentMethods;
    private const PAYMENT_METHOD_MEDIA_DIR = 'SaferpayShopware/src/Resources/icons';
    private FileSaver $fileSaver;

    /**
     * Initialize the payment methods and paymentMethodRepository
     */
    public function __construct(
        string $pluginId,
        EntityRepository $paymentMethodRepository,
        EntityRepository $mediaRepository,
        EntityRepository $mediaFolderRepository,
        EntityRepository $languageRepository,
        FileSaver $fileSaver
    ) {
        $this->paymentMethodRepository = $paymentMethodRepository;
        $this->mediaRepository = $mediaRepository;
        $this->mediaFolderRepository = $mediaFolderRepository;
        $this->fileSaver = $fileSaver;
        $this->languageRepository = $languageRepository;
        $paymentMethodArray = [
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID,
            SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY_ID,
            SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT_ID
        ];
        $paymentMethodList = [];
        $count = 0;
        foreach ($paymentMethodArray as $eachMethod) {
            $paymentMethodList[] = [
                'pluginId' => $pluginId,
                'handlerIdentifier' => SaferpayPaymentHandler::class,
                'position' => $count,
                'afterOrderEnabled' => true,
                'id' => $eachMethod,
                'translations' => $this->getPaymentTranslationArray($eachMethod),
                'technicalName' => $this->getPaymentMethodTechnicalName($eachMethod)
            ];
            $count++;
        }
        $this->paymentMethods = $paymentMethodList;
    }

    /**
     * Get the technical name of payment method
     */
    private function getPaymentMethodTechnicalName($paymentMethod): string
    {
        $technicalName = "";
        $spswText = 'SPSW - ';
        switch ($paymentMethod) {
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS;
                break;
            case SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID:
                $technicalName = 'SPSW - CREDIT_CARD';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY;
                break;
            case SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS_ID:
                $technicalName = $spswText . SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS;
                break;
            default:
                $technicalName = $spswText;
        }
        return $technicalName;
    }

    /**
     * Get translations for payment method
     */
    private function getPaymentTranslationArray($paymentMethod): array
    {
        $nameDesc = $this->getNameDescriptionTranslation($paymentMethod);
        $paymentMeansWallets = $this->getPaymentMeansWallets($paymentMethod);
        $customFieldsArray = [
            'saferpay_shopware_payment_means' => $paymentMeansWallets[0],
            'saferpay_shopware_wallets' => $paymentMeansWallets[1],
            'saferpay_shopware_payment_countries' => $this->getPaymentCountries($paymentMethod),
            'saferpay_shopware_payment_currencies' => $this->getPaymentCurrencies($paymentMethod),
            'saferpay_shopware_payment_min_order_total' => 0,
            'saferpay_shopware_payment_max_order_total' => 0,
            'saferpay_shopware_payment_customer_address' => 'none',
            'saferpay_shopware_payment_customer_confirmation_email' => '0',
        ];
        if ($this->checkIfPreAuthPaymentMethod($paymentMethod)) {
            $customFieldsArray['saferpay_shopware_pre_authorizations'] = '0';
        }
        if ($this->checkIfCapturingPaymentMethod($paymentMethod)) {
            $customFieldsArray['saferpay_shopware_capturing'] = 'direct';
        }
        if ($this->checkIfAuthorizationMethodPaymentMethod($paymentMethod)) {
            if ($paymentMethod == SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID) {
                $customFieldsArray['saferpay_shopware_authorization_method'] = 'widgetAuthorization';
            } else {
                $customFieldsArray['saferpay_shopware_authorization_method'] = 'paymentPage';
            }
        }
        if ($paymentMethod == SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL_ID) {
            $customFieldsArray['saferpay_shopware_account_details'] = [
                'ABNANL2A', 'ASNBNL21', 'BUNQNL2A', 'INGBNL2A', 'KNABNL2H',
                'RABONL2U', 'RBRBNL21', 'REVOLT21', 'SNSBNL2A', 'TRIONL2U',
                'FVLBNL22', 'BITSNL2A'
            ];
            $customFieldsArray['saferpay_shopware_test_account_details'] = [
                '0091', '0092'
            ];
        }
        if ($this->checkIfSaveCardPaymentMethod($paymentMethod)) {
            $customFieldsArray['saferpay_shopware_save_card'] = 'inactive';
        }

        if ($paymentMethod == SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID) {
            $customFieldsArray['saferpay_shopware_field_access_token'] = '';
            $customFieldsArray['saferpay_shopware_field_js_library_url'] = '';
            $customFieldsArray['saferpay_shopware_field_api_url'] = '';
            $customFieldsArray['saferpay_credit_card_theme'] = 'sample_1';
            $customFieldsArray['saferpay_shopware_field_css_url'] = '';
        }
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('locale.code', 'de-DE'));
        $language = $this->languageRepository->search($criteria, Context::createDefaultContext())->first();

        $translationArray = [
            Defaults::LANGUAGE_SYSTEM => [
                'name' => $nameDesc[0],
                'description' => $nameDesc[1],
                'customFields' => $customFieldsArray,
            ],
            $language->id => [
                'name' => $nameDesc[0],
                'description' => $nameDesc[1],
                'customFields' => $customFieldsArray,
            ]
        ];
        if ($paymentMethod == SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID) {
            $translationArray['de-DE'] = [
                'name' => 'Kredit-/Debitkarte',
                'description' => 'Kredit-/Debitkarte (Saferpay)'
            ];
        } elseif ($paymentMethod == SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT_ID) {
            $translationArray['de-DE'] = [
                'name' => 'Lastschrift',
                'description' => 'Lastschrift (Saferpay)'
            ];
        }
        return $translationArray;
    }

    /**
     * Get array if countries for which the payment method is enabled
     */
    private function getPaymentCountries($paymentMethod): array
    {
        $countries = array();
        if ($paymentMethod === SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID) {
            $countries = array('AT', 'BE', 'DK', 'FI', 'FR', 'DE', 'IT', 'NL', 'NO', 'ES', 'SE', 'CH', 'GB');
        }
        return $countries;
    }

    /**
     * Select the currency for payment method
     */
    private function getPaymentCurrencies($paymentMethod): array
    {
        switch ($paymentMethod) {
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY_ID:
                $allowedCurrencies = array('CHF');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID:
            case SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY_ID:
            case SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY_ID:
                $allowedCurrencies = array(
                    'CHF', 'AED', 'AFN', 'ALL', 'AMD', 'AOA', 'ARS', 'AUD', 'AWG', 'AZN', 'BAM',
                    'BBD', 'BDT', 'BGN', 'BHD', 'BIF', 'BMD', 'BND', 'BOB', 'BOV', 'BRL', 'BSD',
                    'BTN', 'BWP', 'BYN', 'BZD', 'CAD', 'CDF', 'CHW', 'CLF', 'CLP', 'CNY', 'COP',
                    'COU', 'CRC', 'CUP', 'CVE', 'CZK', 'DJF', 'DKK', 'DOP', 'DZD', 'EGP', 'ERN',
                    'ETB', 'EUR', 'FJD', 'FKP', 'GBP', 'GEL', 'GHS', 'GIP', 'GMD', 'GNF', 'GTQ',
                    'GYD', 'HKD', 'HNL', 'HTG', 'HUF', 'IDR', 'ILS', 'INR', 'IQD', 'IRR', 'ISK',
                    'JMD', 'JOD', 'JPY', 'KES', 'KGS', 'KHR', 'KMF', 'KPW', 'KRW', 'KWD', 'KYD',
                    'KZT', 'LAK', 'LBP', 'LKR', 'LRD', 'LSL', 'LYD', 'MAD', 'MDL', 'MGA', 'MKD',
                    'MNT', 'MOP', 'MUR', 'MVR', 'MWK', 'MXN', 'MXV', 'MYR', 'MZN', 'NAD', 'NGN',
                    'NIO', 'NOK', 'NPR', 'NZD', 'OMR', 'PAB', 'PEN', 'PGK', 'PHP', 'PKR', 'PLN',
                    'PYG', 'QAR', 'RON', 'RUB', 'RWF', 'SAR', 'SBD', 'SCR', 'SDG', 'SEK', 'SGD',
                    'SHP', 'SLL', 'SOS', 'SRD', 'SVC', 'SYP', 'SZL', 'THB', 'TJS', 'TND', 'TOP',
                    'TRY', 'TTD', 'TWD', 'TZS', 'UAH', 'UGX', 'USD', 'USN', 'UYU', 'UZS', 'VEF',
                    'VND', 'VUV', 'WST', 'XAF', 'XCD', 'XOF', 'XPF', 'YER', 'ZAR', 'ZWL'
                );
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID:
                $allowedCurrencies = array('EUR');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID:
                $allowedCurrencies = array(
                    'CHF', 'AUD', 'BGN', 'BRL', 'CAD', 'CZK', 'DKK', 'EUR',
                    'GBP', 'HKD', 'HUF', 'ILS', 'INR', 'KRW', 'NOK', 'NZD', 'PLN', 'RON', 'RUB', 'SEK',
                    'SGD', 'THB', 'TRY', 'USD', 'ZAR'
                );
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT_ID:
                $allowedCurrencies = array('EUR');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY_ID:
                $allowedCurrencies = array('PLN');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS_ID:
                $allowedCurrencies = array('EUR');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL_ID:
                $allowedCurrencies = array('EUR');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID:
                $allowedCurrencies = array('CHF', 'EUR', 'GBP', 'USD');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID:
                $allowedCurrencies = array('EUR', 'DKK', 'NOK', 'SEK', 'CHF', 'GBP');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT_ID:
                $allowedCurrencies = array('EUR', 'CHF', 'CZK', 'PLN' , 'GBP', 'HUF');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID:
                $allowedCurrencies = array('CHF', 'EUR');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID:
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID:
                $allowedCurrencies = array(
                    'CHF', 'AED', 'ARS', 'AUD', 'AZN', 'BGN', 'BHD', 'BRL',
                    'BYN', 'CAD', 'CLP', 'CNY', 'COP', 'CZK', 'DKK', 'DZD', 'EGP', 'EUR', 'GBP', 'GEL',
                    'HKD', 'HUF', 'IDR', 'ILS', 'INR', 'ISK', 'JOD', 'JPY', 'KRW', 'KWD', 'KZT', 'LKR',
                    'MAD', 'MXN', 'MYR', 'NGN', 'NOK', 'NZD', 'OMR', 'PHP', 'PKR', 'PLN', 'QAR', 'RON',
                    'RSD', 'RUB', 'SAR', 'SEK', 'SGD', 'THB', 'TND', 'TRY', 'TWD', 'TZS', 'UAH', 'USD', 'VND', 'ZAR'
                );
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE_ID:
                $allowedCurrencies = array('CHF');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL_ID:
                $allowedCurrencies = array(
                    'CHF', 'AUD', 'CAD', 'CZK', 'DKK', 'EUR', 'GBP', 'HKD',
                    'HUF', 'ILS', 'JPY', 'MXN', 'NOK', 'NZD', 'PLN', 'SEK', 'SGD', 'THB', 'USD'
                );
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT_ID:
                $allowedCurrencies = array('EUR');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD_ID:
                $allowedCurrencies = array('CHF');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE_ID:
                $allowedCurrencies = array('CHF');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT_ID:
                $allowedCurrencies = array('CHF');
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY_ID:
                $allowedCurrencies = array(
                    'CHF', 'AED', 'ALL', 'AMD', 'ARS', 'AUD', 'AZN', 'BDT',
                    'BGN', 'BHD', 'BIF', 'BOB', 'BRL', 'BWP', 'BYN', 'CAD', 'CLP', 'COP', 'CZK', 'DKK',
                    'DZD', 'EGP', 'EUR', 'FJD', 'GBP', 'GEL', 'GMD', 'HKD', 'HUF', 'ILS', 'INR', 'IQD',
                    'ISK', 'JOD', 'JPY', 'KES', 'KRW', 'KWD', 'KZT', 'LBP', 'LKR', 'LRD', 'MAD', 'MKD',
                    'MXN', 'MYR', 'NGN', 'NOK', 'NPR', 'NZD', 'OMR', 'PHP', 'PKR', 'PLN', 'PYG', 'QAR',
                    'RON', 'RSD', 'RUB', 'SAR', 'SEK', 'SGD', 'SYP', 'THB', 'TND', 'TRY', 'TWD', 'UAH',
                    'USD', 'VND', 'XAF', 'XOF', 'YER', 'ZAR'
                );
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS_ID:
                $allowedCurrencies = array('CHF');
                break;
            default:
                $allowedCurrencies = array();
        }
        return $allowedCurrencies;
    }

    /**
     * Get payment means of payment method
     */
    private function getPaymentMeansWallets($paymentMethod): array
    {
        switch ($paymentMethod) {
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY_ID:
                $paymentMeans = [];
                $wallets = [SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID:
                $paymentMeans = [
                    SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS,
                    SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT,
                    SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB,
                    SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB,
                    SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO,
                    SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD,
                    SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA,
                ];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY_ID:
                $paymentMeans = [];
                $wallets = [SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS];
                $wallets = [];
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT_ID:
                $paymentMeans = [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT];
                $wallets = [];
                break;
            default:
                $paymentMeans = [];
                $wallets = [];
                break;
        }
        return [$paymentMeans, $wallets];
    }

    /**
     * Get name and description of payment
     */
    private function getNameDescriptionTranslation($paymentMethod): array
    {
        switch ($paymentMethod) {
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY_ID:
                $name = 'Alipay';
                $desc = 'Alipay (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID:
                $name = 'American Express';
                $desc = 'American Express (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY_ID:
                $name = 'Apple Pay';
                $desc = 'Apple Pay (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID:
                $name = 'Bancontact';
                $desc = 'Bancontact (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID:
                $name = 'Credit Card';
                $desc = 'Credit Card (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID:
                $name = 'Diners Club/Discover';
                $desc = 'Diners Club/Discover (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT_ID:
                $name = 'SEPA Direct Debit';
                $desc = 'SEPA Direct Debit (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS_ID:
                $name = 'eps';
                $desc = 'eps (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY_ID:
                $name = 'Przelewy24';
                $desc = 'Przelewy24 (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY_ID:
                $name = 'Google Pay';
                $desc = 'Google Pay (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL_ID:
                $name = 'iDEAL';
                $desc = 'iDEAL (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID:
                $name = 'JCB';
                $desc = 'JCB (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID:
                $name = 'Klarna';
                $desc = 'Klarna (Saferpay). The checkout total and the amount deducting from
                 your account may have minute difference in some scenarios.';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT_ID:
                $name = 'Sofort';
                $desc = 'Sofort (Saferpay).';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID:
                $name = 'Maestro';
                $desc = 'Maestro (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID:
                $name = 'Mastercard';
                $desc = 'Mastercard (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE_ID:
                $name = 'myONE';
                $desc = 'myONE (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT_ID:
                $name = 'paydirekt';
                $desc = 'paydirekt (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL_ID:
                $name = 'Paypal';
                $desc = 'Paypal (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD_ID:
                $name = 'PostFinance Card';
                $desc = 'PostFinance Card (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE_ID:
                $name = 'PostFinance E-Finance';
                $desc = 'PostFinance E-Finance (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT_ID:
                $name = 'TWINT';
                $desc = 'TWINT (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY_ID:
                $name = 'UnionPay';
                $desc = 'UnionPay (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID:
                $name = 'Visa';
                $desc = 'Visa (Saferpay)';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS_ID:
                $name = 'WL Crypto Payments';
                $desc = 'WL Crypto Payments (Saferpay)';
                break;
            default:
                $name = '';
                $desc = '';
                break;
        }
        return [$name, $desc];
    }

    /**
     * Install all the payment methods initialized
     */
    public function install(InstallContext $installContext): void
    {
        $context = $installContext->getContext();
        foreach ($this->paymentMethods as $eachPaymentMethod) {
            $this->createPaymentMethodIfNotExists($eachPaymentMethod, $context);
        }
    }

    /**
     * Add plugin logo
     */
    private function addMediaForPaymentMethod(array $paymentMethod, Context $context)
    {
        $criteria = new Criteria([$paymentMethod['id']]);
        $criteria->addAssociation('media');
        /* @var PaymentMethodEntity $paymentMethodEntity */
        $paymentMethodEntity = $this->paymentMethodRepository->search($criteria, $context)->first();
        if ($paymentMethodEntity === null) {
            return;
        }
        switch ($paymentMethod['id']):
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY_ID:
                $fileName = 'Alipay';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID:
                $fileName = 'AMEX';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID:
                $fileName = 'Bancontact';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID:
                $fileName = 'credit_debit';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID:
                $fileName = 'diners';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT_ID:
                $fileName = 'elv';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS_ID:
                $fileName = 'eps';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY_ID:
                $fileName = 'przelewy24';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL_ID:
                $fileName = 'Ideal';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID:
                $fileName = 'jcb';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID:
                $fileName = 'klarna';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID:
                $fileName = 'maestro';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID:
                $fileName = 'mastercard';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE_ID:
                $fileName = 'myone';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL_ID:
                $fileName = 'paypal';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT_ID:
                $fileName = 'paydirekt';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD_ID:
                $fileName = 'postfinance';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE_ID:
                $fileName = 'postfinance_efinance';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT_ID:
                $fileName = 'twint';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY_ID:
                $fileName = 'unionpay';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID:
                $fileName = 'Visa';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS_ID:
                $fileName = 'cryptopayments';
                break;
            case SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY_ID:
                $fileName = 'Apple_Pay';
                break;
            case SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY_ID:
                $fileName = 'googlepay';
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT_ID:
                $fileName = 'sofort';
                break;
            default:
                $fileName = null;
        endswitch;
        if ($fileName === null) {
            return;
        }
        $paymentMethodMediaFile = $this->getMediaFile($fileName);
        if (!$paymentMethodMediaFile) {
            return;
        }
        $savedFileName = \sprintf('saferpay_shopware_%s', $fileName);

        $this->fileSaver->persistFileToMedia(
            $paymentMethodMediaFile,
            $savedFileName,
            $this->getMediaId($savedFileName, $paymentMethodEntity, $context),
            $context
        );
    }

    /**
     * Reads the media file in plugin to be uploaded into the payment method
     */
    private function getMediaFile(string $fileName): MediaFile
    {
        $filePath = \sprintf('%s/%s/%s.svg', \dirname(__DIR__, 3), self::PAYMENT_METHOD_MEDIA_DIR, $fileName);

        return new MediaFile(
            $filePath,
            \mime_content_type($filePath) ?: '',
            \pathinfo($filePath, \PATHINFO_EXTENSION),
            \filesize($filePath) ?: 0
        );
    }

    /**
     * 1. Checks if media id of the payment method is already there
     * 2. If not a new uuid is created for the media
     * 3. Updates the default folder id for media upload
     * 4. Returns mediaId
     */
    private function getMediaId(string $fileName, PaymentMethodEntity $paymentMethod, Context $context): string
    {
        $media = $paymentMethod->getMedia();
        if ($media !== null && $media->getFileName() === $fileName) {
            return $media->getId();
        }

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('fileName', $fileName));
        $mediaId = $this->mediaRepository->searchIds($criteria, $context)->firstId();

        if ($mediaId === null) {
            $mediaId = Uuid::randomHex();
        }

        $this->paymentMethodRepository->update(
            [[
                'id' => $paymentMethod->getId(),
                'media' => [
                    'id' => $mediaId,
                    'mediaFolderId' => $this->getMediaDefaultFolderId($context),
                ],
            ]],
            $context
        );

        return $mediaId;
    }

    /**
     * Each type of entity has a specific folder as default folder for media uploads.
     * This function checks for the default folder for payment_method entity's media upload
     */
    private function getMediaDefaultFolderId(Context $context): ?string
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter(
            'media_folder.defaultFolder.entity',
            $this->paymentMethodRepository->getDefinition()->getEntityName()
        ));
        $criteria->addAssociation('defaultFolder');
        $criteria->setLimit(1);

        return $this->mediaFolderRepository->searchIds($criteria, $context)->firstId();
    }

    /**
     * 1. Checks if payment method already exists
     * 2. Returns if already exists
     * 3. Create new payment method
     */
    private function createPaymentMethodIfNotExists(array $paymentMethod, Context $context)
    {
        $this->paymentMethodRepository->upsert([$paymentMethod], $context);
        $this->addMediaForPaymentMethod($paymentMethod, $context);
    }

    /**
     * Uninstall all payment methods
     */
    public function uninstall(UninstallContext $uninstallContext): void
    {
        $this->deactivatePaymentMethods($uninstallContext->getContext());
    }

    /**
     * Deactivate all payment methods
     */
    public function deactivate(DeactivateContext $deactivateContext): void
    {
        $this->deactivatePaymentMethods($deactivateContext->getContext());
    }

    /**
     * 1. Gets context from deactivateContext
     * 2. Retrieves all paymentMethods
     * 3. Set active false for all payment method and update
     */
    private function deactivatePaymentMethods(Context $context): void
    {
        $paymentMethodIds = $this->fetchPaymentMethodIds($context);
        if (!$paymentMethodIds) {
            return;
        }
        $paymentMethods = [];
        foreach ($paymentMethodIds as $id) {
            $paymentMethods[] = [
                'id' => $id,
                'active' => false
            ];
        }

        $this->paymentMethodRepository->update($paymentMethods, $context);
    }

    /**
     * Get all the payment methods installed
     */
    private function fetchPaymentMethodIds(Context $context): ?array
    {
        $paymentCriteria = (new Criteria())->addFilter(
            new EqualsFilter('handlerIdentifier', SaferpayPaymentHandler::class)
        );

        return $this->paymentMethodRepository->searchIds($paymentCriteria, $context)->getIds();
    }
}
