<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Service;

use Shopware\Core\Checkout\Payment\Aggregate\PaymentMethodTranslation\PaymentMethodTranslationEntity;
use Shopware\Core\Content\Mail\Service\AbstractMailService;
use Shopware\Core\Defaults;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Validation\DataBag\DataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;

class EmailService
{
    private AbstractMailService $mailService;
    private EntityRepository $paymentMethodTranslationRepository;

    /**
     * Initiate the constructor parameters
     */
    public function __construct(AbstractMailService $mailService, EntityRepository $paymentMethodTranslationRepository)
    {
        $this->mailService = $mailService;
        $this->paymentMethodTranslationRepository = $paymentMethodTranslationRepository;
    }

    /**
     * Send payment confirmation mail to merchant email addresses
     */
    public function sendMerchantMails($emailData, $salesChannelId, $context)
    {
        $recipients = [];
        foreach ($emailData['recipients'] as $eachRecipient) {
            $recipients[$eachRecipient] = $eachRecipient;
        }
        $message = '<p>Dear Customer,</p><p>The following authorization has been completed successfully</p>';
        $liabilityInfo = "";
        if ($emailData['liabilityShiftStatus'] === 'Yes') {
            $liabilityInfo .= $emailData['liabilityShiftStatus'] . ' (' . $emailData['liableEntity'] . ')';
        } else {
            $liabilityInfo .= $emailData['liabilityShiftStatus'];
        }
        $message .= '<hr><table>
            <tr><td><b>Amount</b></td><td>' . $emailData['amount'] . ' ' . $emailData['currencyCode'] . '</td></tr>
            <tr><td><b>Date</b></td><td>' . $emailData['date'] . '</td></tr>
            <tr><td><b>Transaction ID</b></td><td>' . $emailData['transactionId'] . '</td></tr>
            <tr><td><b>Reference Number</b></td><td>' . $emailData['referenceNumber'] . '</td></tr>
            <tr><td><b>Terminal Number</b></td><td>' . $emailData['terminalNumber'] . '</td></tr>
            <tr><td><b>Payment Method</b></td><td>' . $emailData['paymentMethod'] . '</td><tr>
            <tr><td><b>Liability Shift</b></td><td>' . $liabilityInfo . '</td></tr>
            <tr><td><b>Description</b></td><td>' . $emailData['paymentDescription'] . '</td></tr>
        </table><hr>';
        $message .= '<p>For more information, please refer Saferpay Backoffice.</p>';
        $this->sendMail(
            $recipients,
            $emailData['salesChannelName'],
            'Payment Successful',
            $message,
            $salesChannelId,
            $context
        );
    }


    /**
     * Send confirmation mail to customer
     */
    public function sendUserConfirmationMail($emailData, $salesChannelId, $context)
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('paymentMethodId', $emailData['paymentId']));
        $criteria->addFilter(new EqualsFilter('languageId', Defaults::LANGUAGE_SYSTEM));
        /**
         * @var PaymentMethodTranslationEntity
         */
        $paymentMethodTranslation = $this->paymentMethodTranslationRepository->search($criteria, $context)->first();
        $customFields = $paymentMethodTranslation->getCustomFields();
        if (
            !empty($customFields) &&
            isset($customFields['saferpay_shopware_payment_customer_confirmation_email']) &&
            $customFields['saferpay_shopware_payment_customer_confirmation_email'] === '1'
        ) {
            $recipients = [];
            $customerEmail = $emailData['order']->getOrderCustomer()->getEmail();
            $recipients[$customerEmail] = $customerEmail;
            $message = '<p>Dear Customer,</p><p>Your payment has been successfully processed.</p>';
            $message .= '<hr><table>
            <tr><td><b>Amount</b></td><td>' . $emailData['amount'] . ' ' . $emailData['currencyCode'] . '</td></tr>
            <tr><td><b>Date</b></td><td>' . $emailData['date'] . '</td></tr>
            <tr><td><b>Transaction ID</b></td><td>' . $emailData['transactionId'] . '</td></tr>
            <tr><td><b>Reference Number</b></td><td>' . $emailData['referenceNumber'] . '</td></tr>
            <tr><td><b>Payment Means</b></td><td>' . $emailData['paymentName'] . '</td></tr>
            <tr><td><b>Card Number</b></td><td>' . $emailData['cardNumber'] . '</td><tr>
            <tr><td><b>Description</b></td><td>' . $emailData['paymentDescription'] . '</td></tr>
        </table><hr>';
            $message .= '<p>For more information, please refer Saferpay Backoffice.</p>';
            $this->sendMail(
                $recipients,
                $emailData['salesChannelName'],
                'Thank you for your payment',
                $message,
                $salesChannelId,
                $context
            );
        }
    }

    /**
     * Method for sending an email notification
     *
     * @param array $recipients
     * @param string $senderName
     * @param string $subject
     * @param string $messageHtml
     * @param SalesChannelContext $salesChannelContext
     */
    public function sendMail(
        array $recipients,
        string $senderName,
        string $subject,
        string $messageHtml,
        String $salesChannelId,
        Context $context
    ) {
        $data = new DataBag();

        //basic e-mail data
        $data->set('recipients', $recipients);     //format: ['email address' => 'recipient name']
        $data->set('senderName', $senderName);
        $data->set('subject', $subject);
        $data->set('contentHtml', $messageHtml);
        $data->set('contentPlain', strip_tags($messageHtml));

        //set sales channel context
        $data->set('salesChannelId', $salesChannelId);

        //send the e-mail
        $this->mailService->send($data->all(), $context, []);
    }
}
