<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Service;

use Saferpay\Shopware\Api\Exception\ApiRequestException;
use Saferpay\Shopware\Api\SaferpayPaymentApiClient;
use Saferpay\Shopware\Core\Content\Saferpay\SaferpayPaymentTransactionEntity;
use Saferpay\Shopware\Core\Content\Saferpay\SaferpayTransactionHistoryEntity;
use Saferpay\Shopware\SaferpayShopware;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Checkout\Payment\Aggregate\PaymentMethodTranslation\PaymentMethodTranslationEntity;
use Shopware\Core\Checkout\Payment\PaymentException;
use Shopware\Core\Checkout\Payment\PaymentMethodEntity;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Shopware\Core\Defaults;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\System\Currency\CurrencyEntity;
use Shopware\Core\System\SalesChannel\SalesChannelEntity;
use Shopware\Core\System\StateMachine\Aggregation\StateMachineTransition\StateMachineTransitionActions;
use Shopware\Core\System\StateMachine\Exception\IllegalTransitionException;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Contracts\Translation\TranslatorInterface;
use Symfony\Component\DependencyInjection\Container as DIC;
use Shopware\Core\System\StateMachine\StateMachineRegistry;
use Shopware\Core\System\StateMachine\Transition;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStates;

class OrderTransactionAlreadyHandledException extends \RuntimeException {}

class EndofPaymentProcessException extends \RuntimeException {}

class PaymentService extends PaymentServiceHelper
{
    public const TRANSACTION_TEXT = 'Transaction';
    private OrderTransactionStateHandler $orderTransactionStateHandler;
    private EntityRepository $orderTransactionRepository;
    private EntityRepository $paymentMethodRepository;
    private EntityRepository $paymentMethodTranslationRepository;
    private RouterInterface $router;
    private TranslatorInterface $translator;
    public PaymentTransactionSaveService $paymentTransactionSaveService;
    private EntityRepository $currencyRepository;
    private LoggerService $logger;
    private EntityRepository $orderTransactionCaptureRefundRepository;
    private EntityRepository $orderTransactionCaptureRepository;
    private RequestStack $requestStack;
    private EntityRepository $orderRepository;
    public EntityRepository $productRepository;
    public EntityRepository  $categoryTranslationRepository;
    private DIC $container;
    private StateMachineRegistry $stateMachineRegistry;
    private $transactionFailCheck;

    /**
     * Initialize the parameters for the class
     */
    public function __construct(
        DIC $container,
        OrderTransactionStateHandler $orderTransactionStateHandler,
        LoggerService $logger,
        PaymentTransactionSaveService $paymentTransactionSaveService,
        StateMachineRegistry $stateMachineRegistry
    ) {
        $this->container = $container;
        $this->orderTransactionStateHandler = $orderTransactionStateHandler;
        $this->orderTransactionRepository = $this->container->get('order_transaction.repository');
        $this->paymentMethodRepository = $this->container->get('payment_method.repository');
        $this->paymentMethodTranslationRepository = $this->container->get('payment_method_translation.repository');
        $this->router = $this->container->get('router');
        $this->translator = $this->container->get('translator');
        $this->paymentTransactionSaveService = $paymentTransactionSaveService;
        $this->currencyRepository = $this->container->get('currency.repository');
        $this->orderTransactionCaptureRefundRepository = $this->container
            ->get('order_transaction_capture_refund.repository');
        $this->orderTransactionCaptureRepository = $this->container->get('order_transaction_capture.repository');
        $this->orderRepository = $this->container->get('order.repository');
        $this->logger = $logger;
        $this->requestStack = $this->container->get('request_stack');
        $this->productRepository = $this->container->get('product.repository');
        $this->categoryTranslationRepository = $this->container->get('category.repository');
        $this->stateMachineRegistry = $stateMachineRegistry;
    }

    /**
     * 1. Checks the order number validity
     * 2. Checks if payment method exists, if not tries to get it from payment id
     * 3. Gets custom fields of payment method
     * 4. Gets payment means set in the plugin
     * 5. Gets wallets set in the plugin
     * 6. Creates api client for sales channel
     * 7. Generates notify token
     * 8. Prepares delivery address
     * 9. Gets delivery country subdivision code
     * 10. Generates response for payment initialization page
     * 11. Calls the function to save the payment transaction and transaction history
     * 12. Returns the redirect url from the response
     *
     * @throws PaymentException
     */
    public function initializePayment(
        OrderTransactionEntity $orderTransaction,
        OrderEntity $order,
        string $returnUrl,
        SalesChannelEntity $salesChannel,
        Context $context
    ): string {
        $orderNumber = $order->getOrderNumber();
        if (!$orderNumber) {
            throw PaymentException::invalidTransaction(
                'Order Number is missing'
            );
        }

        $paymentMethod = $this->getPaymentMethod($orderTransaction, $context);

        $paymentMethodCustomFields = $this->getPaymentMethodCustomFields($paymentMethod, $context);

        $paymentMeans = $paymentMethodCustomFields['saferpay_shopware_payment_means'] ?? [];
        if (!is_array($paymentMeans)) {
            $paymentMeans = [];
        }

        $wallets = $paymentMethodCustomFields['saferpay_shopware_wallets'] ?? [];
        if (!is_array($wallets)) {
            $wallets = [];
        }

        $customerConfirmationSettings =
            $paymentMethodCustomFields['saferpay_shopware_payment_customer_confirmation_email'] ?? '0';
        $customerAddressSettings = $paymentMethodCustomFields['saferpay_shopware_payment_customer_address'] ?? 'none';

        $preAuthSetting = $paymentMethodCustomFields['saferpay_shopware_pre_authorizations'] ?? '0';

        $paymentApiClient = $this->createSaferpayPaymentApiClientForSalesChannel($salesChannel);

        $notifyToken = $this->generateNotifyToken();
        $billingAddress = $this->getOrderBillingAddress($order);
        $deliveryAddress = $this->getOrderShippingAddress($order);

        $customerSelectedIdealBank = $order->getCustomFieldsValue('customer_selected_ideal_bank');
        $orderData = array();
        $currentLang = $this->requestStack->getSession()->get('currentLangId');
        if ($currentLang === '' || $currentLang === null) {
            $currentLang = $context->getLanguageId();
        }
        if (!$currentLang || $currentLang === "") {
            $this->requestStack->getSession()->set('currentLangId', $orderTransaction->getOrder()->getLanguageId());
        }
        if ($paymentMethod->getId() === SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID) {
            $orderData = $this->buildOrderData(
                $order,
                $orderData,
                $this->productRepository,
                $this->categoryTranslationRepository,
                $currentLang
            );
        }

        $currency = $order->getCurrency();
        if (!$currency) {
            /**
             * @var CurrencyEntity
             */
            $currency = $this->currencyRepository->search(new Criteria([$order->getCurrencyId()]), $context)->first();
        }
        $cardHolderNameDisplay = null;
        if ($paymentMethod->getId() !== SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID) {
            if (
                isset($paymentMethodCustomFields['saferpay_shopware_holder_name_field']) &&
                $paymentMethodCustomFields['saferpay_shopware_holder_name_field'] == 'yes'
            ) {
                $cardHolderNameDisplay = 'MANDATORY';
            } else {
                $cardHolderNameDisplay = 'NONE';
            }
        }
        $payloadData = array(
            'orderNumber' => $orderNumber,
            'paymentMethods' => $paymentMeans,
            'wallets' => $wallets,
            'totalAmount' => $orderTransaction->getAmount()->getTotalPrice(),
            'currencyCode' => $currency->getIsoCode(),
            'customerEmail' => $order->getOrderCustomer()->getEmail(),
            'customerId' => $order->getOrderCustomer()->getId(),
            'deliveryAddress' => $deliveryAddress,
            'billingAddress' => $billingAddress,
            'returnUrl' => $returnUrl,
            'successNotifyUrl' => $this->buildNotifyUrl($orderTransaction, $notifyToken, 'success'),
            'failNotifyUrl' => $this->buildNotifyUrl($orderTransaction, $notifyToken, 'failure'),
            'description' => $this->buildPaymentDescription($order, $salesChannel),
            'merchantEmails' => $this->paymentTransactionSaveService->buildMerchantEmails($salesChannel),
            'theme' => $this->paymentTransactionSaveService->getPluginConfig('theme', $salesChannel) ?: 'DEFAULT',
            'configSet' => $this->paymentTransactionSaveService
                ->getPluginConfig('paymentPageConfigName', $salesChannel),
            'customerConfirmationEmailSettings' => $customerConfirmationSettings,
            'customerAddressSettings' => $customerAddressSettings,
            'preAuthSetting' => $preAuthSetting,
            'customerSelectedIdealBank' => $customerSelectedIdealBank,
            'currentPaymentMethodId' => $paymentMethod->getId(),
            'orderData' => $orderData,
            'saferpayFieldsToken' => $order->getCustomFieldsValue('saferpayFieldsToken'),
            'HolderName' => $cardHolderNameDisplay
        );
        if ($this->checkForceAuthenticationPaymentMethod($paymentMethod->getId())) {
            $payloadData['forceAuthentication'] = $this->paymentTransactionSaveService
                ->getPluginConfig('forceAuthentication', $salesChannel) ?: 'no';
        }
        $isWidgetAuthorization = false;
        if ($order->getCustomFieldsValue('save_card_to_account') === 'on') {
            $payloadData['isSaveCard'] = true;
        }
        if (
            isset($paymentMethodCustomFields['saferpay_shopware_authorization_method']) &&
            $paymentMethodCustomFields['saferpay_shopware_authorization_method'] === 'widgetAuthorization'
        ) {
            $isWidgetAuthorization = true;
        }
        if ($isWidgetAuthorization) {
            $payloadData['cssUrl'] =  $this->paymentTransactionSaveService
                ->getPluginConfig('paymentPageCssUrl', $salesChannel);
            $payloadData['verificationCodeMandatory'] =  $this->paymentTransactionSaveService
                ->getPluginConfig('verificationCodeMandatory', $salesChannel);
            $payloadData['usePreviousSavedCard'] = $order->getCustomFieldsValue('customer_selected_alias');
            $response = $paymentApiClient->transactionInitialize($payloadData);
        } elseif ($order->getCustomFieldsValue('customer_selected_alias')) {
            $payloadData['usePreviousSavedCard'] = $order->getCustomFieldsValue('customer_selected_alias');
            $response = $paymentApiClient->transactionAuthorizeDirect($payloadData);
        } else {
            $response = $paymentApiClient->initializePaymentPage($payloadData);
        }
        $testMode = $this->paymentTransactionSaveService->getPluginConfig('operationMode', $salesChannel) !== 'live';
        $paymentTransactionData = array(
            'response' => $response,
            'orderTransaction' => $orderTransaction,
            'notifyToken' => $notifyToken,
            'saferpayCustomerId' => $this->paymentTransactionSaveService->getPluginConfig($testMode ?
                'customerIdTest' : 'customerIdLive', $salesChannel),
            'transactionMode' => $testMode ? false : true,
            'customerId' => $order->getOrderCustomer()->getCustomerId(),
            'authorizationMethod' => $isWidgetAuthorization ? 'WidgetAuthorization' : 'PaymentPage',
            'preAuth' => $preAuthSetting === '0' ? false : true
        );
        $this->paymentTransactionSaveService->createSaferpayPaymentTransaction(
            $paymentTransactionData,
            $context
        );
        $responseToken = "";
        if (isset($response->Token)) {
            $responseToken = $response->Token;
        }
        $this->persistSaferpayToken($orderTransaction->getId(), $responseToken, $notifyToken, $context);

        if ($this->checkIfAuthorizeDirectTransaction($orderTransaction->getId(), $context)) {
            return $returnUrl;
        }

        return $this->getInitializeReturn($response, $isWidgetAuthorization, $returnUrl, $orderTransaction->getId());
    }

    /**
     * 1. Checks the current state of payment
     * 2. Checks the token, notifyToken in order, if not found Exception is thrown
     * 3. Creates api client for sales channel
     * 4. Order is cancelled if the transaction status is cancelled
     * 5. If there is error in getting the payment page, Exception is thrown with error name
     * 6. If the status of the transaction is not Captured or Authorized, Exception is thrown
     * 7. If the status is captured, payment is processed as paid, transaction data is persisted through function call
     * 8. Gets the payment method, payment method custom fields
     * 9. Gets liability shift behavior, if no value found, transaction is cancelled and Exception is thrown
     * 10. Tries to capture transaction from api client, generates a pending transaction route
     * 11. If transaction is already captured, Exception is thrown with error
     * 12. If capture response has captured status, transaction is processed as paid
     * 13. If capture response has pending status, transaction is processes as pending
     * 14. If no supported status is got from capture response, Exception is thrown
     *
     * @param OrderTransactionEntity $orderTransaction
     * @param SalesChannelEntity $salesChannel
     * @param Context $context
     * @param string $calledFrom
     * @throws PaymentException
     * @throws ApiRequestException
     * @return void
     *
     */
    public function finalizePayment(
        OrderTransactionEntity $orderTransaction,
        SalesChannelEntity $salesChannel,
        Context $context,
        string $calledFrom
    ): void {
        $checkAuthorizeDirect = $this->checkIfAuthorizeDirectTransaction($orderTransaction->getId(), $context);
        if ($checkAuthorizeDirect && is_string($checkAuthorizeDirect)) {
            $this->manualCapture($checkAuthorizeDirect, $context);
            return;
        }

        $state = $orderTransaction->getStateMachineState();
        $transactionId = $orderTransaction->getId();
        if ($state == null) {
            $orderTransactionRepository = $this->container->get('order_transaction.repository');
            $criteria = new Criteria([$orderTransaction->getId()]);
            $criteria->addAssociation('stateMachineState');
            $criteria->addAssociation('order');
            $orderTransaction = $orderTransactionRepository->search($criteria, $context)->first();
            $state = $orderTransaction->getStateMachineState();
        }

        $token = $orderTransaction->getCustomFields()[SaferpayShopware::CUSTOM_FIELD_SAFERPAY_TOKEN] ?? '';
        $notifyToken = $orderTransaction->getCustomFields()[SaferpayShopware::CUSTOM_FIELD_SAFERPAY_NOTIFY_TOKEN] ?? '';
        if ($state->getTechnicalName() == 'reminded') {
            $this->orderTransactionStateHandler->reopen($transactionId, $context);
        }
        try {
            $this->validationChecks(
                $state,
                $token,
                $notifyToken,
                $transactionId,
                $orderTransaction
            );
        } catch (EndofPaymentProcessException $th) {
            return;
        }

        $currentLang = $this->requestStack->getSession()->get('currentLangId');
        if (!$currentLang || $currentLang === "" || $currentLang === null) {
            $currentLang = $context->getLanguageId();
            $this->requestStack->getSession()->set('currentLangId', $orderTransaction->getOrder()->getLanguageId());
        }

        $paymentApiClient = $this->createSaferpayPaymentApiClientForSalesChannel($salesChannel);
        $paymentMethod = $this->handleOrderTransaction($orderTransaction, $context);
        $customFields = $this->getPaymentMethodCustomFields($paymentMethod, $context);

        try {

            list($type, $assertResponse)  = $this->checkIfCancelledTransaction(
                $paymentApiClient,
                $token,
                $transactionId,
                $context,
                $calledFrom,
                $customFields,
                $orderTransaction->getOrderId()
            );

            if ($type == 'CANCELED') {
                return;
            }
        } catch (ApiRequestException $apiRequestException) {

            try {
                $this->handleApiRequestRetryLaterException($apiRequestException, $transactionId, $context, $calledFrom);
            } catch (EndofPaymentProcessException $ch) {
                return;
            }
        }
        $this->checkIfNotCapturedAuthorized($assertResponse, $transactionId);

        /**  below code is commented to resolve the bug 4121 : Auto capture is not working in Twint. */
        // try {
        //     $this->klarnaAndTwintCheck(
        //         $paymentMethod,
        //         $transactionId,
        //         $context,
        //         $calledFrom
        //     );
        // } catch (EndofPaymentProcessException $th) {
        //     return;
        // }

        // checkCaptureStatusAndProcess
        try {
            $this->checkCaptureStatusAndProcess(
                $assertResponse,
                $salesChannel,
                $paymentMethod,
                $transactionId,
                $context,
                $paymentApiClient,
                $calledFrom
            );
        } catch (EndofPaymentProcessException $Invd) {
            return;
        }

        try {
            $this->checkifCapturedElseFail($assertResponse, $transactionId, $context);
        } catch (EndofPaymentProcessException $th) {
            return;
        }

        $captureResponseResult = $this->handleCaptureTransaction(
            $paymentApiClient,
            $assertResponse,
            $orderTransaction,
            $notifyToken
        );
        if ($captureResponseResult) {
            $captureResponse = $captureResponseResult;
        } else {
            return;
        }

        try {
            $this->checkIfPaymentPaidPendingCaptured($captureResponse, $transactionId, $context, $assertResponse);
        } catch (EndofPaymentProcessException $th) {
            return;
        }

        throw PaymentException::invalidTransaction(
            self::TRANSACTION_TEXT . ' ' . $transactionId . ' has an unsupported status after capturing: ' .
                $captureResponse->Status
        );
    }

    /**
     * Function for validation checks
     *
     * @param mixed $state
     * @param mixed $token
     * @param mixed $notifyToken
     * @param mixed $transactionId
     * @param mixed $orderTransaction
     * @return void
     */
    public function validationChecks($state, $token, $notifyToken, $transactionId, $orderTransaction)
    {
        if (
            $state->getTechnicalName() === StateMachineTransitionActions::ACTION_PAID
            || $state->getTechnicalName() === StateMachineTransitionActions::ACTION_PROCESS
            || $state->getTechnicalName() === OrderTransactionStates::STATE_IN_PROGRESS
            || $state->getTechnicalName() == OrderTransactionStates::STATE_UNCONFIRMED
            || $state->getTechnicalName() == 'refunded_partially'
        ) {
            throw new EndofPaymentProcessException();
        }

        if (!is_string($token) || trim($token) === '') {
            $this->logger->error('Error Code : 1004 : No Token in order transaction ' . $transactionId);
            throw PaymentException::invalidTransaction(
                'Saferpay token is not set in order transaction ' . $orderTransaction->getId()
            );
        }

        if (!is_string($notifyToken) || trim($notifyToken) === '') {
            $this->logger->error('Error Code : 1004 : No Notify Token in order transaction ' . $transactionId);
            throw PaymentException::invalidTransaction(
                'Saferpay notify token is not set in order transaction ' . $orderTransaction->getId()
            );
        }
    }


    /**
     * Check payment status of capture response
     */
    public function checkIfPaymentPaidPendingCaptured($captureResponse, $transactionId, $context, $assertResponse)
    {
        if ($captureResponse->Status === 'CAPTURED') {
            try {
                $this->orderTransactionStateHandler->paid($transactionId, $context);
            } catch (IllegalTransitionException $illegalTransitionException) {
                // Silent Fail
                // In api controller, transaction state is already changed asynchronously by notification URI
            }
        }
        if ($captureResponse->Status === 'PENDING') {
            try {
                $this->orderTransactionStateHandler->process($transactionId, $context);
            } catch (IllegalTransitionException $illegalTransitionException) {
                // Silent Fail
                // In api controller, transaction state is already changed asynchronously by notification URI
            }
        }
        if ($captureResponse->Status === 'CAPTURED' || $captureResponse->Status === 'PENDING') {
            $this->persistSaferpayTransactionData(
                $transactionId,
                $assertResponse->Transaction->Id,
                (string) ($captureResponse->CaptureId ?? ''),
                $context
            );

            $this->paymentTransactionSaveService->updateCaptureTransaction(
                $transactionId,
                $captureResponse,
                $assertResponse->Transaction->Id,
                $context
            );
            throw new EndofPaymentProcessException();
        }
    }

    /**
     * checkifCapturedElseFail
     *
     * @param $assertResponse
     * @param $transactionId
     * @param $context
     * @return void
     */
    public function checkifCapturedElseFail($assertResponse, $transactionId, $context)
    {
        if ($assertResponse->Transaction->Status === 'CAPTURED') {
            try {
                $this->orderTransactionStateHandler->paid($transactionId, $context);
            } catch (IllegalTransitionException $illegalTransitionException) {
                // Silent Fail
                // In api controller, transaction state is already changed asynchronously by notification URI
            }

            $this->persistSaferpayTransactionData(
                $transactionId,
                $assertResponse->Transaction->Id,
                (string) ($assertResponse->Transaction->CaptureId ?? ''),
                $context
            );

            throw new EndofPaymentProcessException();
        }
    }

    /**
     * klarnaAndTwintCheck
     *
     * @param $paymentMethod
     * @param $transactionId
     * @param $context
     * @param $calledFrom
     * @return void
     */
    public function klarnaAndTwintCheck($paymentMethod, $transactionId, $context, $calledFrom)
    {
        if (in_array(
            $paymentMethod->getId(),
            [SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID, SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT_ID]
        )  && ($calledFrom === 'paymentHandler')) {
            $this->orderTransactionStateHandler->processUnconfirmed($transactionId, $context);
            throw new EndofPaymentProcessException();
        }
    }
    /**
     * function
     *
     * @param $assertResponse
     * @param $salesChannel
     * @param $paymentMethod
     * @param $transactionId
     * @param $context
     * @param $paymentApiClient
     * @throws PaymentException
     * @throws IllegalTransitionException
     * @return void
     */
    public function checkCaptureStatusAndProcess(
        $assertResponse,
        $salesChannel,
        $paymentMethod,
        $transactionId,
        $context,
        $paymentApiClient,
        $calledFrom
    ) {
        if ($assertResponse->Transaction->Status !== 'CAPTURED') {
            $captureStatus = $this->checkCaptureStatusOfPayment(
                $assertResponse,
                $salesChannel,
                $paymentMethod,
                $context
            );
            if ($captureStatus === self::CAPTURE_STATUS_CANCEL) {
                $paymentApiClient->cancelTransaction($assertResponse->Transaction->Id);
                $this->paymentTransactionSaveService->updateCancelTransactionHistory($transactionId, $context);
                if ($calledFrom === 'iframe') {
                    $this->orderTransactionStateHandler->fail($transactionId, $context);
                }
                throw PaymentException::invalidTransaction('Transaction ' . $transactionId .
                    ' is cancelled due to capture status' . $paymentMethod->getName());
            } elseif ($captureStatus === self::CAPTURE_STATUS_MANUAL_CAPTURE) {
                try {
                    $this->orderTransactionStateHandler->processUnconfirmed($transactionId, $context);
                } catch (IllegalTransitionException $illegalTransitionException) {
                    // Silent Fail
                    // In api controller, transaction state is
                    // already changed asynchronously by notification URI
                }
                throw new EndofPaymentProcessException();
            }
        }
    }
    /**
     * Handle API request exception.
     *
     * @param ApiRequestException $apiRequestException
     * @param string $transactionId
     * @param Context $context
     *
     * @throws ApiRequestException
     */
    private function handleApiRequestRetryLaterException(
        ApiRequestException $apiRequestException,
        string $transactionId,
        Context $context,
        string $calledFrom
    ): void {
        if (!$this->transactionFailCheck) {
            if (
                $apiRequestException->getErrorName() === ApiRequestException::ERROR_NAME_INTERNAL_ERROR &&
                $apiRequestException->getBehaviour() === ApiRequestException::BEHAVIOR_RETRY_LATER
            ) {
                throw new EndofPaymentProcessException();
            } elseif ($apiRequestException->getErrorName() === ApiRequestException::ERROR_NAME_TRANSACTION_ABORT) {
                $this->paymentTransactionSaveService->updateCancelTransactionHistory($transactionId, $context);
            } else {
                if ($calledFrom === 'iframe') {
                    $this->orderTransactionStateHandler->fail($transactionId, $context);
                    $this->paymentTransactionSaveService->updateFailedTransactionHistory($transactionId, $context);
                }
            }
        }
        $this->transactionFailCheck = 1;
        throw $apiRequestException;
    }

    /**
     * checkIfCancelledTransaction
     *
     * @param $paymentApiClient
     * @param $token
     * @param $transactionId
     * @param $context
     * @param $calledFrom
     * @param string $order
     * @return void
     */
    private function checkIfCancelledTransaction(
        $paymentApiClient,
        $token,
        $transactionId,
        $context,
        $calledFrom,
        $customFields,
        $orderId
    ) {
        /**
         * @var OrderEntity
         */
        $order = $this->orderRepository->search(new Criteria([$orderId]), $context)->first();
        if ($calledFrom === 'iframe') {
            $isSaveCard = false;
            $saveToCardCustomField = '';
            $usePreviousCard = false;
            if ($order->getCustomFields()) {
                $saveToCardCustomField = $order->getCustomFieldsValue('save_card_to_account');
                $usePreviousCard = $order->getCustomFieldsValue('customer_selected_alias') ? true : false;
            }
            if (
                !empty($customFields) && isset($customFields['saferpay_shopware_save_card'])
                && $customFields['saferpay_shopware_save_card'] === 'active'
                && $saveToCardCustomField === 'on'
                && !$usePreviousCard
            ) {
                $isSaveCard = true;
            }
            try {
                $assertResponse = $paymentApiClient->transactionAuthorize($token, $isSaveCard);
            } catch (ApiRequestException $apiRequestException) {
                $assertResponse = $apiRequestException->getResponseObject();
                $saveAssertData = [
                    'transactionId' => $transactionId,
                    'response' => $assertResponse,
                    'context' => $context
                ];
                if ($apiRequestException->getErrorName() === ApiRequestException::ERROR_NAME_TRANSACTION_ABORT) {
                    $this->paymentTransactionSaveService->updatePaymentTransaction($saveAssertData);
                    $this->orderTransactionStateHandler->cancel($transactionId, $context);
                    $this->paymentTransactionSaveService->updateCancelTransactionHistory($transactionId, $context);
                    return ['CANCELED', $assertResponse];
                } else {
                    $this->paymentTransactionSaveService->updatePaymentTransaction($saveAssertData);
                    $this->handleApiRequestRetryLaterException($apiRequestException, $transactionId, $context, $calledFrom);
                }
            }
        } else {
            try {
                $assertResponse = $paymentApiClient->assertPaymentPage($token);
            } catch (ApiRequestException $apiRequestException) {
                $assertResponse = $apiRequestException->getResponseObject();
                $saveAssertData = [
                    'transactionId' => $transactionId,
                    'response' => $assertResponse,
                    'context' => $context
                ];
                $this->paymentTransactionSaveService->updatePaymentTransaction($saveAssertData);
                $this->handleApiRequestRetryLaterException($apiRequestException, $transactionId, $context, $calledFrom);
            }
        }
        $saveAssertData = [
            'transactionId' => $transactionId,
            'response' => $assertResponse,
            'context' => $context
        ];
        $this->paymentTransactionSaveService->updatePaymentTransaction($saveAssertData);
        if ($assertResponse->Transaction->Status === 'CANCELED') {
            $this->orderTransactionStateHandler->cancel($transactionId, $context);
            $this->paymentTransactionSaveService->updateCancelTransactionHistory($transactionId, $context);
            return ['CANCELED', $assertResponse];
        }
        return ['', $assertResponse];
    }

    /**
     * Process the order transaction, including state check, token retrieval, and payment method retrieval.
     *
     * @param OrderTransactionEntity $orderTransaction
     * @param SalesChannelEntity $salesChannel
     * @param Context $context
     *
     * @return PaymentMethodEntity
     *
     * @throws PaymentException
     */
    private function handleOrderTransaction(
        OrderTransactionEntity
        $orderTransaction,
        Context $context
    ): PaymentMethodEntity {
        $this->checkOrderTransactionState($orderTransaction);

        $transactionId = $orderTransaction->getId();
        $token = $orderTransaction->getCustomFields()[SaferpayShopware::CUSTOM_FIELD_SAFERPAY_TOKEN] ?? '';
        $notifyToken = $orderTransaction->getCustomFields()[SaferpayShopware::CUSTOM_FIELD_SAFERPAY_NOTIFY_TOKEN] ?? '';

        $this->validateToken($token, 'Saferpay token', $transactionId);
        $this->validateToken($notifyToken, 'Saferpay notify token', $transactionId);

        return $this->getPaymentMethod($orderTransaction, $context);
    }

    /**
     * Get the payment method for the given order transaction.
     *
     * @param OrderTransactionEntity $orderTransaction
     * @param Context $context
     *
     * @return PaymentMethodEntity
     *
     * @throws PaymentException
     */
    private function getPaymentMethod(OrderTransactionEntity $orderTransaction, Context $context): PaymentMethodEntity
    {
        $paymentMethod = $orderTransaction->getPaymentMethod();

        if (!$paymentMethod) {
            $paymentMethod = $this->paymentMethodRepository
                ->search(new Criteria([$orderTransaction->getPaymentMethodId()]), $context)
                ->first();
        }

        return $paymentMethod;
    }

    /**
     * Get the capture status based on assert response params, backend configurations, payment configurations
     */
    private function checkCaptureStatusOfPayment(
        Object $assertResponse,
        SalesChannelEntity $salesChannel,
        PaymentMethodEntity $paymentMethod,
        Context $context
    ): string {
        $liabilityShiftBehaviour = $this->paymentTransactionSaveService
            ->getPluginConfig('liabilityShiftBehavior', $salesChannel) ?? self::CAPTURE_STATUS_MANUAL_CAPTURE;
        $paymentMethodCustomFields = $this->getPaymentMethodCustomFields($paymentMethod, $context);
        $paymentMethodSettings = $paymentMethodCustomFields['saferpay_shopware_capturing'] ?? '';
        $paymentMethodBehaviour =  $paymentMethodSettings === 'delayed'
            ? self::CAPTURE_STATUS_MANUAL_CAPTURE
            : self::CAPTURE_STATUS_AUTO_CAPTURE;

        if (!$this->check3dsSupportedPaymentMethod($assertResponse)) {
            return $paymentMethodBehaviour;
        }
        if ($liabilityShiftBehaviour == self::CAPTURE_STATUS_AUTO_CAPTURE) {
            return $paymentMethodBehaviour;
        }

        $liability = isset($assertResponse->Liability->LiabilityShift) ?
            $assertResponse->Liability->LiabilityShift : false;
        if ($liability) {
            $captureStatus = $this->getCaptureStatus(
                $paymentMethodBehaviour
            );
        } else {
            $captureStatus = $this->getCaptureStatus(
                $liabilityShiftBehaviour
            );
        }

        return $captureStatus;
    }

    /**
     * 1. Checks the status of the order
     * 2. If not paid status, checks for capture id
     * 3. Creates api client for sales channel
     * 4. Checks for the capture response
     * 5. If response in pending status, returns
     * 6. If response not in captured status, Exception is thrown
     * 7. Transaction is set as paid
     *
     * @param OrderTransactionEntity $orderTransaction
     * @param SalesChannelEntity $salesChannel
     * @param Context $context
     * @throws PaymentException
     * @return void
     */
    public function updatePendingTransaction(
        OrderTransactionEntity $orderTransaction,
        SalesChannelEntity $salesChannel,
        Context $context
    ): void {
        $state = $orderTransaction->getStateMachineState();

        if ($state->getTechnicalName() === StateMachineTransitionActions::ACTION_PAID) {
            return;
        }

        $captureId = $orderTransaction->getCustomFields()[SaferpayShopware::CUSTOM_FIELD_SAFERPAY_CAPTURE_ID] ?? '';
        if (!is_string($captureId) || trim($captureId) === '') {
            throw PaymentException::invalidTransaction(
                'Saferpay capture ID is not set in order transaction ' . $orderTransaction->getId()
            );
        }

        $transactionId = $orderTransaction->getId();
        $paymentApiClient = $this->createSaferpayPaymentApiClientForSalesChannel($salesChannel);
        $assertResponse = $paymentApiClient->assertCapture($captureId);

        if (in_array($assertResponse->Status, ['PENDING', 'CAPTURED'])) {
            $this->paymentTransactionSaveService->updateAssertCaptureTransaction(
                $transactionId,
                $captureId,
                $assertResponse,
                $context
            );
        }

        if ($assertResponse->Status === 'PENDING') {
            return;
        }
        if ($assertResponse->Status !== 'CAPTURED') {
            throw PaymentException::invalidTransaction(
                'Transaction ' . $transactionId . ' has an unsupported status: ' . $assertResponse->Status,
            );
        }

        $this->orderTransactionStateHandler->paid($transactionId, $context);
    }

    /**
     * 1. Finds the order transaction entity
     * 2. Updates the current state of the transaction, token and notify token in order transaction
     */
    protected function persistSaferpayToken(
        string $transactionId,
        string $saferpayToken,
        string $notifyToken,
        Context $context
    ): void {
        $orderTransactionEntity = $this->orderTransactionRepository
            ->search(new Criteria([$transactionId]), $context)
            ->first();

        if (!$orderTransactionEntity instanceof OrderTransactionEntity) {
            throw PaymentException::invalidTransaction(
                'Cannot find order transaction with ID ' . $transactionId
            );
        }

        $context->scope(
            Context::SYSTEM_SCOPE,
            function (Context $context) use ($orderTransactionEntity, $saferpayToken, $notifyToken) {
                $this->orderTransactionRepository->update(
                    [
                        [
                            'id' => $orderTransactionEntity->getId(),
                            'stateId' => $orderTransactionEntity->getStateId(),
                            'customFields' => [
                                SaferpayShopware::CUSTOM_FIELD_SAFERPAY_TOKEN => $saferpayToken,
                                SaferpayShopware::CUSTOM_FIELD_SAFERPAY_NOTIFY_TOKEN => $notifyToken,
                            ],
                        ]
                    ],
                    $context
                );
            }
        );
    }

    /**
     * Updates the transaction id and capture id in custom fields of transaction
     */
    protected function persistSaferpayTransactionData(
        string $transactionId,
        string $saferpayTransactionId,
        string $saferpayCaptureId,
        Context $context
    ): void {
        $this->orderTransactionRepository->update(
            [
                [
                    'id' => $transactionId,
                    'customFields' => [
                        SaferpayShopware::CUSTOM_FIELD_SAFERPAY_TRANSACTION_ID => $saferpayTransactionId,
                        SaferpayShopware::CUSTOM_FIELD_SAFERPAY_CAPTURE_ID => $saferpayCaptureId,
                    ]
                ]
            ],
            $context
        );
    }

    /**
     * 1. Checks if the mode is test or live.
     * 2. Populates the customerId, terminalId, username, password accordingly
     * 3. Creates api client for sales channel using the values
     */
    public function createSaferpayPaymentApiClientForSalesChannel(
        SalesChannelEntity $salesChannel
    ): SaferpayPaymentApiClient {
        $testMode = $this->paymentTransactionSaveService->getPluginConfig('operationMode', $salesChannel) !== 'live';
        $customerId = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'customerIdTest' : 'customerIdLive', $salesChannel);
        $apiVersion = $this->paymentTransactionSaveService->getPluginConfig('saferpayAPIVersion', $salesChannel);
        $terminalId = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'terminalIdTest' : 'terminalIdLive', $salesChannel);
        $username = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'usernameTest' : 'usernameLive', $salesChannel);
        $password = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'passwordTest' : 'passwordLive', $salesChannel);
        $apiBaseUrl = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'testApiBaseUrl' : 'liveApiBaseUrl', $salesChannel);

        return new SaferpayPaymentApiClient(
            $customerId,
            $terminalId,
            $username,
            $password,
            (string) $salesChannel->getTranslation('name'),
            $apiBaseUrl,
            (string) $apiVersion
        );
    }

    /**
     * Gets the name, order number of order
     */
    protected function buildPaymentDescription(OrderEntity $order, SalesChannelEntity $salesChannel): string
    {
        return $salesChannel->getName() . "\n\n" . $this->translator->trans('account.orderNumber') .
            ' ' . $order->getOrderNumber();
    }

    /**
     * Generates the url to notify success/failure of payment
     */
    protected function buildNotifyUrl(
        OrderTransactionEntity $orderTransaction,
        string $notifyToken,
        string $process = ""
    ): string {
        if ($process === 'success') {
            return $this->router->generate('api.saferpay-shopware.notify.success', [
                'transactionId' => $orderTransaction->getId(),
                'notifyToken' => $notifyToken
            ], UrlGeneratorInterface::ABSOLUTE_URL);
        } elseif ($process === 'failure') {
            return $this->router->generate('api.saferpay-shopware.notify.fail', [
                'transactionId' => $orderTransaction->getId(),
                'notifyToken' => $notifyToken
            ], UrlGeneratorInterface::ABSOLUTE_URL);
        } else {
            return $this->router->generate('api.saferpay-shopware.notify.notify', [
                'transactionId' => $orderTransaction->getId(),
                'notifyToken' => $notifyToken
            ], UrlGeneratorInterface::ABSOLUTE_URL);
        }
    }

    /**
     * Gets all the custom fields of payment method
     */
    public function getPaymentMethodCustomFields(PaymentMethodEntity $paymentMethod, Context $context): array
    {
        $customFields = [];
        $langId = $this->requestStack->getSession()->get('currentLangId');
        if ($langId == "") {
            $langId = $context->getLanguageId();
        }
        $paymentMethodCurrentTranslation = $this->paymentMethodTranslationRepository->search(
            (new Criteria())
                ->addFilter(new EqualsFilter('paymentMethodId', $paymentMethod->getId()))
                ->addFilter(new EqualsFilter('languageId', $langId)),
            $context
        )->first();
        if (
            $paymentMethodCurrentTranslation
            && $paymentMethodCurrentTranslation instanceof PaymentMethodTranslationEntity
        ) {
            $customFields = $paymentMethodCurrentTranslation->getCustomFields();
        }
        if (empty($customFields)) {
            /**
             * Else, get the default language system's custom fields
             */
            $paymentMethodDefaultTranslation = $this->paymentMethodTranslationRepository->search(
                (new Criteria())
                    ->addFilter(new EqualsFilter('paymentMethodId', $paymentMethod->getId()))
                    ->addFilter(new EqualsFilter('languageId', Defaults::LANGUAGE_SYSTEM)),
                $context
            )->first();
            if ($paymentMethodDefaultTranslation instanceof PaymentMethodTranslationEntity) {
                $customFields = $paymentMethodDefaultTranslation->getCustomFields();
            }
        }
        return $customFields;
    }

    /**
     * cancel Payments function
     *
     * @param SalesChannelEntity $salesChannel
     * @return bool
     */
    public function cancelPayments(string $paymentTransactionId, Context $context): bool
    {
        try {
            /** @var SaferpayPaymentTransactionEntity */
            $paymentTransaction = $this->paymentTransactionSaveService
                ->getPaymentTransaction($paymentTransactionId, $context);
            $salesChannel = $this->paymentTransactionSaveService
                ->getSalesChannelFromOrderId($paymentTransaction->getOrderId(), $context);
            $paymentApiClient = $this->createSaferpayPaymentApiClientForSalesChannel(
                $salesChannel
            );
            $transactionId = $this->paymentTransactionSaveService
                ->getTransactionIdFromHistory($paymentTransactionId, $context);
            $paymentApiClient->cancelTransaction($transactionId);
            $this->orderTransactionStateHandler->cancel($paymentTransaction->getOrderTransactionId(), $context);
            $this->paymentTransactionSaveService
                ->updateCancelTransactionHistory($paymentTransaction->getOrderTransactionId(), $context);

            $this->stateMachineRegistry->transition(
                new Transition('order', $paymentTransaction->getOrderId(), 'cancel', 'stateId'),
                $context
            );
            return true;
        } catch (\Throwable $th) {
            return false;
        }
    }

    /**
     * Manual Capture
     */
    public function manualCapture(string $paymentTransactionId, Context $context)
    {
        /** @var SaferpayPaymentTransactionEntity */
        $paymentTransaction = $this->paymentTransactionSaveService
            ->getPaymentTransaction($paymentTransactionId, $context);
        $salesChannel = $this->paymentTransactionSaveService
            ->getSalesChannelFromOrderId($paymentTransaction->getOrderId(), $context);
        $paymentApiClient = $this->createSaferpayPaymentApiClientForSalesChannel($salesChannel);
        $transactionId = $this->paymentTransactionSaveService
            ->getTransactionIdFromHistory($paymentTransactionId, $context);
        if ($transactionId) {
            $orderTransactionId = $paymentTransaction->getOrderTransactionId();
            /** @var OrderTransactionEntity */
            $orderTransaction = $this->paymentTransactionSaveService
                ->getOrderTransactionFomId($orderTransactionId, $context);
            $notifyToken = $orderTransaction
                ->getCustomFields()[SaferpayShopware::CUSTOM_FIELD_SAFERPAY_NOTIFY_TOKEN] ?? '';
            if (!is_string($notifyToken) || trim($notifyToken) === '') {
                throw PaymentException::invalidTransaction(
                    'Saferpay notify token is not set in order transaction ' . $orderTransactionId
                );
            }
            try {
                $captureResponse = $paymentApiClient->captureTransaction(
                    $transactionId,
                    $this->router->generate(
                        'api.saferpay-shopware.notify-pending',
                        [
                            'transactionId' => $orderTransactionId,
                            'notifyToken' => $notifyToken
                        ],
                        UrlGeneratorInterface::ABSOLUTE_URL
                    )
                );
            } catch (ApiRequestException $apiRequestException) {
                $this->handleApiRequestException($apiRequestException);
            }

            if ($captureResponse->Status === 'CAPTURED') {
                $this->handlePaidOrderTransaction(
                    $orderTransactionId,
                    $transactionId,
                    $captureResponse->CaptureId ?? '',
                    $context
                );
            }
            if ($captureResponse->Status === 'PENDING') {
                $this->processOrderTransaction(
                    $orderTransactionId,
                    $transactionId,
                    $captureResponse->CaptureId ?? '',
                    $context
                );
            }
            if ($captureResponse->Status === 'CAPTURED' || $captureResponse->Status === 'PENDING') {
                $this->paymentTransactionSaveService->updateCaptureTransaction(
                    $orderTransactionId,
                    $captureResponse,
                    $transactionId,
                    $context
                );
            }
        }
    }

    /**
     * Process order transaction
     */
    private function processOrderTransaction(
        string $orderTransactionId,
        string $transactionId,
        ?string $captureId,
        Context $context
    ): void {
        try {
            $this->orderTransactionStateHandler->process($orderTransactionId, $context);
        } catch (IllegalTransitionException $illegalTransitionException) {
            // Silent Fail
            // In API controller, transaction state is already changed asynchronously by notification URI
        }

        $this->persistSaferpayTransactionData(
            $orderTransactionId,
            $transactionId,
            (string) ($captureId ?? ''),
            $context
        );
    }

    /**
     * Handle Paid transaction
     */
    private function handlePaidOrderTransaction(
        string $orderTransactionId,
        string $transactionId,
        ?string $captureId,
        Context $context
    ): void {
        try {
            $this->orderTransactionStateHandler->paid($orderTransactionId, $context);
        } catch (IllegalTransitionException $illegalTransitionException) {
            // Silent Fail
            // In API controller, transaction state is already changed asynchronously by notification URI
        }

        $this->persistSaferpayTransactionData(
            $orderTransactionId,
            $transactionId,
            (string) ($captureId ?? ''),
            $context
        );
    }

    /**
     * handleCaptureTransaction function
     *
     * @param SaferpayPaymentApiClient $paymentApiClient
     * @param object $assertResponse
     * @param OrderTransactionEntity $orderTransaction
     * @param string $notifyToken
     * @return object
     */
    private function handleCaptureTransaction($paymentApiClient, $assertResponse, $orderTransaction, $notifyToken)
    {
        try {
            return $paymentApiClient->captureTransaction(
                $assertResponse->Transaction->Id,
                $this->router->generate(
                    'api.saferpay-shopware.notify-pending',
                    [
                        'transactionId' => $orderTransaction->getId(),
                        'notifyToken' => $notifyToken
                    ],
                    UrlGeneratorInterface::ABSOLUTE_URL
                )
            );
        } catch (ApiRequestException $apiRequestException) {
            if ($apiRequestException->getErrorName() === ApiRequestException::ERROR_NAME_TRANSACTION_ALREADY_CAPTURED) {
                return 0;
            }

            if (
                $apiRequestException->getErrorName() === ApiRequestException::ERROR_NAME_INTERNAL_ERROR &&
                $apiRequestException->getBehaviour() === ApiRequestException::BEHAVIOR_RETRY_LATER
            ) {
                return 0;
            }

            throw $apiRequestException;
        }
    }

    /**
     * handleRefund function
     *
     * @param array $requestArray
     * @param Context $context
     * @return void
     */
    public function handleRefund(array $requestArray, Context $context)
    {
        $salesChannel = $this->paymentTransactionSaveService
            ->getSalesChannelFromOrderId($requestArray['orderId'], $context);
        $paymentApiClient = $this->createSaferpayPaymentApiClientForSalesChannel($salesChannel);
        /** @var OrderTransactionEntity */
        $orderTransactionEntity = $this->orderTransactionRepository
            ->search(new Criteria([$requestArray['orderTransactionId']]), $context)
            ->first();
        $orderNotifyToken = trim((string)
        ($orderTransactionEntity->getCustomFields()[SaferpayShopware::CUSTOM_FIELD_SAFERPAY_NOTIFY_TOKEN] ?? ''));
        $requestArray['notifyUrl'] = $this->buildNotifyUrl($orderTransactionEntity, $orderNotifyToken);
        $order = $this->orderRepository->search(new Criteria([$requestArray['orderId']]), $context)->first();
        $requestArray['orderId'] = $order->getOrderNumber();
        $refundResponse = $paymentApiClient->refund($requestArray);
        $saveAssertData = [
            'transactionId' => $requestArray['orderTransactionId'],
            'response' => $refundResponse,
            'context' => $context
        ];
        $this->paymentTransactionSaveService->updateTransactionHistoryAfterRefund($saveAssertData);
        switch ($refundResponse->Transaction->Status) {
            case 'AUTHORIZED':
                $status = $paymentApiClient->refundTransactionCapture(
                    [
                        'transactionId' => $refundResponse->Transaction->Id
                    ]
                );
                $saveAssertData = [
                    'transactionId' => $requestArray['orderTransactionId'],
                    'response' => $status,
                    'context' => $context
                ];
                $this->paymentTransactionSaveService->updateTransactionHistoryAfterRefund($saveAssertData);
                $this->manageRefundRelatedDataInShopware($requestArray, $orderTransactionEntity, $context);
                break;
            case 'PENDING':
                $this->manageRefundRelatedDataInShopware($requestArray, $orderTransactionEntity, $context);
                $saveAssertData = [
                    'transactionId' => $requestArray['orderTransactionId'],
                    'response' => $refundResponse,
                    'context' => $context
                ];
                $this->paymentTransactionSaveService->updateTransactionHistoryAfterRefund($saveAssertData);

                break;
            case 'CAPTURED':
                $status = $refundResponse;
                $this->manageRefundRelatedDataInShopware($requestArray, $orderTransactionEntity, $context);
                $saveAssertData = [
                    'transactionId' => $requestArray['orderTransactionId'],
                    'response' => $refundResponse,
                    'context' => $context
                ];
                $this->paymentTransactionSaveService->updateTransactionHistoryAfterRefund($saveAssertData);
                break;
            default:
                $status = $refundResponse;
                break;
        }

        return $status;
    }

    /**
     * function to save Order Transaction Capture Refund
     *
     * @param array $saveArray
     * @param OrderTransactionEntity $orderTransactionEntity
     * @param Context $context
     * @return void
     */
    public function saveOrderTransactionCaptureRefund(
        array $saveArray,
        OrderTransactionEntity $orderTransactionEntity,
        Context $context
    ) {
        $refundAmount = clone $orderTransactionEntity->getAmount();
        $refundAmount->overwrite(
            $refundAmount->getUnitPrice(),
            (float) $saveArray['refundAmount'],
            $refundAmount->getCalculatedTaxes()
        );
        $orderTransactionCapture = $this->orderTransactionCaptureRepository->create([
            [
                'orderTransactionId' => $saveArray['orderTransactionId'],
                'transactionId' => $saveArray['orderTransactionId'],
                'stateId' => $orderTransactionEntity->getStateId(),
                'externalReference' => '',
                'amount' => $refundAmount,
                'customFields' => [],
            ]
        ], $context);


        /** @var EntityWrittenEvent */
        $savedOrderTransactionCaptureArray = !empty($orderTransactionCapture->getEvents()->getElements()) ?
            $orderTransactionCapture->getEvents()->getElements()[0] : [];

        if (!empty($savedOrderTransactionCaptureArray)) {
            /** @var EntityWriteResult */
            $savedOrderTransactionCaptureResultArray = !empty($savedOrderTransactionCaptureArray->getWriteResults()) ?
                $savedOrderTransactionCaptureArray->getWriteResults()[0] : [];
        }

        if (!empty($savedOrderTransactionCaptureResultArray)) {
            $this->orderTransactionCaptureRefundRepository->create(
                [
                    [
                        'captureId' => $savedOrderTransactionCaptureResultArray->getPrimaryKey(),
                        'stateId' => $orderTransactionEntity->getStateId(),
                        'reason' => $saveArray['reasonRefund'],
                        'totalAmount' => $saveArray['refundAmount'],
                        'amount' => $refundAmount
                    ]
                ],
                $context
            );
        }
    }



    /**
     * manage Refund Related Data In Shopware function
     *
     * @param array $requestArray
     * @param OrderTransactionEntity $orderTransactionEntity
     * @param Context $context
     * @return void
     * @throws IllegalTransitionException $illegalTransitionException
     */
    public function manageRefundRelatedDataInShopware(
        array $requestArray,
        OrderTransactionEntity $orderTransactionEntity,
        Context $context
    ) {
        try {
            $this->saveOrderTransactionCaptureRefund($requestArray, $orderTransactionEntity, $context);
            if (round((float) $requestArray['refundAmount']) < round((float) $requestArray['totalAvailableToRefund'])) {
                $this->orderTransactionStateHandler->refundPartially($requestArray['orderTransactionId'], $context);
            } else {
                $this->orderTransactionStateHandler->refund($requestArray['orderTransactionId'], $context);
            }
        } catch (IllegalTransitionException $illegalTransitionException) {
            // Silent Fail
            // In api controller, transaction state is already changed asynchronously by notification URI
        }
    }

    /**
     * handle Refund Notify Url function
     *
     * @param string $orderTransactionId
     * @param string $transactionId
     * @param Context $context
     * @return void
     */
    public function handleRefundNotifyUrl($orderTransactionId, Context $context)
    {

        /** @var OrderTransactionEntity */
        $orderTransactionEntity = $this->orderTransactionRepository
            ->search(new Criteria([$orderTransactionId]), $context)
            ->first();

        /** @var SaferpayTransactionHistoryEntity */
        $paymentTransationHistory = $this->paymentTransactionSaveService->getPaymentTransactionWithOrderTransaction(
            $orderTransactionId,
            $context
        );

        $orderId = $orderTransactionEntity->getOrderId();
        $salesChannel = $this->paymentTransactionSaveService
            ->getSalesChannelFromOrderId($orderId, $context);
        $paymentApiClient = $this->createSaferpayPaymentApiClientForSalesChannel($salesChannel);
        try {
            $status = $paymentApiClient->refundAssertCall([
                'transactionId' => $paymentTransationHistory->getTransactionId(),
                // 'OrderId' => $orderId
            ]);
            $saveAssertData = [
                'transactionId' => $orderTransactionId,
                'response' => $status,
                'context' => $context
            ];

            $this->paymentTransactionSaveService->updateTransactionHistoryAfterRefund($saveAssertData);
        } catch (\Throwable $th) {
            // Fail
        }
    }

    /**
     * Check if an authorize redirect transaction
     */
    private function checkIfAuthorizeDirectTransaction(string $transactionId, Context $context)
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderTransactionId', $transactionId));
        /**
         * @var SaferpayPaymentTransactionEntity
         */
        $paymentTransaction = $this->container->get('saferpay_payment_transaction.repository')
            ->search($criteria, $context)->first();
        if (!$paymentTransaction) {
            return false;
        }
        $historyCriteria = new Criteria();
        $historyCriteria->addFilter(new EqualsFilter('paymentTransactionId', $paymentTransaction->getId()));
        $historyCriteria->addFilter(new EqualsFilter(
            'transactionStatus',
            $this->paymentTransactionSaveService::API_PAYMENT_STATUS_AUTHORIZED_DIRECT
        ));
        /**
         * @var SaferpayTransactionHistoryEntity
         */
        $transactionHistory = $this->container->get('saferpay_transaction_history.repository')
            ->search($historyCriteria, $context)->first();
        return $transactionHistory ? $paymentTransaction->getId() : false;
    }
}
