<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Service;

use Saferpay\Shopware\Api\Exception\ApiRequestException;
use Saferpay\Shopware\SaferpayShopware;
use Shopware\Core\Checkout\Order\Aggregate\OrderAddress\OrderAddressEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderDelivery\OrderDeliveryEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Checkout\Payment\PaymentException;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\System\StateMachine\Aggregation\StateMachineTransition\StateMachineTransitionActions;

class PaymentServiceHelper
{
    public const CAPTURE_STATUS_AUTO_CAPTURE = 'auto-capture';
    public const CAPTURE_STATUS_MANUAL_CAPTURE = 'manual-capture';
    public const CAPTURE_STATUS_CANCEL = 'auto-cancel';

    /**
     * Format price for converting to integer
     */
    protected function priceFormat($price)
    {
        return (int)round(abs($price) * 100);
    }

    /**
     * Generates a random 32 lengthed string as notify token
     */
    protected function generateNotifyToken(): string
    {
        return bin2hex(random_bytes(32));
    }

    /**
     * Check if 3ds supported payment method 
     */
    protected function checkForceAuthenticationPaymentMethod($paymentMethodId)
    {
        if (in_array($paymentMethodId, array(
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID,
            SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY_ID,
            SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY_ID,
            SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID
        ))) {
            return true;
        }
        return false;
    }

    /**
     * Check if the payment method is 3ds supported
     */
    protected function check3dsSupportedPaymentMethod($assertResponse)
    {
        $brandName = isset($assertResponse->PaymentMeans->Brand->PaymentMethod) ?
            $assertResponse->PaymentMeans->Brand->PaymentMethod : null;
        if (!$brandName) {
            return false;
        }
        $allowedPaymentMethods = array(
            'VISA',
            'MASTERCARD',
            'MAESTRO',
            'AMEX',
            'DINERS',
            'UNIONPAY',
            'BANCONTACT',
        );
        if (in_array($brandName, $allowedPaymentMethods)) {
            return true;
        }
        return false;
    }

    /**
     * Build Order Details for Klarna Payment Method
     */
    protected function buildOrderData(
        OrderEntity $order,
        $orderData,
        $productRepository,
        $categoryRepository,
        $currentLang
    ) {
        $orderItems = array();
        $criteria = new Criteria();
        $criteriaTwo = new Criteria();
        $lineItems = $order->getLineItems();
        foreach ($lineItems as $eachItem) {
            /**
             * @var OrderLineItemEntity $eachItem
             */
            switch ($eachItem->getType()) {
                case 'product':
                    $productState = $eachItem->getStates();
                    if (in_array('is-download', $productState)) {
                        $orderType = 'DIGITAL';
                    }
                    if (in_array('is-physical', $productState)) {
                        $orderType = 'PHYSICAL';
                    }
                    $orderId = $eachItem->getProductId();
                    $unitPrice = $this->priceFormat($eachItem->getUnitPrice());
                    $discountAmount = 0;
                    break;
                case 'promotion':
                    $orderType = 'DISCOUNT';
                    $orderId = $eachItem->getPromotionId();
                    $unitPrice = 0;
                    $discountAmount = $this->priceFormat($eachItem->getUnitPrice());
                    break;
                default:
                    break;
            }
            $criteria->addFilter(new EqualsFilter('id', $eachItem->getProductId()));
            $products = $productRepository->search($criteria, Context::createDefaultContext())->getEntities()->first();

            if (count($products->categoryIds) > 0) {
                try {
                    $criteriaTwo->addFilter(new EqualsFilter('id', $products->categoryIds[0]));
                    $criteriaTwo->addAssociation('translations');

                    $category = $categoryRepository->search(
                        $criteriaTwo,
                        Context::createDefaultContext()
                    )->getEntities()->first();
                    $translations = $category->getTranslations();
                    foreach ($translations as $translated) {
                        if ($translated->getLanguageId() == $currentLang) {
                            $categoryName = $translated->getName();
                            if (strlen($categoryName) > 100) {
                                // Truncate $categoryName to the first 100 characters
                                $categoryName = substr($categoryName, 0, 100);
                            }
                        }
                    }
                } catch (\Throwable $th) {
                    // echo $th->getMessage();
                }
            }

            $orderItems[] = [
                'Type' => $orderType,
                'Id' => $orderId,
                'Name' => $eachItem->getLabel(),
                'UnitPrice' => $unitPrice,
                'DiscountAmount' => $discountAmount,
                'Quantity' => $eachItem->getQuantity(),
                'TaxRate' => 0,
                'TaxAmount' => 0,
                'CategoryName' =>  $categoryName ?? '',
            ];
        }
        if ($order->getShippingTotal() > 0) {
            $orderItems[] = [
                'Type' => 'SHIPPINGFEE',
                'Id' => $order->getId(),
                'Name' =>  'Shipping Total',
                'UnitPrice' => $this->priceFormat($order->getShippingTotal()),
                'DiscountAmount' => 0,
                'Quantity' => 1,
                'TaxRate' => 0,
                'TaxAmount' => 0
            ];
        }
        $orderData['Order']['Items'] = $orderItems;

        return $orderData;
    }

    /**
     * checkIfNotCapturedAuthorized
     *
     * @param $assertResponse
     * @param $transactionId
     * @return void
     */
    protected function checkIfNotCapturedAuthorized($assertResponse, $transactionId)
    {
        if (!in_array($assertResponse->Transaction->Status, ['CAPTURED', 'AUTHORIZED'])) {
            throw PaymentException::invalidTransaction('The Transaction ' . $transactionId . ' has an unsupported status: ' .
                $assertResponse->Transaction->Status,);
        }
    }

    /**
     * Get the capture status when liability node is false or true
     */
    protected function getCaptureStatus(
        string $liabilityShiftBehaviour
    ): string {
            return $liabilityShiftBehaviour;
    }

    /**
     * Handle api request exception
     */
    protected function handleApiRequestException(ApiRequestException $apiRequestException): void
    {
        if ($apiRequestException->getErrorName() === ApiRequestException::ERROR_NAME_TRANSACTION_ALREADY_CAPTURED) {
            return;
        }

        if (
            $apiRequestException->getErrorName() === ApiRequestException::ERROR_NAME_INTERNAL_ERROR &&
            $apiRequestException->getBehaviour() === ApiRequestException::BEHAVIOR_RETRY_LATER
        ) {
            return;
        }

        throw $apiRequestException;
    }

    /**
     * Gets the delivery address of the order
     */
    protected function getOrderBillingAddress(OrderEntity $order): array
    {
        $deliveryAddress = $this->findDeliveryAddress($order);

        if (!isset($deliveryAddress)) {
            return [];
        }

        $deliveryCountrySubdivisionCode = $this->getDeliveryCountrySubdivisionCode($deliveryAddress);

        return $this->unsetEmptyValueFromArray([
            'FirstName' => $this->getValueOrDefault($deliveryAddress->getFirstName(), ''),
            'LastName' => $this->getValueOrDefault($deliveryAddress->getLastName(), ''),
            'Street' => $this->getValueOrDefault($deliveryAddress->getStreet(), ''),
            'Zip' => $this->getValueOrDefault($deliveryAddress->getZipcode(), ''),
            'City' => $this->getValueOrDefault($deliveryAddress->getCity(), ''),
            'CountryCode' => $this->getValueOrDefault((string)$deliveryAddress->getCountry()->getIso(), ''),
            'CountrySubdivisionCode' => $this->getValueOrDefault((string)$deliveryCountrySubdivisionCode, ''),
            'Company' => $this->getValueOrDefault((string)$deliveryAddress->getCompany(), ''),
        ]);
    }

    /**
     * Unset any empty value in array
     */
    protected function unsetEmptyValueFromArray($arrayValueFix)
    {
        if (!empty($arrayValueFix)) {
            foreach ($arrayValueFix as $key => $value) {
                if ($value === "") {
                    unset($arrayValueFix[$key]);
                }
            }
        }
        return $arrayValueFix;
    }

    /**
     * Find delivery address
     *
     * @param OrderEntity $order
     * @return mixed|null
     */
    protected function findDeliveryAddress(OrderEntity $order)
    {
        $deliveryAddress = $order->getBillingAddress();

        $addresses = $order->getAddresses();
        if ($addresses) {
            foreach ($addresses as $address) {
                if ($address->getId() !== $order->getBillingAddressId()) {
                    $deliveryAddress = $address;
                    break;
                }
            }
        }

        return $deliveryAddress;
    }

    /**
     * Get delivery subdivision code
     *
     * @param mixed $deliveryAddress
     * @return string
     */
    protected function getDeliveryCountrySubdivisionCode(mixed $deliveryAddress): string
    {
        if (!$deliveryAddress || !$deliveryAddress->getCountryState()) {
            return '';
        }

        $stateCode = $deliveryAddress->getCountryState()->getShortCode();
        if (!$stateCode) {
            return '';
        }

        [$deliveryCountrySubdivisionCode] = array_pad(explode('-', $stateCode, 2), 2, '');

        return $deliveryCountrySubdivisionCode;
    }

    /**
     * Get value or set it as default
     *
     * @param mixed $value
     * @param mixed $default
     * @return mixed
     */
    protected function getValueOrDefault($value, $default)
    {
        return $value !== '' ? $value : $default;
    }


    /**
     * Get shipping address of the order
     */
    protected function getOrderShippingAddress(OrderEntity $order): array
    {
        $firstDelivery = $this->getFirstDelivery($order);

        if (!$firstDelivery) {
            return [];
        }

        $shippingAddress = $firstDelivery->getShippingOrderAddress();
        if (!$shippingAddress) {
            return [];
        }

        $shippingCountrySubdivisionCode = $this->getShippingCountrySubdivisionCode($shippingAddress);

        $shippingArray = [
            'FirstName' => $shippingAddress->getFirstName(),
            'LastName' => $shippingAddress->getLastName(),
            'Street' => $shippingAddress->getStreet(),
            'Zip' => $shippingAddress->getZipcode(),
            'City' => $shippingAddress->getCity(),
            'CountryCode' => (string) $shippingAddress->getCountry()->getIso(),
            'CountrySubdivisionCode' => (string) $shippingCountrySubdivisionCode,
            'Company' => (string) $shippingAddress->getCompany(),
        ];

        return array_filter($shippingArray);
    }

    /**
     * Get the first delivery from the order
     */
    protected function getFirstDelivery(OrderEntity $order): ?OrderDeliveryEntity
    {
        $deliveries = $order->getDeliveries();
        return ($deliveries && $deliveries->count() > 0) ? $deliveries->first() : null;
    }

    /**
     * Get the shipping country subdivision code
     */
    protected function getShippingCountrySubdivisionCode(OrderAddressEntity $shippingAddress): string
    {
        if ($shippingAddress->getCountryState()) {
            $stateCode = $shippingAddress->getCountryState()->getShortCode();
            $stateCodeSegments = explode('-', $stateCode, 2);
        }

        return isset($stateCodeSegments[1]) && $stateCodeSegments[1] ? $stateCodeSegments[1] : '';
    }

    /**
     * Validate that a token is set.
     *
     * @param string $token
     * @param string $tokenType
     * @param string $transactionId
     *
     * @throws PaymentException
     */
    protected function validateToken(string $token, string $tokenType, string $transactionId): void
    {
        if (!is_string($token) || trim($token) === '') {
            throw PaymentException::invalidToken($tokenType . ' is not set in order transaction ' . $transactionId);
        }
    }

    /**
     * Check the order transaction state.
     *
     * @param OrderTransactionEntity $orderTransaction
     *
     * @throws OrderTransactionAlreadyHandledException
     */
    protected function checkOrderTransactionState(OrderTransactionEntity $orderTransaction): void
    {
        $state = $orderTransaction->getStateMachineState();

        if (
            $state->getTechnicalName() === StateMachineTransitionActions::ACTION_PAID
            || $state->getTechnicalName() === StateMachineTransitionActions::ACTION_PROCESS
            || $state->getTechnicalName() === 'in_progress'
        ) {
            throw new OrderTransactionAlreadyHandledException('Order transaction already handled.');
        }
    }

    /**
     * Get the return url for payment method after initialize
     */
    protected function getInitializeReturn(object $response, bool $isWidgetAuthorization, string $returnUrl): string
    {
        $return = '';
        if ($isWidgetAuthorization) {
            if ($response->RedirectRequired) {
                $return = $response->Redirect->RedirectUrl;
            } else {
                $return = $returnUrl;
            }
        } else {
            $return = $response->RedirectUrl;
        }
        return $return;
    }
}
