<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Service;

use DateTime;
use Doctrine\DBAL\Exception as DBALException;
use Exception;
use Saferpay\Shopware\Core\Content\Saferpay\SaferpayCustomerSecureTransactionEntity;
use Saferpay\Shopware\Core\Content\Saferpay\SaferpayPaymentTransactionEntity;
use Saferpay\Shopware\Core\Content\Saferpay\SaferpayTransactionHistoryEntity;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\EntityWriteResult;
use Shopware\Core\Framework\DataAbstractionLayer\Event\EntityWrittenEvent;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsAnyFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\System\Language\LanguageEntity;
use Shopware\Core\System\SalesChannel\SalesChannelEntity;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use Symfony\Component\HttpFoundation\RequestStack;

class PaymentTransactionSaveService
{

    public const API_PAYMENT_STATUS_INITIALIZE    = 'INITIALIZE';
    public const API_PAYMENT_STATUS_AUTHORIZED    = 'AUTHORIZED';
    public const API_PAYMENT_STATUS_AUTHORIZED_DIRECT = 'AUTHORIZED_DIRECT';
    public const API_PAYMENT_STATUS_CAPTURE       = 'CAPTURED';
    public const API_PAYMENT_STATUS_PENDING       = 'PENDING';
    public const API_PAYMENT_STATUS_REFUND_INIT   = 'REFUND INIT';
    public const API_PAYMENT_STATUS_REFUND_ASSERT = 'REFUNDED';
    public const API_PAYMENT_STATUS_CANCEL        = 'CANCEL';
    public const API_PAYMENT_STATUS_FAILED        = 'FAILED';

    private EntityRepository $paymentTransactionRepository;
    private EntityRepository $transactionHistoryRepository;
    private EntityRepository $languageRepository;
    private EntityRepository $orderRepository;
    private EntityRepository $orderTransactionRepository;
    private EmailService $emailService;
    private SystemConfigService $systemConfigService;
    private LoggerService $logger;
    private RequestStack $requestStack;
    private EntityRepository $customerSecureTransactionRepository;

    /**
     * Initialize all the parameters of the service
     */
    public function __construct(
        StorePaymentTransactionSaveService $storePaymentTransactionSaveService,
        EmailService $emailService,
        SystemConfigService $systemConfigService,
        LoggerService $logger
    ) {
        $this->paymentTransactionRepository = $storePaymentTransactionSaveService->paymentTransactionRepository;
        $this->transactionHistoryRepository = $storePaymentTransactionSaveService->transactionHistoryRepository;
        $this->languageRepository           = $storePaymentTransactionSaveService->languageRepository;
        $this->orderRepository              = $storePaymentTransactionSaveService->orderRepository;
        $this->orderTransactionRepository   = $storePaymentTransactionSaveService->orderTransactionRepository;
        $this->emailService                 = $emailService;
        $this->systemConfigService          = $systemConfigService;
        $this->logger                       = $logger;
        $this->requestStack = $storePaymentTransactionSaveService->requestStack;
        $this->customerSecureTransactionRepository = $storePaymentTransactionSaveService
            ->customerSecureTransactionRepository;
    }

    /**
     * Save payment transaction on payment initialization
     */
    public function createSaferpayPaymentTransaction(array $paymentTransactionData, Context $context): void
    {
        /** @var LanguageEntity */
        $languageObj = $this->getLanguageObject($context);
        try {
            $previousPaymentTransaction = $this
                ->getPaymentTransactionFromOrderTransactionId(
                    $paymentTransactionData['orderTransaction']->id,
                    $context
                );
            $paymentTransactionSaveData = $this->buildPaymentTransactionSaveData($paymentTransactionData, $languageObj);
            $authorizeDirect = false;
            if (
                isset($paymentTransactionData['response']->Transaction->Status)
                && $paymentTransactionData['response']->Transaction->Status === 'AUTHORIZED'
            ) {
                $authorizeDirect = true;
            }
            if (!$previousPaymentTransaction) {
                $paymentTransaction = $this->paymentTransactionRepository->create(
                    array(
                        $paymentTransactionSaveData,
                    ),
                    $context
                );
                /** @var EntityWrittenEvent */
                $savedPaymentTransactionArray = !empty($paymentTransaction->getEvents()->getElements()) ?
                    $paymentTransaction->getEvents()->getElements()[0] : array();
                if (empty($savedPaymentTransactionArray)) {
                    return;
                }
                /** @var EntityWriteResult */
                $savedPaymentTransactionResultArray = !empty($savedPaymentTransactionArray->getWriteResults()) ?
                    $savedPaymentTransactionArray->getWriteResults()[0] : array();
                if (empty($savedPaymentTransactionResultArray)) {
                    return;
                }
                $savedPaymentTransactionId = $savedPaymentTransactionResultArray->getPrimaryKey();
                $this->createInitializeTransactionHistory(
                    $savedPaymentTransactionId,
                    $paymentTransactionData['response']->ResponseHeader->RequestId,
                    $context,
                    $authorizeDirect
                );
                if ($authorizeDirect) {
                    $this->updateTransactionHistory(array(
                        'paymentTransactionId' => $savedPaymentTransactionId,
                        'response'             => $paymentTransactionData['response'],
                        'context'              => $context,
                    ));
                }
            } else {
                $paymentTransaction               = $previousPaymentTransaction;
                $paymentTransactionSaveData['id'] = $previousPaymentTransaction->getId();
                $this->paymentTransactionRepository
                    ->update(array($paymentTransactionSaveData), $context);

                $this->createInitializeTransactionHistory(
                    $paymentTransaction->getId(),
                    $paymentTransactionData['response']->ResponseHeader->RequestId,
                    $context,
                    $authorizeDirect
                );
            }
        } catch (Exception $ex) {
            $this->logger->error(
                'Error Code : 1009 : Error occured while saving to payment transaction ' . $ex->getMessage()
            );
            throw new DBALException('Error occured while saving to payment transaction ' . $ex->getMessage());
        }
    }

    /**
     * Build payment transaction save data
     */
    private function buildPaymentTransactionSaveData($paymentTransactionData, $languageObj)
    {
        $paymentTransactionSaveData = array(
            'paymentId'               => $paymentTransactionData['orderTransaction']->paymentMethodId,
            'orderId'                 => $paymentTransactionData['orderTransaction']->orderId,
            'orderTransactionId'      => $paymentTransactionData['orderTransaction']->id,
            'customerId'              => $paymentTransactionData['customerId'],
            'saferpayRequestId'       => $paymentTransactionData['response']->ResponseHeader->RequestId,
            'aliasRegisterStatus'     => false,
            'authorisationMethod'     => $paymentTransactionData['authorizationMethod'],
            'transactionMode'         => $paymentTransactionData['transactionMode'],
            'languageCode'            => $languageObj->getLocale()->getCode(),
            'saferpayCustomerId'      => (int) $paymentTransactionData['saferpayCustomerId'],
            'preauth'                 => $paymentTransactionData['preAuth'],
            'recurring'               => false,
        );
        if (isset($paymentTransactionData['response']->Token)) {
            $paymentTransactionSaveData['saferpayToken'] = $paymentTransactionData['response']->Token;
        }
        if (isset($paymentTransactionData['response']->Expiration)) {
            $paymentTransactionSaveData['saferpayTokenExpiryDate'] = $paymentTransactionData['response']
                ->Expiration;
        }
        return $paymentTransactionSaveData;
    }

    /**
     * Get language object from session or from context
     */
    private function getLanguageObject($context)
    {
        $langId = $this->requestStack->getSession()->get('currentLangId');
        if ($langId === "" || $langId === null) {
            $langId = $context->getLanguageId();
        }
        $langCriteria = new Criteria(
            array(
                'id' => $langId,
            )
        );
        $langCriteria->addAssociation('locale');
        return $this->languageRepository->search($langCriteria, $context)->first();
    }

    /**
     * Function to save initialize history of transaction
     *
     * @param int    $paymentTransactionId payment transaction id.
     * @param string $requestId request id.
     * @param object $context context object.
     */
    private function createInitializeTransactionHistory($paymentTransactionId, $requestId, $context, $isAuthorizeDirect)
    {
        try {
            $this->transactionHistoryRepository->create(
                array(
                    array(
                        'paymentTransactionId' => $paymentTransactionId,
                        'transactionStatus'    => $isAuthorizeDirect ?
                            (self::API_PAYMENT_STATUS_AUTHORIZED_DIRECT) : (self::API_PAYMENT_STATUS_INITIALIZE),
                        'requestId'            => $requestId,
                        'transactionDate'      => new DateTime('now'),
                    ),
                ),
                $context
            );
        } catch (Exception $ex) {
            $this->logger->error(
                'Error Code : 1016 : Error occured while saving to payment history ' . $ex->getMessage()
            );
            throw new DBALException('Error occured while saving to payment history ' . $ex->getMessage());
        }
    }

    /**
     * Update saferpay payment transaction fields
     */
    public function updatePaymentTransaction(array $transactionFields): void
    {
        $paymentTransaction = $this->getPaymentTransactionFromOrderTransactionId(
            $transactionFields['transactionId'],
            $transactionFields['context']
        );
        if (!$paymentTransaction) {
            return;
        }
        $response = $transactionFields['response'];
        try {
            $updateData = $this->buildUpdatePaymentTransactionData($paymentTransaction, $response);
            $this->paymentTransactionRepository->update(array($updateData), $transactionFields['context']);
            $this->updateTransactionHistory(
                array(
                    'paymentTransactionId' => $paymentTransaction->getId(),
                    'response'             => $response,
                    'context'              => $transactionFields['context'],
                )
            );
            if ($updateData['aliasRegisterStatus']) {
                $saferpayPaymentTransactionData = array(
                    'customerId' => $paymentTransaction->getCustomerId(),
                    'saferpayCustomerId' => $paymentTransaction->getSaferpayCustomerId(),
                    'liveTransaction' => $paymentTransaction->getTransactionMode(),
                    'saferpayToken' => $paymentTransaction->getSaferpayToken()
                );
                $this->saveCardInfo(
                    $response,
                    $transactionFields['context'],
                    $saferpayPaymentTransactionData
                );
            }
            // if ($updateData['paidStatus']) {
            //     $this->checkMerchantEmail($paymentTransaction->getId(), $transactionFields['context']);
            //     $this->checkCustomerConfirmationEmail($paymentTransaction->getId(), $transactionFields['context']);
            // }
        } catch (Exception $ex) {
            $this->logger->error(
                'Error Code : 1024 : Error occured while updating payment transaction ' . $ex->getMessage()
            );
            throw new DBALException('Error occured while updating payment transaction ' . $ex->getMessage());
        }
    }

    /**
     * Build update data for saferpay payment transaction table
     */
    private function buildUpdatePaymentTransactionData($paymentTransaction, $response)
    {
        $authorizationAmount = $this->getAuthorizationAmount($response);
        return array(
            'id'                    => $paymentTransaction->getId(),
            'transactionType'       => isset($response->Transaction->Type) ? $response->Transaction->Type : null,
            'authorizationAmount'   => $authorizationAmount,
            'currencyCode'          => isset($response->Transaction->Amount->CurrencyCode) ?
                $response->Transaction->Amount->CurrencyCode : null,
            'captureId'             => isset($response->Transaction->CaptureId) ?
                $response->Transaction->CaptureId : null,
            'capturedAmount'        => isset($response->Transaction->Status) &&
            $response->Transaction->Status === self::API_PAYMENT_STATUS_CAPTURE ?
            $authorizationAmount : 0,
            'saferpayPaymentInfo'   => $this->getSaferpayInfo($response),
            'liabilityShiftStatus'  => isset($response->Liability->LiabilityShift) ?
                $response->Liability->LiabilityShift : null,
            'liableEntity'          => isset($response->Liability->LiableEntity) ?
                $response->Liability->LiableEntity : null,
            'threeDsLiabilityShift' => isset($response->Liability->ThreeDs->LiabilityShift) ?
                $response->Liability->ThreeDs->LiabilityShift : null,
            'dccStatus'             => isset($response->Dcc->PayerAmount->Value) ?
                true : false,
            'dccInfo'               => $this->getDccInfo($response),
            'aliasRegisterStatus'   => isset($response->RegistrationResult->Success) ?
                true : false,
            'paidStatus'            => isset($response->Transaction->Status) &&
                $response->Transaction->Status === self::API_PAYMENT_STATUS_CAPTURE ?
                true : false,
            'transactionStatus'     => true,
            'active'                => true,
            'preauth'               => false,
            'saferpayAliasId'       => isset($response->RegistrationResult->Alias->Id) ?
                $response->RegistrationResult->Alias->Id : null,
            'fraudReviewStatus'     => isset($response->FraudPrevention->Result) ?
                $response->FraudPrevention->Result : null,
        );
    }

    /**
     * Save card info in customer account
     */
    private function saveCardInfo(object $response, Context $context, array $saferpayPaymentTransactionData)
    {
        if (!isset($response->RegistrationResult->Alias->Id)) {
            return;
        }
        /**
         * Check if the alias id already exists
         */
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('saferpayAliasId', $response->RegistrationResult->Alias->Id));
        $criteria->addFilter(new EqualsFilter('customerId', $saferpayPaymentTransactionData['customerId']));
        $previousSaveOfCard = $this->customerSecureTransactionRepository->search($criteria, $context)->first();
        if ($previousSaveOfCard) {
            return;
        }
        $saveCardData = array(
            'saferpayRequestId' => $response->ResponseHeader->RequestId ?? null,
            'saferpayToken' => $saferpayPaymentTransactionData['saferpayToken'],
            'saferpayAliasId' => $response->RegistrationResult->Alias->Id ?? null,
            'saferpayAliasLifetime' => $response->RegistrationResult->Alias->Lifetime ?? null,
            'saferpayPaymentMethod' => $response->PaymentMeans->Brand->PaymentMethod ?? null,
            'saferpayDisplayText' => $response->PaymentMeans->DisplayText ?? null,
            'saferpayMaskedNumber' => $this->getMaskedNumber($response),
            'saferpayHolderName' => $this->getHolderName($response),
            'customerId' => $saferpayPaymentTransactionData['customerId'],
            'saferpayCustomerId' => $saferpayPaymentTransactionData['saferpayCustomerId'],
            'liveTransaction' => $saferpayPaymentTransactionData['liveTransaction'] ? 'Y' : 'N',
            'aliasActive' => 'Y',
            'saferpayCreatedAt' => new DateTime('now')
        );

        if (isset($response->RegistrationResult->AuthenticationResult->Result)) {
            $saveCardData['saferpayAuthenticated'] =
                $response->RegistrationResult->AuthenticationResult->Result === 'OK' ? 'Yes' : 'NA';
        } elseif (isset($response->Liability->ThreeDs->Authenticated) && $response->Liability->LiabilityShift) {
            $saveCardData['saferpayAuthenticated'] =
                $response->Liability->ThreeDs->Authenticated === true && $response->Liability->LiabilityShift === true ? 'Yes' : 'NA';
        } else {
            $saveCardData['saferpayAuthenticated'] = 'NA';
        }
        
        if (isset($response->PaymentMeans->Card->ExpYear)) {
            $saveCardData['saferpayExpYear'] = (string) $response->PaymentMeans->Card->ExpYear;
        }
        if (isset($response->PaymentMeans->Card->ExpMonth)) {
            $saveCardData['saferpayExpMonth'] = (string) $response->PaymentMeans->Card->ExpMonth;
        }
        try {
            $this->customerSecureTransactionRepository->create([$saveCardData], $context);
        } catch (Exception $ex) {
            $this->logger->error(
                'Error Code : 1010 : Error occured while saving card ' . $ex->getMessage()
            );
            throw new DBALException('Error occured while saving card ' . $ex->getMessage());
        }
    }

    /**
     * Get dcc info from response
     */
    private function getDccInfo(object $response): array
    {
        if (isset($response->Dcc->PayerAmount->Value)) {
            $value = $response->Dcc->PayerAmount->Value;
            if ($value > 0) {
                $dccAmount = $value / 100;
            } else {
                $dccAmount = 0;
            }
        } else {
            $dccAmount = null;
        }
        return array(
            'dcc_amount'        => $dccAmount,
            'dcc_currency_code' => isset($response->Dcc->PayerAmount->CurrencyCode) ?
                $response->Dcc->PayerAmount->CurrencyCode : null,
        );
    }

    /**
     * Get saferpay info from response
     */
    private function getSaferpayInfo(object $response): array
    {
        return array(
            'saferpay_display_text'     => $this->getSaferpayDisplayText($response),
            'saferpay_payment_method'   => isset($response->PaymentMeans->Brand->PaymentMethod) ?
                $response->PaymentMeans->Brand->PaymentMethod : null,
            'saferpay_payment_name'     => isset($response->PaymentMeans->Brand->Name) ?
                $response->PaymentMeans->Brand->Name : null,
            'saferpay_masked_number'    => $this->getMaskedNumber($response),
            'saferpay_exp_year'         => isset($response->PaymentMeans->Card->ExpYear) ?
                $response->PaymentMeans->Card->ExpYear : null,
            'saferpay_exp_month'        => isset($response->PaymentMeans->Card->ExpMonth) ?
                $response->PaymentMeans->Card->ExpMonth : null,
            'saferpay_card_country'     => isset($response->PaymentMeans->Card->CountryCode) ?
                $response->PaymentMeans->Card->CountryCode : null,
            'saferpay_holder_name'      => $this->getHolderName($response),
            'acquirer_name'             => isset($response->Transaction->AcquirerName) ?
                $response->Transaction->AcquirerName : null,
            'acquirer_reference'        => isset($response->Transaction->AcquirerReference) ?
                $response->Transaction->AcquirerReference : null,
            'six_transaction_reference' => isset($response->Transaction->SixTransactionReference) ?
                $response->Transaction->SixTransactionReference : null,
            'approval_code'             => isset($response->Transaction->ApprovalCode) ?
                $response->Transaction->ApprovalCode : null,
            'transaction_stamp'         => isset($response->Transaction->IssuerReference->TransactionStamp) ?
                $response->Transaction->IssuerReference->TransactionStamp : null,
            'settlement_date'           => isset($response->Transaction->IssuerReference->SettlementDate) ?
                $response->Transaction->IssuerReference->SettlementDate : null,
        );
    }

    /**
     * Get masked number from response
     */
    public function getMaskedNumber(object $response)
    {
        if (isset($response->PaymentMeans->Card->MaskedNumber)) {
            $maskedNumber = $response->PaymentMeans->Card->MaskedNumber;
        } elseif (isset($response->PaymentMeans->BankAccount->IBAN)) {
            $maskedNumber = $response->PaymentMeans->BankAccount->IBAN;
        } else {
            $maskedNumber = null;
        }
        return $maskedNumber;
    }

    /**
     * Get the holder name from response
     */
    public function getHolderName(object $response)
    {
        if (isset($response->PaymentMeans->Card->HolderName)) {
            $holderName = $response->PaymentMeans->Card->HolderName;
        } elseif (isset($response->PaymentMeans->BankAccount->HolderName)) {
            $holderName = $response->PaymentMeans->BankAccount->HolderName;
        } else {
            $holderName = null;
        }
        return $holderName;
    }

    /**
     * Get the authorization amount from response
     */
    private function getAuthorizationAmount(object $response)
    {
        if (isset($response->Transaction->Amount->Value)) {
            $value = $response->Transaction->Amount->Value;
            if ($value > 0) {
                $authorizationAmount = $value / 100;
            } else {
                $authorizationAmount = 0;
            }
        } else {
            $authorizationAmount = null;
        }
        return $authorizationAmount;
    }

    /**
     * Get the value for saferpay_display_text field
     */
    public function getSaferpayDisplayText(object $response)
    {
        if (isset($response->PaymentMeans->DisplayText)) {
            $displayText = $response->PaymentMeans->DisplayText;
        } elseif (
            isset($response->PaymentMeans->Brand->PaymentMethod) &&
            $response->PaymentMeans->Brand->PaymentMethod === 'TWINT'
        ) {
            $twintCriteria = new Criteria();
            $twintCriteria->addFilter(new EqualsFilter('saferpayPaymentMethod', 'TWINT'));
            /**
             * @var SaferpayCustomerSecureTransactionEntity
             */
            $twintCard = $this->customerSecureTransactionRepository
                ->search($twintCriteria, Context::createDefaultContext())->last();
            if (empty($twintCard) || !$twintCard) {
                $twintCount = 1;
            } else {
                $previousDisplay = $twintCard->getSaferpayDisplayText();
                $previousCount = substr($previousDisplay, strpos($previousDisplay, ' '));
                $twintCount = (int)$previousCount + 1;
            }
            $displayText = 'Twint ' . $twintCount;
        } else {
            $displayText = null;
        }
        return $displayText;
    }

    /**
     * Update saferpay transaction history
     */
    public function updateTransactionHistory(array $transactionHistoryFields): void
    {
        $response = $transactionHistoryFields['response'];
        try {
            $updateData = array(
                'paymentTransactionId' => $transactionHistoryFields['paymentTransactionId'],
                'transactionStatus'    => isset($response->Transaction->Status) ?
                    $response->Transaction->Status : null,
                'requestId'            => isset($response->ResponseHeader->RequestId) ?
                    $response->ResponseHeader->RequestId : null,
                'transactionId'        => isset($response->Transaction->Id) ?
                    $response->Transaction->Id : null,
                'transactionDate'      => isset($response->Transaction->Date) ?
                    $response->Transaction->Date : null,
            );
            $this->transactionHistoryRepository->create(array($updateData), $transactionHistoryFields['context']);
        } catch (Exception $ex) {
            $this->logger->error(
                'Error Code : 1025 : Error occured while saving to transaction history ' . $ex->getMessage()
            );
            throw new DBALException('Error occured while saving to transaction history ' . $ex->getMessage());
        }
    }

    /**
     * Update a cancel transaction
     */
    public function updateCancelTransactionHistory(string $orderTransactionId, Context $context): void
    {
        $paymentTransaction = $this->getPaymentTransactionFromOrderTransactionId($orderTransactionId, $context);
        $updateData         = array(
            'paymentTransactionId' => $paymentTransaction->getId(),
            'transactionStatus'    => self::API_PAYMENT_STATUS_CANCEL,
            'customErrorCode'      => '1050',
            'customErrorMsg'       => 'Payment authentication has been cancelled or failed due to 3Ds Check.',
            'requestId'            => $paymentTransaction->getSaferpayRequestId(),
            'transactionDate'      => new DateTime('now'),
        );
        try {
            $this->transactionHistoryRepository->create(array($updateData), $context);
        } catch (Exception $ex) {
            $this->logger->error('Error Code : 1036 : Error occured while saving cancel transaction history ' .
                $ex->getMessage());
            throw new DBALException('Error occured while saving cancel transaction history ' . $ex->getMessage());
        }
    }

    /**
     * Update a failed transaction
     */
    public function updateFailedTransactionHistory(string $orderTransactionId, Context $context): void
    {
        $paymentTransaction = $this->getPaymentTransactionFromOrderTransactionId($orderTransactionId, $context);
        $updateData         = array(
            'paymentTransactionId' => $paymentTransaction->getId(),
            'transactionStatus'    => self::API_PAYMENT_STATUS_FAILED,
            'customErrorCode'      => '1050',
            'customErrorMsg'       => 'Payment authentication has been cancelled or failed due to 3Ds Check.',
            'requestId'            => $paymentTransaction->getSaferpayRequestId(),
            'transactionDate'      => new DateTime('now'),
        );
        try {
            $this->transactionHistoryRepository->create(array($updateData), $context);
        } catch (Exception $ex) {
            $this->logger->error('Error Code : 1035 : Error occured while saving failed transaction history ' .
                $ex->getMessage());
            throw new DBALException('Error occured while saving failed transaction history ' . $ex->getMessage());
        }
    }

    /**
     * Get order transaction details from order transaction id
     */
    public function getPaymentTransactionFromOrderTransactionId(
        string $orderTransactionId,
        Context $context
    ): SaferpayPaymentTransactionEntity|null {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderTransactionId', $orderTransactionId));
        /** @var SaferpayPaymentTransactionEntity */
        return $this->paymentTransactionRepository->search($criteria, $context)->last();
    }

    /**
     * Update transaction details on payment capture
     */
    public function updateCaptureTransaction(
        string $orderTransactionId,
        object $captureResponse,
        string $saferpayTransactionId,
        Context $context
    ) {
        $paymentTransaction = $this->getPaymentTransactionFromOrderTransactionId($orderTransactionId, $context);
        $paymentData        = array(
            'id'         => $paymentTransaction->getId(),
            'captureId'  => isset($captureResponse->CaptureId) ? $captureResponse->CaptureId : null,
            'paidStatus' => true,
            'capturedAmount' => $paymentTransaction->getAuthorizationAmount(),
        );
        try {
            $this->paymentTransactionRepository->update(array($paymentData), $context);
        } catch (Exception $ex) {
            $this->logger->error(
                'Error Code : 1011 : Error occured while saving capture transaction ' . $ex->getMessage()
            );
            throw new DBALException('Error occured while saving capture transaction ' . $ex->getMessage());
        }
        $historyData = array(
            'paymentTransactionId' => $paymentTransaction->getId(),
            'transactionStatus'    => $captureResponse->Status,
            'requestId'            => $captureResponse->ResponseHeader->RequestId,
            'transactionId'        => $saferpayTransactionId,
            'transactionDate'      => ($captureResponse->Date ?? false ?
                $captureResponse->Date : new DateTime('now')),
        );
        try {
            $this->transactionHistoryRepository->create(array($historyData), $context);
        } catch (Exception $ex) {
            $this->logger->error(
                'Error Code : 1011 : Error occured while saving capture transaction history ' .
                    $ex->getMessage()
            );
            throw new DBALException('Error occured while saving capture transaction history ' . $ex->getMessage());
        }

        // $this->checkMerchantEmail($paymentTransaction->getId(), $context);
        // $this->checkCustomerConfirmationEmail($paymentTransaction->getId(), $context);
    }

    /**
     * Update transaction details after assert capture
     */
    public function updateAssertCaptureTransaction(
        string $orderTransactionId,
        string $captureId,
        object $assertResponse,
        Context $context
    ) {
        $paymentTransaction = $this->getPaymentTransactionFromOrderTransactionId($orderTransactionId, $context);
        $paymentData        = array(
            'id'        => $paymentTransaction->getId(),
            'captureId' => $captureId,
            'capturedAmount' => $paymentTransaction->getAuthorizationAmount(),
        );
        try {
            $this->paymentTransactionRepository->update(array($paymentData), $context);
        } catch (Exception $ex) {
            $this->logger->error(
                'Error Code : 1011 : Error occured while saving capture transaction ' . $ex->getMessage()
            );
            throw new DBALException('Error occured while saving capture transaction ' . $ex->getMessage());
        }
        $historyData = array(
            'paymentTransactionId' => $paymentTransaction->getId(),
            'transactionStatus'    => $assertResponse->Status,
            'requestId'            => $assertResponse->ResponseHeader->RequestId,
            'transactionId'        => $assertResponse->TransactionId,
            'transactionDate'      => $assertResponse->Date,
        );
        try {
            $this->transactionHistoryRepository->create(array($historyData), $context);
        } catch (Exception $ex) {
            $this->logger->error(
                'Error Code : 1011 : Error occured while saving capture transaction history ' .
                    $ex->getMessage()
            );
            throw new DBALException('Error occured while saving capture transaction history ' . $ex->getMessage());
        }
        // $this->checkMerchantEmail($paymentTransaction->getId(), $context);
        // $this->checkCustomerConfirmationEmail($paymentTransaction->getId(), $context);
    }

    /**
     * Get the payment transaction entity from id
     *
     * @return SaferpayPaymentTransactionEntity
     */
    public function getPaymentTransaction(string $paymentTransactionId, Context $context)
    {
        $criteria = new Criteria(array($paymentTransactionId));
        $criteria->addAssociation('orderTransaction')
            ->addAssociation('order')
            ->addAssociation('order.salesChannel');
        return $this->paymentTransactionRepository->search($criteria, $context)->first();
    }

    /**
     * Get Saleschannel
     */
    public function getSalesChannelFromOrderId(string $orderId, Context $context)
    {
        $criteria = new Criteria(array($orderId));
        $criteria->addAssociation('salesChannel');
        /** @var OrderEntity */
        $order = $this->orderRepository->search($criteria, $context)->first();
        if ($order) {
            return $order->getSalesChannel();
        }
        return false;
    }

    /**
     * Get order transaction
     */
    public function getOrderTransactionFomId(string $orderTransactionId, Context $context)
    {
        $criteria = new Criteria(array($orderTransactionId));
        return $this->orderTransactionRepository->search($criteria, $context)->first();
    }

    /**
     * Get the transaction id from history
     */
    public function getTransactionIdFromHistory(string $paymentTransactionId, Context $context)
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('paymentTransactionId', $paymentTransactionId))
            ->addFilter(new EqualsFilter('transactionStatus', 'AUTHORIZED'));
        /** @var SaferpayTransactionHistoryEntity */
        $transactionHistory = $this->transactionHistoryRepository->search($criteria, $context)->first();
        if ($transactionHistory) {
            return $transactionHistory->getTransactionId();
        }
        return false;
    }

    /**
     * Gets a particular plugin config using the key
     */
    public function getPluginConfig(string $key, ?SalesChannelEntity $salesChannel = null): string
    {
        $configValue = $this->systemConfigService->get(
            'SaferpayShopware.config.' . $key,
            $salesChannel ? $salesChannel->getId() : null
        );
        return trim((string) $configValue ?? '');
    }

    /**
     * Gets array of all the merchant email addresses set in the plugin
     */
    public function buildMerchantEmails(SalesChannelEntity $salesChannel): array
    {
        $merchantEmailsList = $this->getPluginConfig('merchantEmails', $salesChannel);
        if (!$merchantEmailsList) {
            return array();
        }

        $merchantEmailsArray = explode(',', $merchantEmailsList);
        $merchantEmails      = array();
        foreach ($merchantEmailsArray as $merchantEmail) {
            $merchantEmail = trim($merchantEmail);
            if ($merchantEmail) {
                $merchantEmails[] = $merchantEmail;
            }
        }

        return $merchantEmails;
    }

    /**
     * Get the order number of the order
     *
     * @return OrderEntity
     */
    private function getOrderFromId($orderId, $context)
    {
        $criteria = new Criteria(array($orderId));
        return $this->orderRepository->search($criteria, $context)->first();
    }

    /**
     * Check if capture transaction exists for the payment transaction id
     *
     * @param string  $paymentTransactionId
     * @param Context $contexts
     * @return SaferpayTransactionHistoryEntity
     */
    private function getCaptureTransaction($paymentTransactionId, $context)
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('paymentTransactionId', $paymentTransactionId))
            ->addFilter(new EqualsFilter('transactionStatus', 'CAPTURED'));
        /**
         * @var SaferpayTransactionHistoryEntity
         */
        return $this->transactionHistoryRepository->search($criteria, $context)->first();
    }

    /**
     * Get the common content for email data
     */
    private function getCommonEmailData($paymentTransactionId, $context)
    {
        $paymentTransaction  = $this->getPaymentTransaction($paymentTransactionId, $context);
        $salesChannel        = $this->getSalesChannelFromOrderId($paymentTransaction->getOrderId(), $context);
        $order               = $this->getOrderFromId($paymentTransaction->getOrderId(), $context);
        $orderNumber         = $order->getOrderNumber();
        $saferpayPaymentInfo = $paymentTransaction->getSaferpayPaymentInfo();
        $testMode            = $this->getPluginConfig('operationMode', $salesChannel) !== 'live';
        $terminalId          = $this->getPluginConfig($testMode ? 'terminalIdTest' : 'terminalIdLive', $salesChannel);
        $captureTransaction  = $this->getCaptureTransaction($paymentTransactionId, $context);
        return array(
            'paymentTransaction'  => $paymentTransaction,
            'salesChannel'        => $salesChannel,
            'orderNumber'         => $orderNumber,
            'saferpayPaymentInfo' => $saferpayPaymentInfo,
            'terminalId'          => $terminalId,
            'captureTransaction'  => $captureTransaction,
            'order'               => $order,
        );
    }

    /**
     * Check if merchant email has to be sent, build the necessary information for merchant email
     */
    public function checkMerchantEmail($paymentTransactionId, $context)
    {
        $commonData    = $this->getCommonEmailData($paymentTransactionId, $context);
        $merchantEmails = $this->buildMerchantEmails($commonData['salesChannel']);

        if (!empty($merchantEmails) && $commonData['captureTransaction']) {
            $emailData = array(
                'recipients'           => $merchantEmails,
                'amount'               => $commonData['paymentTransaction']->getAuthorizationAmount(),
                'currencyCode'         => $commonData['paymentTransaction']->getCurrencyCode(),
                'date'                 => $commonData['captureTransaction']
                    ->getTransactionDate()->format('m/d/Y h:i:s A'),
                'transactionId'        => $commonData['captureTransaction']->getTransactionId(),
                'terminalNumber'       => $commonData['terminalId'],
                'paymentDescription'   => $commonData['salesChannel']->getName() . "\n\n" . 'Order Number ' .
                    $commonData['orderNumber'],
                'liabilityShiftStatus' => $commonData['paymentTransaction']->getLiabilityShiftStatus() ? 'Yes' : 'No',
                'liableEntity'         => $commonData['paymentTransaction']->getLiableEntity(),
                'paymentMethod'        => $commonData['saferpayPaymentInfo']['saferpay_payment_method'],
                'referenceNumber'      => $commonData['orderNumber'],
                'salesChannelName'     => $commonData['salesChannel']->getName(),
            );
            $this->emailService
                ->sendMerchantMails(
                    $emailData,
                    $commonData['salesChannel']->getId(),
                    $context
                );
        }
    }

    /**
     * Send customer confirmation email
     */
    public function checkCustomerConfirmationEmail($paymentTransactionId, $context)
    {
        $commonData = $this->getCommonEmailData($paymentTransactionId, $context);
        if ($commonData['captureTransaction']) {
            $emailData = array(
                'amount'             => $commonData['paymentTransaction']->getAuthorizationAmount(),
                'currencyCode'       => $commonData['paymentTransaction']->getCurrencyCode(),
                'date'               => $commonData['captureTransaction']
                    ->getTransactionDate()->format('m/d/Y h:i:s A'),
                'transactionId'      => $commonData['captureTransaction']->getTransactionId(),
                'paymentDescription' => $commonData['salesChannel']->getName() . "\n\n" . 'Order Number ' .
                    $commonData['orderNumber'],
                'referenceNumber'    => $commonData['orderNumber'],
                'salesChannelName'   => $commonData['salesChannel']->getName(),
                'paymentId'          => $commonData['paymentTransaction']->getPaymentId(),
                'order'              => $commonData['order'],
            );
            if (isset($commonData['saferpayPaymentInfo']) && $commonData['saferpayPaymentInfo'] !== null) {
                $emailData['cardNumber'] = $commonData['saferpayPaymentInfo']['saferpay_masked_number'];
                $emailData['paymentName'] = $commonData['saferpayPaymentInfo']['saferpay_payment_name'];
            }
            $this->emailService->sendUserConfirmationMail($emailData, $commonData['salesChannel']->getId(), $context);
        }
    }


    /**
     * Update saferpay payment transaction fields
     */
    public function updateTransactionHistoryAfterRefund(array $transactionFields): void
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderTransactionId', $transactionFields['transactionId']));
        /** @var SaferpayPaymentTransactionEntity */
        $paymentTransaction = $this->paymentTransactionRepository
            ->search($criteria, $transactionFields['context'])
            ->first();
        if (!$paymentTransaction) {
            return;
        }
        $response = $transactionFields['response'];
        try {
            $this->updateTransactionHistory(
                array(
                    'paymentTransactionId' => $paymentTransaction->getId(),
                    'response'             => $response,
                    'context'              => $transactionFields['context'],
                )
            );
        } catch (Exception $ex) {
            throw new DBALException('Error occured while updating payment transaction ' . $ex->getMessage());
        }
    }

    /**
     * get Payment Transaction With Order Transaction function
     *
     * @param string  $transactionId
     * @param Context $context
     * @return SaferpayTransactionHistoryEntity
     */
    public function getPaymentTransactionWithOrderTransaction($transactionId, Context $context)
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderTransactionId', $transactionId));
        /** @var SaferpayPaymentTransactionEntity */

        $paymentTransacton = $this->paymentTransactionRepository
            ->search($criteria, $context)
            ->first();

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('paymentTransactionId', $paymentTransacton->getId()))
            ->addFilter(new EqualsFilter('transactionStatus', 'PENDING'));
        /**
         * @var SaferpayTransactionHistoryEntity
         */
        return $this->transactionHistoryRepository->search($criteria, $context)->first();
    }
}
