<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Subscriber;

use Saferpay\Shopware\Core\Content\Saferpay\SaferpayCustomerSecureTransactionEntity;
use Saferpay\Shopware\SaferpayShopware;
use Saferpay\Shopware\Service\PaymentService;
use Shopware\Core\Checkout\Cart\Event\CheckoutOrderPlacedEvent;
use Shopware\Core\Checkout\Cart\Order\CartConvertedEvent;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsAnyFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Symfony\Component\DependencyInjection\Container as DIC;
use Shopware\Core\Framework\Struct\ArrayStruct;
use Shopware\Core\System\Country\CountryEntity;
use Shopware\Core\System\Currency\CurrencyEntity;
use Shopware\Core\System\Language\LanguageEntity;
use Shopware\Storefront\Page\Account\Order\AccountEditOrderPageLoadedEvent;
use Shopware\Storefront\Page\Checkout\Confirm\CheckoutConfirmPageLoadedEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class CheckoutSubscriber implements EventSubscriberInterface
{
    private EntityRepository $paymentMethodRepository;
    private EntityRepository $currencyRepository;
    private EntityRepository $countryRepository;
    private PaymentService $paymentService;
    private EntityRepository $customFieldRepository;
    private RequestStack $requestStack;
    private EntityRepository $languageRepository;
    private EntityRepository $customerSecureTransactionRepository;
    private DIC $container;

    /**
     * Initialize the class parameters
     */
    public function __construct(
        DIC $container,
        PaymentService $paymentService
    ) {
        $this->container                           = $container;
        $this->paymentMethodRepository             = $this->container->get('payment_method.repository');
        $this->currencyRepository                  = $this->container->get('currency.repository');
        $this->countryRepository                   = $this->container->get('country.repository');
        $this->paymentService                      = $paymentService;
        $this->customFieldRepository               = $this->container->get('custom_field.repository');
        $this->requestStack                        = $this->container->get('request_stack');
        $this->languageRepository                  = $this->container->get('language.repository');
        $this->customerSecureTransactionRepository = $this->container
            ->get('saferpay_customer_secure_transaction.repository');
    }

    /**
     * Function overridden from parent
     */
    public static function getSubscribedEvents(): array
    {
        return array(
            CheckoutConfirmPageLoadedEvent::class  => array('onCheckoutPageLoaded', 1),
            AccountEditOrderPageLoadedEvent::class => array('onEditPageLoaded', 1),
            CartConvertedEvent::class              => array('saveCustomField', 1),
            CheckoutOrderPlacedEvent::class        => array('saveOrderId', 1),
        );
    }

    /**
     * Save the order id in session
     */
    public function saveOrderId(CheckoutOrderPlacedEvent $event): void
    {
        $orderId = $event->getOrderId();
        $session = $this->requestStack->getSession();
        $session->set('currentOrderId', $orderId);
    }

    /**
     * Save the ideal bank saved to custom field
     */
    public function saveCustomField(CartConvertedEvent $event): void
    {
        $orderData         = $event->getConvertedCart();
        $orderCustomFields = $orderData['customFields'] ?? array();

        $currentRequest = $this->requestStack->getCurrentRequest()->request;

        $customerSelectedIdealBank = $currentRequest->get('customer_selected_ideal_bank');

        if ($customerSelectedIdealBank) {
            $orderCustomFields['customer_selected_ideal_bank'] = $customerSelectedIdealBank;
        } else {
            if (isset($orderCustomFields['customer_selected_ideal_bank'])) {
                unset($orderCustomFields['customer_selected_ideal_bank']);
            }
        }

        $saveCardToAccount = $currentRequest->get('save_card_to_account');

        if ($saveCardToAccount) {
            $orderCustomFields['save_card_to_account'] = $saveCardToAccount;
        } else {
            if (isset($orderCustomFields['save_card_to_account'])) {
                unset($orderCustomFields['save_card_to_account']);
            }
        }

        $saveCardToAccount = $currentRequest->get('customer_selected_alias');

        if ($saveCardToAccount) {
            $orderCustomFields['customer_selected_alias'] = $saveCardToAccount;
        } else {
            if (isset($orderCustomFields['customer_selected_alias'])) {
                unset($orderCustomFields['customer_selected_alias']);
            }
        }

        $orderData['customFields'] = $orderCustomFields;

        $event->setConvertedCart($orderData);
    }

    /**
     * Check if the payment methods are loaded as per currency and country selected
     */
    public function onCheckoutPageLoaded(CheckoutConfirmPageLoadedEvent $event): void
    {
        $session = $this->requestStack->getSession();
        $session->remove('currentOrderId');
        if ($event->getContext()->getLanguageIdChain() !== null && !empty($event->getContext()->getLanguageIdChain())) {
            $session->set('currentLangId', $event->getContext()->getLanguageIdChain()[0]);
        }
        $this->checkPaymentMethodAvailability($event, 'checkout', Context::createDefaultContext());
        [$live_banks, $test_banks] = $this->getIdealPaymentBanks($event->getContext());
        $event->getPage()->addExtension(
            'ideal_banks',
            new ArrayStruct(
                array(
                    'test' => $test_banks,
                    'live' => $live_banks,
                )
            )
        );
        $customerId = $event->getSalesChannelContext()->getCustomerId();
        $userCards = $this->getSavedCardsOfUser($event, $customerId, $event->getContext());
        $event->getPage()->addExtension(
            'saved_cards',
            new ArrayStruct($userCards)
        );
    }

    /**
     * Check if the payment methods are loaded as per currency and country selected
     */
    public function onEditPageLoaded(AccountEditOrderPageLoadedEvent $event): void
    {
        $orderId = $event->getPage()->getOrder()->id;
        $session = $this->requestStack->getSession();
        $session->set('currentOrderId', $orderId);
        $this->checkPaymentMethodAvailability($event, 'edit-order', Context::createDefaultContext());
        [$live_banks, $test_banks] = $this->getIdealPaymentBanks($event->getContext());
        $event->getPage()->addExtension(
            'ideal_banks',
            new ArrayStruct(
                array(
                    'test' => $test_banks,
                    'live' => $live_banks,
                )
            )
        );
        $customerId = $event->getSalesChannelContext()->getCustomerId();
        $userCards = $this->getSavedCardsOfUser($event, $customerId, $event->getContext());
        $event->getPage()->addExtension(
            'saved_cards',
            new ArrayStruct($userCards)
        );
    }

    /**
     * Get Saved cards of user logged in
     */
    private function getSavedCardsOfUser($event, string $customerId, Context $context)
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('customerId', $customerId));
        $criteria->addFilter(new EqualsFilter('aliasActive', 'Y'));
        $savedCards = $this->customerSecureTransactionRepository->search($criteria, $context)->getEntities();
        $userCards  = array();
        [$currencyCode, $countryCode] = $this->getCountryAndCurrencyCode($event);
        if (!empty($savedCards)) {
            /**
             * @var SaferpayCustomerSecureTransactionEntity
             */
            foreach ($savedCards as $eachCard) {
                $paymentId = $this->getPaymentIdFromCode($eachCard->getSaferpayPaymentMethod());
                if($this->currencyAvailableForPaymentMethod($currencyCode, $paymentId, $context)
                && $this->countryAvailableForPaymentMethod($countryCode, $paymentId, $context)) {
                    $userCards[$eachCard->getSaferpayPaymentMethod()][$eachCard->getSaferpayAliasId()]
                        = $eachCard->getSaferpayDisplayText();
                }
            }
            return $userCards;
        }
    }

    /**
     * Get the custom field options for ideal bank names
     */
    private function getIdealPaymentBanks(Context $context)
    {
        $criteria = new Criteria();
        $criteria->addFilter(
            new EqualsAnyFilter(
                'name',
                array(
                    'saferpay_shopware_test_account_details',
                    'saferpay_shopware_account_details',
                )
            )
        );
        $langId = $this->requestStack->getSession()->get('currentLangId');
        
        if ($langId === '' || $langId === null) {
            $langId = $context->getLanguageId();
        }
        $customFields = $this->customFieldRepository->search($criteria, $context);
        $langCriteria = new Criteria();
        $langCriteria->addFilter(new EqualsFilter('id', $langId));
        $langCriteria->addAssociation('locale');
        /**
         * @var LanguageEntity
         */
        $language   = $this->languageRepository->search($langCriteria, $context)->first();
        $langCode   = $language->getLocale()->getCode();
        $live_banks = array();
        $test_banks = array();
        foreach ($customFields as $eachResult) {
            $config  = $eachResult->getConfig();
            $options = $config['options'];

            foreach ($options as $eachOption) {
                $optionKey = $eachOption['value'];
                if (isset($eachOption['label'][$langCode])) {
                    $optionValue = $eachOption['label'][$langCode];
                } elseif (isset($eachOption['label'][$langId])) {
                    $optionValue = $eachOption['label'][$langId];
                } else {
                    $optionValue = $eachOption['label']['en-GB'];
                }
                if ($eachResult->getName() === 'saferpay_shopware_test_account_details') {
                    $test_banks[$optionKey] = $optionValue;
                } else {
                    $live_banks[$optionKey] = $optionValue;
                }
            }
        }
        return array($live_banks, $test_banks);
    }

    /**
     * 1. Check if payment method available for current payment settings
     * 2. Remove payment methods from Checkout if they are not available
     */
    private function checkPaymentMethodAvailability($event, $eventName, $context)
    {
        [$currencyCode, $countryCode] = $this->getCountryAndCurrencyCode($event);
        /**
         * Min Order Total, Max Order Total
         */
        if ($eventName === 'checkout') {
            $orderTotal = $event->getPage()->getCart()->getPrice()->getTotalPrice();
        } else {
            $orderTotal = $event->getPage()->getOrder()->getAmountTotal();
        }

        $paymentMethods = $event->getPage()->getPaymentMethods();
        $creditCardPayments = [];
        $creditCardAvailablePayments = $this->getCreditCardAvailablePaymentMethod($event->getContext());
        foreach ($paymentMethods as $eachPaymentMethod) {
            $paymentId = $eachPaymentMethod->getId();
            if (
                !$this->currencyAvailableForPaymentMethod($currencyCode, $paymentId, $event->getContext())
                || !$this->countryAvailableForPaymentMethod($countryCode, $paymentId, $event->getContext())
                || !$this->minMaxOrderTotalForPaymentMethod($orderTotal, $paymentId, $event->getContext())
                || ($paymentId === SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID
                    && !$this->checkCountryCurrencyMappingForKlarna($currencyCode, $countryCode))
            ) {
                $event->getPage()->getPaymentMethods()->remove($paymentId);
            }
        }
        foreach($creditCardAvailablePayments as $creditCardpayment) {
            $criteria = new Criteria(array($creditCardpayment));
            $criteria->addAssociation('media');
            $paymentMethodEntity = $this->paymentMethodRepository->search($criteria, $context)->first();
            if($paymentMethodEntity &&
            $this->currencyAvailableForPaymentMethod($currencyCode, $paymentMethodEntity->getId(), $event->getContext())
            && $this->countryAvailableForPaymentMethod($countryCode, $paymentMethodEntity->getId(), $event->getContext())) {
                $creditCardPayments[] = $paymentMethodEntity;
                $creditCardPaymentCodes[] = $this->getPaymentCodeFromId($paymentMethodEntity->getId());
            }
        }
        if(empty($creditCardPayments))
        {
            $event->getPage()->getPaymentMethods()->remove(SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID);
        }
        $event->getPage()->assign(['creditCardPayments' => $creditCardPayments]);
        $event->getPage()->assign(['creditCardPaymentCodes' => $creditCardPaymentCodes]);
    }

    /**
     * If the country currency mapping is not correct, then remove klarna payment method
     */
    private function checkCountryCurrencyMappingForKlarna($currencyCode, $countryCode): bool
    {
        $klarnaCountryCurrency = array(
            'AU' => 'AUD',
            'AT' => 'EUR',
            'BE' => 'EUR',
            'CA' => 'CAD',
            'CZ' => 'CZK',
            'DK' => 'DKK',
            'FI' => 'EUR',
            'FR' => 'EUR',
            'DE' => 'EUR',
            'GR' => 'EUR',
            'HU' => 'HUF',
            'IE' => 'EUR',
            'IT' => 'EUR',
            'MX' => 'MXN',
            'NL' => 'EUR',
            'NZ' => 'NZD',
            'NO' => 'NOK',
            'PL' => 'PLN',
            'PT' => 'EUR',
            'RO' => 'RON',
            'ES' => 'EUR',
            'SE' => 'SEK',
            'CH' => 'CHF',
            'GB' => 'GBP',
            'US' => 'USD',
        );
        if (isset($klarnaCountryCurrency[$countryCode]) && $klarnaCountryCurrency[$countryCode] === $currencyCode) {
            return true;
        }
        return false;
    }

    /**
     * Check if the payment method is available for the current order total based on the Min Order Total
     * and Max Order Total set for the payment methods
     */
    private function minMaxOrderTotalForPaymentMethod($orderTotal, $paymentId, $context): bool
    {
        $paymentMethodEntity = $this->paymentMethodRepository->search(new Criteria(array($paymentId)), $context)
            ->first();
        $paymentCode         = $this->getPaymentCodeFromId($paymentId);
        if ($paymentCode === '') {
            return true;
        }
        $customFields = $this->paymentService->getPaymentMethodCustomFields($paymentMethodEntity, $context);
        /**
         * Check Min Order Total, Max Order Total
         */
        if (
            !empty($customFields)
            && (
                (isset($customFields['saferpay_shopware_payment_min_order_total'])
                && $customFields['saferpay_shopware_payment_min_order_total'] > 0
                && $orderTotal < $customFields['saferpay_shopware_payment_min_order_total'])
                ||
                (isset($customFields['saferpay_shopware_payment_max_order_total'])
                    && $customFields['saferpay_shopware_payment_max_order_total'] > 0
                    && $orderTotal > $customFields['saferpay_shopware_payment_max_order_total'])
            )

        ) {
            return false;
        }
        return true;
    }

    /**
     * Check for the currency mapping with payment method
     */
    private function currencyAvailableForPaymentMethod($currencyCode, $paymentId, $context): bool
    {
        if (!empty($paymentId)) {
            $criteria = new Criteria([$paymentId]);
            $paymentMethodEntity = $this->paymentMethodRepository->search($criteria, $context)->first();
        }
        $paymentCode         = $this->getPaymentCodeFromId($paymentId);
        if ($paymentCode === '') {
            return true;
        }
        $customFields = $this->paymentService->getPaymentMethodCustomFields($paymentMethodEntity, $context);
        if (
            isset($customFields['saferpay_shopware_payment_currencies']) &&
            count($customFields['saferpay_shopware_payment_currencies']) == 0
        ) {
            return true;
        }
        if (
            !empty($customFields) && isset($customFields['saferpay_shopware_payment_currencies'])
            && in_array($currencyCode, $customFields['saferpay_shopware_payment_currencies'])
        ) {
            return true;
        }
        return false;
    }

    /**
     * Check for the currency mapping with payment method
     */
    private function getCreditCardAvailablePaymentMethod($context): array
    {
        $creditCardpaymentIds = [];
        $paymentMethodEntity = $this->paymentMethodRepository->search(new Criteria(array(SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID)), $context)
            ->first();
        $customFields = $this->paymentService->getPaymentMethodCustomFields($paymentMethodEntity, $context);
        if (
            isset($customFields['saferpay_shopware_payment_means']) && $customFields['saferpay_shopware_payment_means']
        ) {
            foreach ($customFields['saferpay_shopware_payment_means'] as $creditCardpayment) {
                $creditCardpaymentIds[] = $this->getPaymentIdFromCode($creditCardpayment);
            }
        }
        else
        {
            $creditCardpaymentIds = array(
                   SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID,
                   SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID,
                   SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID,
                   SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID,
                   SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID,
                   SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID,
                   SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID
            );
            
        }

        return $creditCardpaymentIds;
    }

    /**
     * Check for the country mapping with payment method
     */
    private function countryAvailableForPaymentMethod($countryCode, $paymentId, $context)
    {
        if (!empty($paymentId)) {
            $criteria = new Criteria([$paymentId]);
            $paymentMethodEntity = $this->paymentMethodRepository->search($criteria, $context)->first();
        }
        $paymentCode         = $this->getPaymentCodeFromId($paymentId);
        if ($paymentCode === '') {
            return true;
        }
        $customFields = $this->paymentService->getPaymentMethodCustomFields($paymentMethodEntity, $context);
        if (
            empty($customFields) ||
            empty($customFields['saferpay_shopware_payment_countries']) ||
            (!empty($customFields) && isset($customFields['saferpay_shopware_payment_countries'])
                && in_array($countryCode, $customFields['saferpay_shopware_payment_countries']))
        ) {
            return true;
        }
        return false;
    }

    /**
     * 1. Get payment code from id
     * 2. If the payment method is not from saferpay, restriction need not be checked
     */
    public static function getPaymentCodeFromId($paymentId): string
    {
        switch ($paymentId):
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS;
                break;
            case SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS;
                break;
            case SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_WALLET_GOOGLE_PAY;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_KLARNA;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_TWINT;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DIRECT_DEBIT;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID:
                $paymentCode = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID;
                break;

            default:
                $paymentCode = '';
        endswitch;
        return $paymentCode;
    }

    /**
     * 1. Get payment id from code
     * 2. If the payment method is not from saferpay, restriction need not be checked
     */
    public static function getPaymentIdFromCode($paymentCode): string
    {
        switch ($paymentCode):
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_ALIPAY_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_AMERICAN_EXPRESS_ID;
                break;
            case SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY:
                $paymentId = SaferpayShopware::SAFERPAY_WALLET_APPLE_PAY_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_BANCONTACT_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_DINERS_CLUB_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_E_PRZELEWY_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_EPS_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_IDEAL_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_JCB_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MAESTRO_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MASTERCARD_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_MY_ONE_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAY_PAL_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_PAYDIREKT_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_CARD_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_POST_FINANCE_E_FINANCE_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_SOFORT_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_UNIONPAY_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_VISA_ID;
                break;
            case SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS:
                $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_WL_CRYPTO_PAYMENTS_ID;
                break;
            default:
                $paymentId = '';
        endswitch;
        return $paymentId;
    }

    /**
     * Get Country and Currency Code
     * 
     * @return array
     * 
     */
    public function getCountryAndCurrencyCode($event)
    {
         /**
         * Currency Restriction
         */
        $currencyId = $event->getContext()->getCurrencyId();
        /** @var CurrencyEntity */
        $currencyObj  = $this->currencyRepository->search(
            new Criteria(
                array(
                    'id' => $currencyId,
                )
            ),
            $event->getContext()
        )->first();
        $currencyCode = $currencyObj->getIsoCode();

        /**
         * Country Restriction
         */
        $countryId = $event->getSalesChannelContext()->getCustomer()->getDefaultBillingAddress()
            ->getCountry()->getId();

        /** @var CountryEntity */
        $countryObj  = $this->countryRepository->search(
            new Criteria(
                array(
                    'id' => $countryId,
                )
            ),
            $event->getContext()
        )->first();
        $countryCode = $countryObj->getIso();

        return [$currencyCode,$countryCode];
    }
}
