<?php

declare(strict_types=1);

namespace Saferpay\Shopware\Subscriber;

use Exception;
use Doctrine\DBAL\Exception as DBALException;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Shopware\Core\Checkout\Payment\PaymentEvents;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\EntityWriteResult;
use Shopware\Core\Framework\DataAbstractionLayer\Event\EntityWrittenEvent;

class PaymentMethodSubscriber implements EventSubscriberInterface
{

    private EntityRepository $paymentMethodTranslationRepository;

    public function __construct(EntityRepository $paymentMethodTranslationRepository)
    {
        $this->paymentMethodTranslationRepository = $paymentMethodTranslationRepository;
    }

    public static function getSubscribedEvents(): array
    {
        return array(
            PaymentEvents::PAYMENT_METHOD_TRANSLATION_WRITTEN_EVENT => array('onPaymentMethodSave', 1),
        );
    }

    /**
     * Check for custom field update of pre authorization field and capturing field
     */
    public function onPaymentMethodSave(EntityWrittenEvent $event): void
    {
        $writeResults = $event->getWriteResults();
        if (empty($writeResults)) {
            return;
        }
        /** @var EntityWriteResult */
        $singleWriteResult = $writeResults[0];
        $payload           = $singleWriteResult->getPayload();
        if (empty($payload) || !isset($payload['customFields'])) {
            return;
        }
        $customFields = $payload['customFields'];
        if (empty($customFields) || (!isset($customFields['saferpay_shopware_pre_authorizations']) &&
            !isset($customFields['saferpay_shopware_save_card']))) {
            return;
        }
        if (isset($customFields['saferpay_shopware_pre_authorizations'])) {
            $preAuth = $customFields['saferpay_shopware_pre_authorizations'];
            if (
                $preAuth === '1'
                && isset($customFields['saferpay_shopware_capturing'])
                && $customFields['saferpay_shopware_capturing'] === 'direct'
            ) {
                $this->updateCustomField(
                    $payload['paymentMethodId'],
                    $payload['languageId'],
                    'saferpay_shopware_capturing',
                    $event->getContext()
                );
            }
        }
        if (isset($customFields['saferpay_shopware_save_card'])) {
            $saveCard = $customFields['saferpay_shopware_save_card'];
            if (
                $saveCard === 'active' && isset($customFields['saferpay_shopware_authorization_method']) &&
                $customFields['saferpay_shopware_authorization_method'] === 'paymentPage'
            ) {
                $this->updateCustomField(
                    $payload['paymentMethodId'],
                    $payload['languageId'],
                    'saferpay_shopware_save_card',
                    $event->getContext()
                );
            }
        }
    }

    /**
     * Save the custom field
     */
    private function updateCustomField($paymentMethodId, $languageId, $customField, $context): void
    {
        if ($customField === 'saferpay_shopware_save_card') {
            $fieldValue = 'inactive';
        } elseif ($customField === 'saferpay_shopware_capturing') {
            $fieldValue = 'delayed';
        }
        try {
            $this->paymentMethodTranslationRepository->update(
                array(
                    array(
                        'paymentMethodId' => $paymentMethodId,
                        'languageId'      => $languageId,
                        'customFields'    => array(
                            $customField => $fieldValue,
                        ),
                    ),
                ),
                $context
            );
        } catch (Exception $ex) {
            throw new DBALException(
                'Error occured while updating payment translation ' . $ex->getMessage()
            );
        }
    }
}
