<?php

declare(strict_types=1);
/**
 *  Saferpay PaymentService
 *  NOTICE OF LICENSE
 *
 * Once you have purchased the software with PIT Solutions AG
 * or one of its  authorised resellers and provided that you comply with the conditions of this contract,
 * PIT Solutions AG grants you a non-exclusive license,
 * unlimited in time for the usage of the software in the manner of and for the purposes specified in License.txt
 * available in extension package, according to the subsequent regulations
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to
 * newer versions in the future.
 */

namespace Saferpay\Shopware\Subscriber;

use Shopware\Core\Framework\Context;
use Saferpay\Shopware\Api\SaferpayPaymentApiClient;
use Saferpay\Shopware\SaferpayShopware;
use Saferpay\Shopware\Service\LoggerService;
use Saferpay\Shopware\Service\PaymentService;
use Saferpay\Shopware\Service\PaymentTransactionSaveService;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\System\SystemConfig\Event\SystemConfigChangedHook;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use Symfony\Component\DependencyInjection\Container;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;

const ERRORMESSAGE = 'failed to generate Field Access Token';
class FailedToGetRequestException extends \RuntimeException
{
}

class SystemConfigSubscriber implements EventSubscriberInterface
{


    private PaymentTransactionSaveService $paymentTransactionSaveService;
    private LoggerService $logger;
    public PaymentService $paymentService;
    public SystemConfigService $systemConfigService;
    private Container $container;
    private Context $context;
    private $requestStack;
    private $paymentId = SaferpayShopware::SAFERPAY_PAYMENT_METHOD_CREDIT_CARD_ID;
    private $paymentMethodRepository;
    private $paymentMethodEntity;
    private $paymentMethodTranslationEntity;
    private $paymentMethodTranslationRepository;




    public function __construct(
        PaymentTransactionSaveService $paymentTransactionSaveService,
        LoggerService $logger,
        Container $container,
        PaymentService $paymentService,
        SystemConfigService $systemConfigService,
        RequestStack $requestStack
    ) {
        $this->paymentTransactionSaveService = $paymentTransactionSaveService;
        $this->logger                       = $logger;
        $this->container = $container;
        $this->paymentService = $paymentService;
        $this->systemConfigService = $systemConfigService;
        $this->requestStack = $requestStack;
        $this->paymentMethodRepository  = $this->container->get('payment_method.repository');
        $this->paymentMethodTranslationRepository  = $this->container->get('payment_method_translation.repository');
    }


    public static function getSubscribedEvents(): array
    {
        return array(
            SystemConfigChangedHook::class => 'onPluginUpdated',
        );
    }

    /**
     * getSubscribedEvents
     *
     * @return void
     */
    public function onPluginUpdated(SystemConfigChangedHook $event)
    {

        if ($this->requestStack->getCurrentRequest() === null) {
            return 0;
        }
        $this->context = Context::createDefaultContext();
        if (!$this->requestStack->getCurrentRequest()) {
            return false;
        }
        $request = $this->requestStack->getCurrentRequest()->request;
        if (!isset($request->all()['null'])) {
            return 0;
        }
        $forceAccessToken = $request->all()['null']['SaferpayShopware.config.forceAccessTokenGeneration'];

        $testMode = $this->paymentTransactionSaveService->getPluginConfig(
            'operationMode'
        ) !== 'live';
        $customerId = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'customerIdTest' : 'customerIdLive');
        $terminalId = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'terminalIdTest' : 'terminalIdLive');

        if (!$forceAccessToken) {
            $this->saveRelatedCustomFields($customerId, !$testMode);
            return  0;
        }

        $paymentApiClient = $this->createSaferpayPaymentApiClientForSalesChannel();

        $this->paymentMethodEntity = $this->paymentMethodRepository->search(
            new Criteria(array($this->paymentId)),
            $this->context
        )->first();

        $customFields = $this->paymentService->getPaymentMethodCustomFields($this->paymentMethodEntity, $this->context);

        // perform api call only if field is empty
        if (
            empty($customFields['saferpay_shopware_field_access_token'])
        ) {
            try {
                $dataArray = [
                    'SourceUrls'  => [
                        $this->getShopUrl(),
                    ],
                    'Description' => 'Saferpay field access token for Shopware',
                    'customerId' => $customerId,
                    'terminalId' => $terminalId
                ];
                $response = $paymentApiClient->generateFieldAccessToken($dataArray);
                if ($response ??
                    false ?
                    $response->AccessToken ?? false
                    : false
                ) {
                    $this->saveFieldAccessToken($response->AccessToken, $customFields);
                } else {
                    // Log the error message using Shopware's logging functionality
                    $this->logger->error('Access token retrieval failed: ' . ERRORMESSAGE);
                }
            } catch (\Throwable $th) {
                // Log the error message using Shopware's logging functionality
                $this->logger->error('Access token retrieval failed: ' . ERRORMESSAGE);
            }
        }
        $this->saveRelatedCustomFields($customerId, !$testMode);
    }

    /**
     * 1. Checks if the mode is test or live.
     * 2. Populates the customerId, terminalId, username, password accordingly
     * 3. Creates api client for sales channel using the values
     */
    protected function createSaferpayPaymentApiClientForSalesChannel(): SaferpayPaymentApiClient
    {
        $testMode = $this->paymentTransactionSaveService->getPluginConfig(
            'operationMode'
        ) !== 'live';
        $customerId = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'customerIdTest' : 'customerIdLive');
        $apiVersion = $this->paymentTransactionSaveService->getPluginConfig('saferpayAPIVersion');
        $terminalId = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'terminalIdTest' : 'terminalIdLive');
        $username = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'usernameTest' : 'usernameLive');
        $password = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'passwordTest' : 'passwordLive');
        $apiBaseUrl = $this->paymentTransactionSaveService
            ->getPluginConfig($testMode ? 'testApiBaseUrl' : 'liveApiBaseUrl');

        return new SaferpayPaymentApiClient(
            $customerId,
            $terminalId,
            $username,
            $password,
            'Storefront',
            $apiBaseUrl,
            (string) $apiVersion
        );
    }

    /**
     * function to save Field Access Token
     *
     * @param string $fieldAccessToken
     * @return void
     */
    private function saveFieldAccessToken($fieldAccessToken, $customFields)
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('paymentMethodId', $this->paymentId));

        $this->paymentMethodTranslationEntity = $this->paymentMethodTranslationRepository->search(
            $criteria,
            $this->context
        )->getEntities();

        foreach ($this->paymentMethodTranslationEntity as $paymentMethodEntity) {
            $customFields['saferpay_shopware_field_access_token'] = $fieldAccessToken;

            $updateData = [
                'paymentMethodId' => $paymentMethodEntity->getPaymentMethodId(),
                'languageId' => $paymentMethodEntity->getLanguageId(),
                'customFields' => $customFields
            ];

            $this->paymentMethodTranslationRepository->update([$updateData], $this->context);
        }
    }

    /**
     * function to save related custom fields
     *
     * @param int $customerId
     * @param bool $isLive
     * @return void
     */
    private function saveRelatedCustomFields($customerId, $isLive)
    {
        $this->paymentMethodEntity = $this->paymentMethodRepository->search(
            new Criteria(array($this->paymentId)),
            $this->context
        )->first();

        $customFields = $this->paymentService->getPaymentMethodCustomFields($this->paymentMethodEntity, $this->context);
        if (empty($customFields['saferpay_shopware_field_api_url'])) {
            $customFields['saferpay_shopware_field_api_url'] = $this->getFieldsApiUrl($customerId, $isLive);
        }
        if (empty($customFields['saferpay_shopware_field_js_library_url'])) {
            $customFields['saferpay_shopware_field_js_library_url'] = $this->getLibraryUrl($isLive);
        }

        $this->paymentMethodTranslationEntity = $this->paymentMethodTranslationRepository->search(
            new Criteria(array($this->paymentId)),
            $this->context
        )->getEntities();

        foreach ($this->paymentMethodTranslationEntity as $paymentMethodEntity) {
            $updateData = [
                'id' => $paymentMethodEntity->getId(),
                'customFields' => $customFields
            ];
            $this->paymentMethodTranslationRepository->update([$updateData], $this->context);
        }
    }

    /**
     * function to get Shop Url
     *
     * @return string
     */
    public function getShopUrl(): string
    {
        $request = $this->requestStack->getCurrentRequest();
        if ($request === null) {
            throw new FailedToGetRequestException('No current request found');
        }

        $schemeAndHttpHost = $request->getSchemeAndHttpHost();
        $basePath = $request->getBasePath();

        // Ensure the URL ends with a trailing slash
        return rtrim($schemeAndHttpHost . $basePath, '/') . '/';
    }

    /**
     * Function to get Field API Url
     *
     * @return string
     */
    public function getFieldsApiUrl($customerId, $isLive)
    {

        $fieldApiUrl = ($isLive)
            ? 'https://www.saferpay.com/Fields/' . $customerId
            : 'https://test.saferpay.com/Fields/' . $customerId;

        return (!empty($customerId)) ? $fieldApiUrl : null;
    }

    /**
     * function to get Js Library Url
     *
     * @param boolean $isLive
     * @return string
     */
    public function getLibraryUrl($isLive)
    {
        if ($isLive) {
            return 'https://www.saferpay.com/Fields/lib/1.6/saferpay-fields.js';
        } else {
            return 'https://test.saferpay.com/Fields/lib/1.6/saferpay-fields.js';
        }
    }
}
